/*
	Synogate Library, a support library for Synogate IP-core generators.
	Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
	
	This program is commercial software: you must not copy, modify or
	distribute it without explicit written license by Synogate to do so. 
	
	The full license text for non-commercial use and evaluation can be found in the distribution package of this core.
	
	This program is distributed WITHOUT ANY WARRANTY; without even the
	implied warranty of MERCHANTABILITY.
*/

#pragma once

#include "Section.h"
#include "Report.h"

#include <map>

namespace gtry::reporting {

class InterfaceDocumentation : public Report
{
    public:
        InterfaceDocumentation(std::filesystem::path destination) : Report(destination) { }
        Section &generateDefaultMainPage(const std::string ipCoreName);

        virtual void generate();
    protected:
        Section *m_tocSection = nullptr;
        Section *m_LOFSection[3] = {nullptr, nullptr, nullptr};

        virtual void buildSidebarPage(std::ostream &stream, const Page *page);

        virtual void buildTableOfContents(std::ostream &stream, const Page *page, unsigned maxDepth);
        virtual void buildSidebarTableOfContents(std::ostream &stream, const Page *page);
        virtual void buildListOfFloats(std::ostream &stream, const Page *page, std::function<bool(const Float*)> floatSelector);
        virtual void buildSidebarListOfFloats(std::ostream &stream, const Page *page, std::function<bool(const Float*)> floatSelector);

        virtual void generatePageHeader(std::ostream &stream, const Page *page) override;
        virtual void generatePageFooter(std::ostream &stream, const Page *page) override;
        virtual void generatePage(std::ostream &stream, const Page *page) override;
        virtual void generateSection(std::ostream &stream, const Section *section, unsigned depth) override;

        virtual void generateWaveformContent(std::ostream &stream, const Waveform *waveform) override;
};

}