/*
	Synogate Library, a support library for Synogate IP-core generators.
	Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
	
	This program is commercial software: you must not copy, modify or
	distribute it without explicit written license by Synogate to do so. 
	
	The full license text for non-commercial use and evaluation can be found in the distribution package of this core.
	
	This program is distributed WITHOUT ANY WARRANTY; without even the
	implied warranty of MERCHANTABILITY.
*/

#pragma once

#include <filesystem>

#include <set>
#include <list>
#include <map>
#include <vector>

namespace gtry::reporting {

class Page;
class Section;
class Float;
class Table;
class Waveform;
class Figure;

class Report
{
    public:
        Report(std::filesystem::path destination);
        virtual ~Report() = default;

        virtual void requireResource(std::string path);

        Page &createPage(std::string name, std::string label);

        virtual void generate();
    protected:
        std::filesystem::path m_destination;
        std::vector<std::string> m_requiredResources;

        struct Ref {
            std::string label;
            std::string href;
        };
        std::map<const void*, Ref> m_refs;
        std::map<std::string, Ref> m_refLabelToRef;

        std::list<Page> m_pages;

        virtual void collectRefs(const Page *page, const std::string &secPrefix);

        virtual void generatePageHeader(std::ostream &stream, const Page *page);
        virtual void generatePageFooter(std::ostream &stream, const Page *page);
        virtual void generatePage(std::ostream &stream, const Page *page);
        virtual void generateSection(std::ostream &stream, const Section *section, unsigned depth);

        virtual void generateText(std::ostream &stream, const std::string_view &text);
        virtual void generateFloat(std::ostream &stream, const Float *float_);

        virtual void generateTableHeader(std::ostream &stream, const Table *table);
        virtual void generateTableFooter(std::ostream &stream, const Table *table);
        virtual void generateTable(std::ostream &stream, const Table *table);

        virtual void generateWaveformContent(std::ostream &stream, const Waveform *waveform);
        virtual void generateWaveform(std::ostream &stream, const Waveform *waveform);

        virtual void generateFigure(std::ostream &stream, const Figure *figure);

        virtual void copyRequiredResources();
};

}