/*
	Synogate Library, a support library for Synogate IP-core generators.
	Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
	
	This program is commercial software: you must not copy, modify or
	distribute it without explicit written license by Synogate to do so. 
	
	The full license text for non-commercial use and evaluation can be found in the distribution package of this core.
	
	This program is distributed WITHOUT ANY WARRANTY; without even the
	implied warranty of MERCHANTABILITY.
*/

#include "Table.h"

#include <gatery/utils.h>

namespace gtry::reporting {

Table::Table(Report &report, std::string name, std::string label) : Float(report, std::move(name), std::move(label))
{
}

Table::Cell &Table::operator()(int row, int col)
{
    if (row < 0) {
        HCL_DESIGNCHECK(-row <= (int)m_numRows);
        row = (int)m_numRows + row;
    } else
        m_numRows = std::max<unsigned>(m_numRows, row+1);

    if (col < 0) {
        HCL_DESIGNCHECK(-col <= (int)m_numCols);
        col = (int)m_numCols + col;
    } else
        m_numCols = std::max<unsigned>(m_numCols, col+1);

    return m_cells[{row, col}];
}

const Table::Cell &Table::operator()(int row, int col) const
{
    if (row < 0) {
        HCL_DESIGNCHECK(-row <= (int)m_numRows);
        row = (int)m_numRows + row;
    }

    if (col < 0) {
        HCL_DESIGNCHECK(-col <= (int)m_numCols);
        col = (int)m_numCols + col;
    }

    auto it = m_cells.find({row, col});
    HCL_DESIGNCHECK(it != m_cells.end());
    return it->second;
}

bool Table::cellEmpty(unsigned row, unsigned col) const
{
    auto it = m_cells.find({row, col});
    return it == m_cells.end();
}

void Table::addHeader(unsigned numCols)
{
    addRow();
    for (auto i : utils::Range(numCols)) {
        (*this)(-1, i).header = true;
    }
}


}
