/*
	Synogate Library, a support library for Synogate IP-core generators.
	Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
	
	This program is commercial software: you must not copy, modify or
	distribute it without explicit written license by Synogate to do so. 
	
	The full license text for non-commercial use and evaluation can be found in the distribution package of this core.
	
	This program is distributed WITHOUT ANY WARRANTY; without even the
	implied warranty of MERCHANTABILITY.
*/

#pragma once

#include "Float.h"

#include <map>
#include <sstream>

namespace gtry::reporting {

class Table : public Float
{
    public:
        struct Cell {
            bool header = false;
            std::stringstream content;

            template<typename type>
            Cell &operator<<(const type &v) { content << v; return *this; }
        };

        Table(Report &report, std::string name, std::string label);

        Cell &operator()(int row, int col);
        const Cell &operator()(int row, int col) const;

        inline unsigned rows() const { return m_numRows; }
        inline unsigned cols() const { return m_numCols; }

        void addHeader(unsigned numCols);

        void addRow() { m_numRows++; }

        bool cellEmpty(unsigned row, unsigned col) const;


    protected:

        unsigned m_numRows = 0;
        unsigned m_numCols = 0;
        std::map<std::pair<unsigned, unsigned>, Cell> m_cells;
};


}