/*
	Synogate Library, a support library for Synogate IP-core generators.
	Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
	
	This program is commercial software: you must not copy, modify or
	distribute it without explicit written license by Synogate to do so. 
	
	The full license text for non-commercial use and evaluation can be found in the distribution package of this core.
	
	This program is distributed WITHOUT ANY WARRANTY; without even the
	implied warranty of MERCHANTABILITY.
*/

#pragma once

#include "Float.h"

#include <vector>
#include <string>

namespace gtry::sim {
struct MemoryTrace;
}

namespace gtry::reporting {

class Waveform : public Float
{
    public:
        struct Style {
            std::string cssPrefix = "wf_defined";
            bool angledTransitions = true;
            bool showLabel = true;
            bool verticalRulerRisingEdge = false;
            bool verticalRulerFallingEdge = false;
            unsigned base = 16;
        };

        static Style defaultInHex;
        static Style defaultInBin;
        static Style defaultHex;
        static Style defaultBin;
        static Style defaultClock;

        struct TraceSignal {
            std::string n;
            Style style = defaultHex;
        };

        struct TraceOptions {
            std::string selectionAnnotationId;
            bool collapse = false;
            bool startTimeZero = false;
        };


        Waveform(Report &report, std::string name, std::string label);

        void fromMemoryTrace(const sim::MemoryTrace &trace, std::vector<TraceSignal> signals, TraceOptions options);

        inline const sim::MemoryTrace *getTrace() const { return m_trace; }
        inline const std::vector<TraceSignal> &getTraceSignals() const { return m_signals; }
        inline const TraceOptions &getOptions() const { return m_options; }
    protected:
        const sim::MemoryTrace *m_trace = nullptr;
        std::vector<TraceSignal> m_signals;
        TraceOptions m_options;

};


}