/*
	Synogate Anti Replay Core, an IP core implementing an algorithm
	designed to detect packet duplication based on packet sequence numbers.
	Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
	
	This program is commercial software: you must not copy, modify or
	distribute it without explicit written license by Synogate to do so. 
	
	This program is distributed WITHOUT ANY WARRANTY; without even the
	implied warranty of MERCHANTABILITY.
*/

#pragma once

#include <stdint.h>
#include <map>
#include <set>
#include <queue>

namespace arc
{
	class AntiReplayCoreModel {
        public:
            struct Cmd {
                uint64_t opcode;
                uint64_t connectionIndex;
                uint64_t sequenceNumber;
                uint64_t windowSize;
            };

            struct Result {
                bool valid;
                uint64_t code;
            };

            AntiReplayCoreModel(uint64_t physicalSize, uint64_t virtualSize);
            AntiReplayCoreModel& windowSize(uint64_t physicalSize, uint64_t virtualSize);

            Result execute(const Cmd& cmd);
        protected:
            struct SaState {
                bool active = false;
                std::set<uint64_t> seen;
            };

            uint64_t m_physicalWindowSize = 64;
            uint64_t m_virtualWindowSize = 128;

            std::map<uint64_t, SaState> m_state;

            uint64_t update(SaState& sa, uint64_t sequenceNumber);
	};
}
