
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;
USE std.textio.all;

ENTITY TB_AntiReplayCore_fuzzing IS
END TB_AntiReplayCore_fuzzing;

ARCHITECTURE tb OF TB_AntiReplayCore_fuzzing IS

    SIGNAL clock : STD_LOGIC := '0';
    SIGNAL in_cmd_opcode : STD_LOGIC_VECTOR(2 downto 0);
    SIGNAL in_cmd_connectionIndex : STD_LOGIC_VECTOR(13 downto 0);
    SIGNAL in_cmd_sequenceNumber : STD_LOGIC_VECTOR(63 downto 0);
    SIGNAL in_cmd_valid : STD_LOGIC;
    SIGNAL in_cmd_windowSize : STD_LOGIC_VECTOR(23 downto 0);
    SIGNAL out_result_valid : STD_LOGIC;
    SIGNAL in_frame_endofpacket : STD_LOGIC;
    SIGNAL out_frame_empty : STD_LOGIC_VECTOR(5 downto 0);
    SIGNAL out_result_data : STD_LOGIC_VECTOR(2 downto 0);
    SIGNAL in_frame_userdata : STD_LOGIC_VECTOR(-1 downto 0);
    SIGNAL in_ready : STD_LOGIC;
    SIGNAL in_frame_startofpacket : STD_LOGIC;
    SIGNAL in_frame_channel : STD_LOGIC_VECTOR(0 downto 0);
    SIGNAL out_ready : STD_LOGIC;
    SIGNAL out_frame_valid : STD_LOGIC;
    SIGNAL out_frame_error : STD_LOGIC_VECTOR(0 downto 0);
    SIGNAL out_frame_channel : STD_LOGIC_VECTOR(0 downto 0);
    SIGNAL in_frame_empty : STD_LOGIC_VECTOR(5 downto 0);
    SIGNAL out_frame_startofpacket : STD_LOGIC;
    SIGNAL in_frame_data : STD_LOGIC_VECTOR(511 downto 0);
    SIGNAL in_frame_error : STD_LOGIC_VECTOR(0 downto 0);
    SIGNAL out_frame_endofpacket : STD_LOGIC;
    SIGNAL out_frame_userdata : STD_LOGIC_VECTOR(-1 downto 0);
    SIGNAL in_frame_valid : STD_LOGIC;
    SIGNAL out_frame_data : STD_LOGIC_VECTOR(511 downto 0);
    SIGNAL bitmap_wr_writedata : STD_LOGIC_VECTOR(255 downto 0);
    SIGNAL bitmap_rd_readdata : STD_LOGIC_VECTOR(255 downto 0);
    SIGNAL bitmap_wr_address : STD_LOGIC_VECTOR(19 downto 0);
    SIGNAL bitmap_rd_address : STD_LOGIC_VECTOR(19 downto 0);
    SIGNAL bitmap_rd_read : STD_LOGIC;
    SIGNAL bitmap_wr_write : STD_LOGIC;
    SIGNAL TB_testbench_is_done : STD_LOGIC := '0';

    function stringcompare(v_line : in string; str : in string) return boolean is
    begin
        if not (v_line'length = str'length) then
            return false;
        end if;
        for i in v_line'range loop
            if not (v_line(i) = str(i)) then
                return false;
            end if;
        end loop;
        return true;
     
    end function stringcompare;
    BEGIN
    inst_root : entity work.AntiReplayCore(impl) port map (
        clock => clock,
        in_cmd_opcode => in_cmd_opcode,
        in_cmd_connectionIndex => in_cmd_connectionIndex,
        in_cmd_sequenceNumber => in_cmd_sequenceNumber,
        in_cmd_valid => in_cmd_valid,
        in_cmd_windowSize => in_cmd_windowSize,
        out_result_valid => out_result_valid,
        in_frame_endofpacket => in_frame_endofpacket,
        out_frame_empty => out_frame_empty,
        out_result_data => out_result_data,
        in_frame_userdata => in_frame_userdata,
        in_ready => in_ready,
        in_frame_startofpacket => in_frame_startofpacket,
        in_frame_channel => in_frame_channel,
        out_ready => out_ready,
        out_frame_valid => out_frame_valid,
        out_frame_error => out_frame_error,
        out_frame_channel => out_frame_channel,
        in_frame_empty => in_frame_empty,
        out_frame_startofpacket => out_frame_startofpacket,
        in_frame_data => in_frame_data,
        in_frame_error => in_frame_error,
        out_frame_endofpacket => out_frame_endofpacket,
        out_frame_userdata => out_frame_userdata,
        in_frame_valid => in_frame_valid,
        out_frame_data => out_frame_data,
        bitmap_wr_writedata => bitmap_wr_writedata,
        bitmap_rd_readdata => bitmap_rd_readdata,
        bitmap_wr_address => bitmap_wr_address,
        bitmap_rd_address => bitmap_rd_address,
        bitmap_rd_read => bitmap_rd_read,
        bitmap_wr_write => bitmap_wr_write
    );
    sim_process : PROCESS
    VARIABLE v_in_cmd_opcode : STD_LOGIC_VECTOR(2 downto 0);
    VARIABLE v_in_cmd_connectionIndex : STD_LOGIC_VECTOR(13 downto 0);
    VARIABLE v_in_cmd_sequenceNumber : STD_LOGIC_VECTOR(63 downto 0);
    VARIABLE v_in_cmd_valid : STD_LOGIC;
    VARIABLE v_in_cmd_windowSize : STD_LOGIC_VECTOR(23 downto 0);
    VARIABLE v_out_result_valid : STD_LOGIC;
    VARIABLE v_in_frame_endofpacket : STD_LOGIC;
    VARIABLE v_out_frame_empty : STD_LOGIC_VECTOR(5 downto 0);
    VARIABLE v_out_result_data : STD_LOGIC_VECTOR(2 downto 0);
    VARIABLE v_in_frame_userdata : STD_LOGIC_VECTOR(-1 downto 0);
    VARIABLE v_in_ready : STD_LOGIC;
    VARIABLE v_in_frame_startofpacket : STD_LOGIC;
    VARIABLE v_in_frame_channel : STD_LOGIC_VECTOR(0 downto 0);
    VARIABLE v_out_ready : STD_LOGIC;
    VARIABLE v_out_frame_valid : STD_LOGIC;
    VARIABLE v_out_frame_error : STD_LOGIC_VECTOR(0 downto 0);
    VARIABLE v_out_frame_channel : STD_LOGIC_VECTOR(0 downto 0);
    VARIABLE v_in_frame_empty : STD_LOGIC_VECTOR(5 downto 0);
    VARIABLE v_out_frame_startofpacket : STD_LOGIC;
    VARIABLE v_in_frame_data : STD_LOGIC_VECTOR(511 downto 0);
    VARIABLE v_in_frame_error : STD_LOGIC_VECTOR(0 downto 0);
    VARIABLE v_out_frame_endofpacket : STD_LOGIC;
    VARIABLE v_out_frame_userdata : STD_LOGIC_VECTOR(-1 downto 0);
    VARIABLE v_in_frame_valid : STD_LOGIC;
    VARIABLE v_out_frame_data : STD_LOGIC_VECTOR(511 downto 0);
    VARIABLE v_bitmap_wr_writedata : STD_LOGIC_VECTOR(255 downto 0);
    VARIABLE v_bitmap_rd_readdata : STD_LOGIC_VECTOR(255 downto 0);
    VARIABLE v_bitmap_wr_address : STD_LOGIC_VECTOR(19 downto 0);
    VARIABLE v_bitmap_rd_address : STD_LOGIC_VECTOR(19 downto 0);
    VARIABLE v_bitmap_rd_read : STD_LOGIC;
    VARIABLE v_bitmap_wr_write : STD_LOGIC;
        VARIABLE v_line : line;
        VARIABLE time_in_ps : integer;
        VARIABLE v_clk : std_logic;
        FILE test_vector_file : text;
    BEGIN
        file_open(test_vector_file, "TB_AntiReplayCore_fuzzing.testvectors", read_mode);
        IF endfile(test_vector_file) THEN
            REPORT "The test vector file is empty!";
            ASSERT FALSE;
        END IF;
        WHILE NOT endfile(test_vector_file) loop
            readline(test_vector_file, v_line);
            IF stringcompare(v_line(1 to v_line'length), "SET") THEN
                readline(test_vector_file, v_line);
                IF false THEN
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_opcode") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_cmd_opcode);
                    in_cmd_opcode <= v_in_cmd_opcode;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_connectionIndex") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_cmd_connectionIndex);
                    in_cmd_connectionIndex <= v_in_cmd_connectionIndex;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_sequenceNumber") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_cmd_sequenceNumber);
                    in_cmd_sequenceNumber <= v_in_cmd_sequenceNumber;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_valid") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_cmd_valid);
                    in_cmd_valid <= v_in_cmd_valid;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_windowSize") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_cmd_windowSize);
                    in_cmd_windowSize <= v_in_cmd_windowSize;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_endofpacket") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_frame_endofpacket);
                    in_frame_endofpacket <= v_in_frame_endofpacket;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_userdata") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_userdata);
                    in_frame_userdata <= v_in_frame_userdata;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_startofpacket") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_frame_startofpacket);
                    in_frame_startofpacket <= v_in_frame_startofpacket;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_channel") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_channel);
                    in_frame_channel <= v_in_frame_channel;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_ready") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_out_ready);
                    out_ready <= v_out_ready;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_empty") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_empty);
                    in_frame_empty <= v_in_frame_empty;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_data") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_data);
                    in_frame_data <= v_in_frame_data;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_error") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_error);
                    in_frame_error <= v_in_frame_error;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_valid") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_frame_valid);
                    in_frame_valid <= v_in_frame_valid;
                ELSIF stringcompare(v_line(1 to v_line'length), "bitmap_rd_readdata") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_bitmap_rd_readdata);
                    bitmap_rd_readdata <= v_bitmap_rd_readdata;
                ELSE
                    REPORT "An error occured while parsing the test vector file: unknown signal:" & v_line(1 to v_line'length);
                    ASSERT FALSE severity failure;
                END IF;
            ELSIF stringcompare(v_line(1 to v_line'length), "CHECK") THEN
                readline(test_vector_file, v_line);
                IF false THEN
                ELSIF stringcompare(v_line(1 to v_line'length), "out_frame_userdata") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_out_frame_userdata);
                    ASSERT std_match(out_frame_userdata, v_out_frame_userdata) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_opcode") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_cmd_opcode);
                    ASSERT std_match(in_cmd_opcode, v_in_cmd_opcode) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_connectionIndex") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_cmd_connectionIndex);
                    ASSERT std_match(in_cmd_connectionIndex, v_in_cmd_connectionIndex) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_sequenceNumber") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_cmd_sequenceNumber);
                    ASSERT std_match(in_cmd_sequenceNumber, v_in_cmd_sequenceNumber) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_valid") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_cmd_valid);
                    ASSERT std_match(in_cmd_valid, v_in_cmd_valid) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_cmd_windowSize") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_cmd_windowSize);
                    ASSERT std_match(in_cmd_windowSize, v_in_cmd_windowSize) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "bitmap_rd_address") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_bitmap_rd_address);
                    ASSERT std_match(bitmap_rd_address, v_bitmap_rd_address) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "bitmap_wr_writedata") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_bitmap_wr_writedata);
                    ASSERT std_match(bitmap_wr_writedata, v_bitmap_wr_writedata) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "bitmap_rd_read") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_bitmap_rd_read);
                    ASSERT std_match(bitmap_rd_read, v_bitmap_rd_read) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "bitmap_wr_write") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_bitmap_wr_write);
                    ASSERT std_match(bitmap_wr_write, v_bitmap_wr_write) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_endofpacket") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_frame_endofpacket);
                    ASSERT std_match(in_frame_endofpacket, v_in_frame_endofpacket) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_userdata") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_userdata);
                    ASSERT std_match(in_frame_userdata, v_in_frame_userdata) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_startofpacket") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_frame_startofpacket);
                    ASSERT std_match(in_frame_startofpacket, v_in_frame_startofpacket) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_channel") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_channel);
                    ASSERT std_match(in_frame_channel, v_in_frame_channel) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_ready") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_out_ready);
                    ASSERT std_match(out_ready, v_out_ready) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_empty") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_empty);
                    ASSERT std_match(in_frame_empty, v_in_frame_empty) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_data") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_data);
                    ASSERT std_match(in_frame_data, v_in_frame_data) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_error") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_in_frame_error);
                    ASSERT std_match(in_frame_error, v_in_frame_error) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_frame_valid") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_frame_valid);
                    ASSERT std_match(in_frame_valid, v_in_frame_valid) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "bitmap_wr_address") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_bitmap_wr_address);
                    ASSERT std_match(bitmap_wr_address, v_bitmap_wr_address) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_result_data") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_out_result_data);
                    ASSERT std_match(out_result_data, v_out_result_data) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "in_ready") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_in_ready);
                    ASSERT std_match(in_ready, v_in_ready) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_frame_valid") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_out_frame_valid);
                    ASSERT std_match(out_frame_valid, v_out_frame_valid) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_frame_startofpacket") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_out_frame_startofpacket);
                    ASSERT std_match(out_frame_startofpacket, v_out_frame_startofpacket) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_frame_data") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_out_frame_data);
                    ASSERT std_match(out_frame_data, v_out_frame_data) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_frame_channel") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_out_frame_channel);
                    ASSERT std_match(out_frame_channel, v_out_frame_channel) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_frame_error") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_out_frame_error);
                    ASSERT std_match(out_frame_error, v_out_frame_error) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_frame_endofpacket") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_out_frame_endofpacket);
                    ASSERT std_match(out_frame_endofpacket, v_out_frame_endofpacket) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_result_valid") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_out_result_valid);
                    ASSERT std_match(out_result_valid, v_out_result_valid) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "out_frame_empty") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_out_frame_empty);
                    ASSERT std_match(out_frame_empty, v_out_frame_empty) severity failure;
                ELSIF stringcompare(v_line(1 to v_line'length), "bitmap_rd_readdata") THEN
                    readline(test_vector_file, v_line);
                    bread(v_line, v_bitmap_rd_readdata);
                    ASSERT std_match(bitmap_rd_readdata, v_bitmap_rd_readdata) severity failure;
                ELSE
                    REPORT "An error occured while parsing the test vector file: unknown signal:" & v_line(1 to v_line'length);
                    ASSERT FALSE severity failure;
                END IF;
            ELSIF stringcompare(v_line(1 to v_line'length), "CLK") THEN
                readline(test_vector_file, v_line);
                IF false THEN
                ELSIF stringcompare(v_line(1 to v_line'length), "clock") THEN
                    readline(test_vector_file, v_line);
                    read(v_line, v_clk);
                    clock <= v_clk;
                ELSE
                    REPORT "An error occured while parsing the test vector file: unknown clock:" & v_line(1 to v_line'length);
                    ASSERT FALSE severity failure;
                END IF;
            ELSIF stringcompare(v_line(1 to v_line'length), "RST") THEN
                readline(test_vector_file, v_line);
                IF false THEN
                ELSE
                    REPORT "An error occured while parsing the test vector file: unknown clock:" & v_line(1 to v_line'length);
                    ASSERT FALSE severity failure;
                END IF;
            ELSIF stringcompare(v_line(1 to v_line'length), "ADV") THEN
                readline(test_vector_file, v_line);
                read(v_line, time_in_ps);
                wait for time_in_ps * 1 ps;
            ELSE
                REPORT "An error occured while parsing the test vector file: Can't parse line:" & v_line(1 to v_line'length);
                ASSERT FALSE severity failure;
            END IF;
        end loop;
TB_testbench_is_done <= '1';
WAIT;
END PROCESS;
END;
