
--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

PACKAGE GateryHelperPackage IS
    FUNCTION bool2stdlogic(v : BOOLEAN) RETURN STD_LOGIC;
    FUNCTION stdlogic2bool(v : STD_LOGIC) RETURN BOOLEAN;
END PACKAGE GateryHelperPackage;

PACKAGE BODY GateryHelperPackage IS
    FUNCTION bool2stdlogic(v : BOOLEAN) RETURN STD_LOGIC IS
    BEGIN
        IF v THEN
            RETURN '1';
        ELSE
            RETURN '0';
        END IF;
    END bool2stdlogic;

    FUNCTION stdlogic2bool(v : STD_LOGIC) RETURN BOOLEAN IS
    BEGIN
        RETURN v = '1';
    END stdlogic2bool;

END PACKAGE BODY GateryHelperPackage;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

PACKAGE interface_package IS
    -- The number of pipeline stages of the anti replay core, which is also the minimum latency (can be more depending on external memory type)
    constant MINIMUM_LATENCY : natural := 16;
    -- Width (number of bits) of the data bus to the "main storage"
    constant SECOND_WORD_WIDTH : natural := 256;
    -- Number of bits of the connection indices (the maximal number of connections is a power of 2)
    constant CONNECTION_ADDR_WIDTH : natural := 14;
    -- Number of bits for representing the virtual window size
    constant VIRTUAL_WINDOW_WIDTH : natural := 24;
    -- Number of bits of the sequence numbers
    constant SEQUENCE_NUMBER_WIDTH : natural := 64;
    -- The total physical window size
    constant PHYSICAL_WINDOW_SIZE : natural := 16128;
    -- Number of data bits of the packet stream
    constant PACKET_DATA_WIDTH : natural := 512;
    -- Number of bits in the 'empty' signal of the packet stream
    constant PACKET_EMPTY_WIDTH : natural := 6;
    -- Number of bits of the opcode field
    constant OPCODE_WIDTH : natural := 3;
    -- Number of bits of the result field
    constant RESULT_WIDTH : natural := 3;
    -- Initiates an actual replay check for the given sequence number and connection index, triggering a reply on the result stream. Also updates the internal state as having received that packet.
    constant OpUpdate : std_logic_vector(2 downto 0) := "000";
    -- Activates replay checks for a given connection index. OpUpdate querries on that connection index will no longer return PassCheckDisabled.
    constant OpActivate : std_logic_vector(2 downto 0) := "101";
    -- Deactivates replay checks for a given connection index. OpUpdate querries on that connection index will return PassCheckDisabled.
    constant OpDeactivate : std_logic_vector(2 downto 0) := "001";
    -- For a given connection index, clears and initializes the list of all seen packets to the given sequence number, so that all other sequence numbers are deemed unseen. Connections are disabled and uninitialized on reset. Use OpSetSeqNr and a sequence number of 0 to initialize a connection.
    constant OpSetSeqNr : std_logic_vector(2 downto 0) := "010";
    -- Sets the virtual window size for all connections. This can also be larger than the actual physical size. The reset value equals the physical window size to effectively disable the virtual window feature.
    constant OpSetWindowSize : std_logic_vector(2 downto 0) := "011";
    -- The packet is deemed new because its sequence number was larger than the largest so far seen sequence number for that connection.
    constant PassAboveWindow : std_logic_vector(2 downto 0) := "001";
    -- The packet is deemed new because actually checking was disabled for that connection.
    constant PassCheckDisabled : std_logic_vector(2 downto 0) := "011";
    -- The packet is deemed new by definition because its sequence number is older than the physical window size but still within the virtual window size. This never occurs if the virtual window size is set to be equal or smaller than the physical one.
    constant PassInVirtualWindow : std_logic_vector(2 downto 0) := "101";
    -- The packet is deemed new because it falls within the virtual and physical windows and had not been marked as having been seen before.
    constant PassInPhysicalWindow : std_logic_vector(2 downto 0) := "111";
    -- The packet is deemed a replay because it is older than the virtual window size.
    constant FailBelowWindow : std_logic_vector(2 downto 0) := "000";
    -- The packer is deemed a replay because it fell within the virtual and physical windows and had been marked as having been seen before.
    constant FailReplay : std_logic_vector(2 downto 0) := "010";
END PACKAGE interface_package;

PACKAGE BODY interface_package IS
END PACKAGE BODY interface_package;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: merge
-- 
------------------------------------------------

ENTITY merge IS 
    PORT(
        in_out_ready : IN STD_LOGIC;
        in_empty_not_3 : IN STD_LOGIC;
        in_out_data_Bit : IN STD_LOGIC;
        in_empty_not_4 : IN STD_LOGIC;
        out_out_ready_mux1_2 : OUT STD_LOGIC;
        out_out_ready_mux1 : OUT STD_LOGIC;
        out_empty_not_mux1 : OUT STD_LOGIC;
        out_empty_not_mux1_2 : OUT STD_LOGIC;
        in_empty_not : IN STD_LOGIC;
        in_empty_not_2 : IN STD_LOGIC
    );
END merge;

ARCHITECTURE impl OF merge IS 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_empty_not_mux1 : STD_LOGIC; 
        VARIABLE v_empty_not_mux1_2 : STD_LOGIC; 
    BEGIN
        IF (( not in_out_data_Bit = '1') and in_empty_not = '1') THEN
            out_out_ready_mux1_2 <= '0';
        ELSE
            out_out_ready_mux1_2 <= in_out_ready;
        END IF;
        IF (( not in_out_data_Bit = '1') and in_empty_not = '1') THEN
            v_empty_not_mux1 := '0';
        ELSE
            v_empty_not_mux1 := in_empty_not_4;
        END IF;
        IF ((in_out_data_Bit = '1' and ( not in_empty_not_2 = '1')) and in_empty_not = '1') THEN
            out_out_ready_mux1 <= '0';
        ELSE
            out_out_ready_mux1 <= in_out_ready;
        END IF;
        IF ((in_out_data_Bit = '1' and ( not in_empty_not_2 = '1')) and in_empty_not = '1') THEN
            v_empty_not_mux1_2 := '0';
        ELSE
            v_empty_not_mux1_2 := in_empty_not_3;
        END IF;
        out_empty_not_mux1 <= v_empty_not_mux1_2;
        out_empty_not_mux1_2 <= v_empty_not_mux1;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_fifo_memory_Memory_Helper_2
-- Auto generated to handle various memory access issues such as read during write and read modify write hazards.
------------------------------------------------

ENTITY scl_fifo_memory_Memory_Helper_2 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_in_valid : IN STD_LOGIC;
        out_conflict_bypass_mux : OUT UNSIGNED(2 downto 0);
        in_unnamed_2 : IN UNSIGNED(2 downto 0);
        in_unnamed : IN UNSIGNED(5 downto 0);
        in_put_rewired : IN UNSIGNED(5 downto 0);
        in_in_data_packed_delayed1 : IN UNSIGNED(2 downto 0)
    );
END scl_fifo_memory_Memory_Helper_2;

ARCHITECTURE impl OF scl_fifo_memory_Memory_Helper_2 IS 
    SIGNAL s_conflict_and_wrEn : STD_LOGIC; 
    SIGNAL s_conflict_and_wrEn_delayed1 : STD_LOGIC := '0'; 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_conflict_bypass_mux : UNSIGNED(2 downto 0); 
        VARIABLE v_conflict : STD_LOGIC; 
    BEGIN

        -- Compare read and write addr for conflicts
        v_conflict := bool2stdlogic(in_unnamed = in_put_rewired);
        s_conflict_and_wrEn <= (v_conflict and in_in_valid);

        -- If read and write addr match and read and write are enabled, forward write data to read output.
        IF s_conflict_and_wrEn_delayed1 = '1' THEN
            v_conflict_bypass_mux := in_in_data_packed_delayed1;
        ELSE
            v_conflict_bypass_mux := in_unnamed_2;
        END IF;

        -- If read and write addr match and read and write are enabled, forward write data to read output.
        out_conflict_bypass_mux <= v_conflict_bypass_mux;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_conflict_and_wrEn_delayed1 <= s_conflict_and_wrEn;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY altera_mf;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_memory_7
-- 
------------------------------------------------

ENTITY scl_memory_7 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_in_data_packed : IN UNSIGNED(2 downto 0);
        in_in_valid : IN STD_LOGIC;
        out_conflict_bypass_mux : OUT UNSIGNED(2 downto 0);
        in_unnamed : IN UNSIGNED(5 downto 0);
        in_put_rewired : IN UNSIGNED(5 downto 0);
        in_in_data_packed_delayed1 : IN UNSIGNED(2 downto 0)
    );
END scl_memory_7;

ARCHITECTURE impl OF scl_memory_7 IS 
    SIGNAL s_altdpram_q : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed : UNSIGNED(2 downto 0); 
BEGIN
    altdpram : altera_mf.altera_mf_components.altdpram
     generic map (
        INDATA_ACLR => "OFF",
        INDATA_REG => "INCLOCK",
        OUTDATA_ACLR => "OFF",
        OUTDATA_REG => "OUTCLOCK",
        RDADDRESS_ACLR => "OFF",
        RDADDRESS_REG => "UNREGISTERED",
        RDCONTROL_ACLR => "OFF",
        RDCONTROL_REG => "UNREGISTERED",
        WIDTHAD => 6,
        WRADDRESS_ACLR => "OFF",
        WRADDRESS_REG => "INCLOCK",
        WRCONTROL_ACLR => "OFF",
        WRCONTROL_REG => "INCLOCK",
        intended_device_family => "Arria 10",
        numwords => 64,
        ram_block_type => "MLAB",
        read_during_write_mode_mixed_ports => "NEW_DATA",
        width => 3
    )
     port map (
        inclock => clock,
        outclock => clock,
        wren => in_in_valid,
        data => STD_LOGIC_VECTOR(in_in_data_packed),
        rdaddress => STD_LOGIC_VECTOR(in_unnamed),
        wraddress => STD_LOGIC_VECTOR(in_put_rewired),
        UNSIGNED(q) => s_altdpram_q
    );
    scl_fifo_memory_Memory_Helper0 : entity work.scl_fifo_memory_Memory_Helper_2(impl) port map (
        clock => clock,
        in_put_rewired => in_put_rewired,
        in_unnamed => in_unnamed,
        in_unnamed_2 => s_unnamed,
        in_in_data_packed_delayed1 => in_in_data_packed_delayed1,
        in_in_valid => in_in_valid,
        out_conflict_bypass_mux => out_conflict_bypass_mux
    );
    default_comb : PROCESS(all)
    BEGIN
        s_unnamed <= s_altdpram_q;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_fifo_2
-- 
------------------------------------------------

ENTITY scl_fifo_2 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_out_code : IN UNSIGNED(2 downto 0);
        out_full : OUT STD_LOGIC;
        out_empty : OUT STD_LOGIC;
        out_almost_full_0 : OUT STD_LOGIC;
        in_in_valid : IN STD_LOGIC;
        in_out_ready : IN STD_LOGIC;
        out_out_data : OUT UNSIGNED(2 downto 0)
    );
END scl_fifo_2;

ARCHITECTURE impl OF scl_fifo_2 IS 
    SIGNAL s_put_plus_const_1 : UNSIGNED(6 downto 0); 
    SIGNAL s_put_rewired : UNSIGNED(5 downto 0); 
    SIGNAL s_m_size_geq_almost_full_level_0 : STD_LOGIC; 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_unnamed_3 : UNSIGNED(5 downto 0); 
    SIGNAL s_get_plus_const_1 : UNSIGNED(6 downto 0); 
    SIGNAL s_in_data_packed_delayed1 : UNSIGNED(2 downto 0); 
    SIGNAL s_conflict_bypass_mux : UNSIGNED(2 downto 0); 
    SIGNAL s_in_data_packed_delayed1_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_full : STD_LOGIC := '0'; 
    SIGNAL s_empty : STD_LOGIC := '1'; 
    SIGNAL s_almost_full_0 : STD_LOGIC := '0'; 
    SIGNAL s_out_ready : STD_LOGIC; 
    SIGNAL s_get : UNSIGNED(6 downto 0) := "0000000"; 
    SIGNAL s_put : UNSIGNED(6 downto 0) := "0000000"; 
    SIGNAL s_in_data_packed : UNSIGNED(2 downto 0); 
    SIGNAL s_in_valid : STD_LOGIC; 
BEGIN
    scl_memory0 : entity work.scl_memory_7(impl) port map (
        clock => clock,
        in_put_rewired => s_put_rewired,
        in_unnamed => s_unnamed_3,
        in_in_data_packed_delayed1 => s_in_data_packed_delayed1_2,
        in_in_data_packed => s_in_data_packed,
        in_in_valid => s_in_valid,
        out_conflict_bypass_mux => s_conflict_bypass_mux
    );
    default_comb : PROCESS(all)
        VARIABLE v_get_mux1 : UNSIGNED(6 downto 0); 
        VARIABLE v_eq : STD_LOGIC; 
        VARIABLE v_in_data : UNSIGNED(2 downto 0); 
        VARIABLE v_m_size : UNSIGNED(6 downto 0); 
        VARIABLE v_almost_full_level_0 : UNSIGNED(6 downto 0); 
        VARIABLE v_get : UNSIGNED(6 downto 0); 
        VARIABLE v_m_put : UNSIGNED(6 downto 0); 
        VARIABLE v_m_get : UNSIGNED(6 downto 0); 
        VARIABLE v_m_get_2 : UNSIGNED(6 downto 0); 
        VARIABLE v_out_data_packed : UNSIGNED(2 downto 0); 
        VARIABLE v_m_put_2 : UNSIGNED(6 downto 0); 
        VARIABLE v_put : UNSIGNED(6 downto 0); 
    BEGIN
        out_full <= s_full;
        out_empty <= s_empty;
        v_almost_full_level_0 := "0110000";
        out_almost_full_0 <= s_almost_full_0;
        v_in_data := in_out_code;
        s_in_data_packed <= v_in_data;
        s_in_valid <= in_in_valid;
        v_put := s_put;
        s_out_ready <= in_out_ready;
        v_get := s_get;
        v_out_data_packed := s_conflict_bypass_mux;
        out_out_data <= v_out_data_packed;
        s_get_plus_const_1 <= (v_get + "0000001");
        IF s_out_ready = '1' THEN
            v_m_get := s_get_plus_const_1;
        ELSE
            v_m_get := v_get;
        END IF;
        v_m_get_2 := v_m_get;
        v_get_mux1 := v_m_get;
        s_unnamed_3 <= v_get_mux1(5 downto 0);
        s_put_plus_const_1 <= (v_put + "0000001");
        IF s_in_valid = '1' THEN
            v_m_put := s_put_plus_const_1;
        ELSE
            v_m_put := v_put;
        END IF;
        v_m_put_2 := v_m_put;
        v_m_size := (v_m_put_2 - v_m_get_2);
        v_eq := bool2stdlogic(v_m_put_2(5 downto 0) = v_m_get_2(5 downto 0));
        s_put_rewired <= v_put(5 downto 0);
        s_m_size_geq_almost_full_level_0 <= bool2stdlogic(v_m_size >= v_almost_full_level_0);
        s_unnamed <= (v_eq and bool2stdlogic(v_m_put_2(6) = v_m_get_2(6)));
        s_unnamed_2 <= (v_eq and bool2stdlogic(v_m_put_2(6) /= v_m_get_2(6)));
        s_in_data_packed_delayed1_2 <= s_in_data_packed_delayed1;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_in_data_packed_delayed1 <= s_in_data_packed;
            s_full <= s_unnamed_2;
            s_empty <= s_unnamed;
            IF (s_in_valid = '1') THEN
                s_put <= s_put_plus_const_1;
            END IF;
            s_almost_full_0 <= s_m_size_geq_almost_full_level_0;
            IF (s_out_ready = '1') THEN
                s_get <= s_get_plus_const_1;
            END IF;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_fifo_memory_Memory_Helper
-- Auto generated to handle various memory access issues such as read during write and read modify write hazards.
------------------------------------------------

ENTITY scl_fifo_memory_Memory_Helper IS 
    PORT(
        clock : IN STD_LOGIC;
        in_in_valid : IN STD_LOGIC;
        out_conflict_bypass_mux : OUT UNSIGNED(523 downto 0);
        in_unnamed_2 : IN UNSIGNED(523 downto 0);
        in_unnamed : IN UNSIGNED(5 downto 0);
        in_put_rewired : IN UNSIGNED(5 downto 0);
        in_in_data_packed_delayed1 : IN UNSIGNED(523 downto 0)
    );
END scl_fifo_memory_Memory_Helper;

ARCHITECTURE impl OF scl_fifo_memory_Memory_Helper IS 
    SIGNAL s_conflict_and_wrEn_delayed1 : STD_LOGIC; 
    SIGNAL s_conflict_and_wrEn : STD_LOGIC; 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_conflict_bypass_mux : UNSIGNED(523 downto 0); 
        VARIABLE v_conflict : STD_LOGIC; 
    BEGIN

        -- Compare read and write addr for conflicts
        v_conflict := bool2stdlogic(in_unnamed = in_put_rewired);
        s_conflict_and_wrEn <= (v_conflict and in_in_valid);

        -- If read and write addr match and read and write are enabled, forward write data to read output.
        IF s_conflict_and_wrEn_delayed1 = '1' THEN
            v_conflict_bypass_mux := in_in_data_packed_delayed1;
        ELSE
            v_conflict_bypass_mux := in_unnamed_2;
        END IF;

        -- If read and write addr match and read and write are enabled, forward write data to read output.
        out_conflict_bypass_mux <= v_conflict_bypass_mux;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_conflict_and_wrEn_delayed1 <= s_conflict_and_wrEn;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY altera_mf;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_memory_6
-- 
------------------------------------------------

ENTITY scl_memory_6 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_in_data_packed : IN UNSIGNED(523 downto 0);
        in_in_valid : IN STD_LOGIC;
        out_conflict_bypass_mux : OUT UNSIGNED(523 downto 0);
        in_unnamed : IN UNSIGNED(5 downto 0);
        in_put_rewired : IN UNSIGNED(5 downto 0);
        in_in_data_packed_delayed1 : IN UNSIGNED(523 downto 0)
    );
END scl_memory_6;

ARCHITECTURE impl OF scl_memory_6 IS 
    SIGNAL s_altdpram_q : UNSIGNED(523 downto 0); 
    SIGNAL s_unnamed : UNSIGNED(523 downto 0); 
BEGIN
    altdpram : altera_mf.altera_mf_components.altdpram
     generic map (
        INDATA_ACLR => "OFF",
        INDATA_REG => "INCLOCK",
        OUTDATA_ACLR => "OFF",
        OUTDATA_REG => "OUTCLOCK",
        RDADDRESS_ACLR => "OFF",
        RDADDRESS_REG => "UNREGISTERED",
        RDCONTROL_ACLR => "OFF",
        RDCONTROL_REG => "UNREGISTERED",
        WIDTHAD => 6,
        WRADDRESS_ACLR => "OFF",
        WRADDRESS_REG => "INCLOCK",
        WRCONTROL_ACLR => "OFF",
        WRCONTROL_REG => "INCLOCK",
        intended_device_family => "Arria 10",
        numwords => 64,
        ram_block_type => "MLAB",
        read_during_write_mode_mixed_ports => "NEW_DATA",
        width => 524
    )
     port map (
        inclock => clock,
        outclock => clock,
        wren => in_in_valid,
        data => STD_LOGIC_VECTOR(in_in_data_packed),
        rdaddress => STD_LOGIC_VECTOR(in_unnamed),
        wraddress => STD_LOGIC_VECTOR(in_put_rewired),
        UNSIGNED(q) => s_altdpram_q
    );
    scl_fifo_memory_Memory_Helper0 : entity work.scl_fifo_memory_Memory_Helper(impl) port map (
        clock => clock,
        in_unnamed => in_unnamed,
        in_put_rewired => in_put_rewired,
        in_unnamed_2 => s_unnamed,
        in_in_data_packed_delayed1 => in_in_data_packed_delayed1,
        in_in_valid => in_in_valid,
        out_conflict_bypass_mux => out_conflict_bypass_mux
    );
    default_comb : PROCESS(all)
    BEGIN
        s_unnamed <= s_altdpram_q;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_fifo
-- 
------------------------------------------------

ENTITY scl_fifo IS 
    PORT(
        clock : IN STD_LOGIC;
        in_in_frame_valid : IN STD_LOGIC;
        in_in_frame_startofpacket : IN STD_LOGIC;
        in_in_frame_endofpacket : IN STD_LOGIC;
        in_in_frame_channel : IN UNSIGNED(0 downto 0);
        in_in_frame_data : IN UNSIGNED(511 downto 0);
        in_in_frame_error : IN UNSIGNED(0 downto 0);
        in_in_frame_empty : IN UNSIGNED(5 downto 0);
        in_dbg_dataIn4 : IN STD_LOGIC;
        out_full : OUT STD_LOGIC;
        out_empty : OUT STD_LOGIC;
        in_in_valid : IN STD_LOGIC;
        in_out_ready : IN STD_LOGIC;
        out_out_data_FrameStream_valid : OUT STD_LOGIC;
        out_out_data_FrameStream_channel : OUT UNSIGNED(0 downto 0);
        out_out_data_FrameStream_data : OUT UNSIGNED(511 downto 0);
        out_out_data_FrameStream_error : OUT UNSIGNED(0 downto 0);
        out_out_data_FrameStream_empty : OUT UNSIGNED(5 downto 0);
        out_out_data_FrameStream_startofpacket : OUT STD_LOGIC;
        out_out_data_FrameStream_endofpacket : OUT STD_LOGIC;
        out_out_data_Bit : OUT STD_LOGIC
    );
END scl_fifo;

ARCHITECTURE impl OF scl_fifo IS 
    SIGNAL s_unnamed : UNSIGNED(5 downto 0); 
    SIGNAL s_get_plus_const_1 : UNSIGNED(6 downto 0); 
    SIGNAL s_put_plus_const_1 : UNSIGNED(6 downto 0); 
    SIGNAL s_put_rewired : UNSIGNED(5 downto 0); 
    SIGNAL s_conflict_bypass_mux : UNSIGNED(523 downto 0); 
    SIGNAL s_in_data_packed_delayed1 : UNSIGNED(523 downto 0); 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_unnamed_3 : STD_LOGIC; 
    SIGNAL s_in_data_packed_delayed1_2 : UNSIGNED(523 downto 0); 
    SIGNAL s_full : STD_LOGIC := '0'; 
    SIGNAL s_in_data_packed : UNSIGNED(523 downto 0); 
    SIGNAL s_in_valid : STD_LOGIC; 
    SIGNAL s_empty : STD_LOGIC := '1'; 
    SIGNAL s_put : UNSIGNED(6 downto 0) := "0000000"; 
    SIGNAL s_get : UNSIGNED(6 downto 0) := "0000000"; 
    SIGNAL s_out_ready : STD_LOGIC; 
BEGIN
    scl_memory0 : entity work.scl_memory_6(impl) port map (
        clock => clock,
        in_unnamed => s_unnamed,
        in_put_rewired => s_put_rewired,
        in_in_data_packed_delayed1 => s_in_data_packed_delayed1_2,
        in_in_data_packed => s_in_data_packed,
        in_in_valid => s_in_valid,
        out_conflict_bypass_mux => s_conflict_bypass_mux
    );
    default_comb : PROCESS(all)
        VARIABLE v_get_mux1 : UNSIGNED(6 downto 0); 
        VARIABLE v_in_data_FrameStream_valid : STD_LOGIC; 
        VARIABLE v_in_data_FrameStream_channel : UNSIGNED(0 downto 0); 
        VARIABLE v_in_data_FrameStream_data : UNSIGNED(511 downto 0); 
        VARIABLE v_in_data_FrameStream_error : UNSIGNED(0 downto 0); 
        VARIABLE v_in_data_FrameStream_empty : UNSIGNED(5 downto 0); 
        VARIABLE v_in_data_FrameStream_startofpacket : STD_LOGIC; 
        VARIABLE v_in_data_FrameStream_endofpacket : STD_LOGIC; 
        VARIABLE v_in_data_Bit : STD_LOGIC; 
        VARIABLE v_eq : STD_LOGIC; 
        VARIABLE v_get : UNSIGNED(6 downto 0); 
        VARIABLE v_m_get : UNSIGNED(6 downto 0); 
        VARIABLE v_out_data_packed : UNSIGNED(523 downto 0); 
        VARIABLE v_m_put : UNSIGNED(6 downto 0); 
        VARIABLE v_m_get_2 : UNSIGNED(6 downto 0); 
        VARIABLE v_m_put_2 : UNSIGNED(6 downto 0); 
        VARIABLE v_put : UNSIGNED(6 downto 0); 
    BEGIN
        out_full <= s_full;
        out_empty <= s_empty;
        v_in_data_FrameStream_valid := in_in_frame_valid;
        v_in_data_FrameStream_channel := in_in_frame_channel;
        v_in_data_FrameStream_data := in_in_frame_data;
        v_in_data_FrameStream_error := in_in_frame_error;
        v_in_data_FrameStream_empty := in_in_frame_empty;
        v_in_data_FrameStream_startofpacket := in_in_frame_startofpacket;
        v_in_data_FrameStream_endofpacket := in_in_frame_endofpacket;
        v_in_data_Bit := in_dbg_dataIn4;
        s_in_data_packed <= (v_in_data_Bit & v_in_data_FrameStream_endofpacket & v_in_data_FrameStream_startofpacket & v_in_data_FrameStream_empty & v_in_data_FrameStream_error & v_in_data_FrameStream_data & v_in_data_FrameStream_channel & v_in_data_FrameStream_valid);
        s_in_valid <= in_in_valid;
        v_put := s_put;
        s_out_ready <= in_out_ready;
        v_get := s_get;
        v_out_data_packed := s_conflict_bypass_mux;
        out_out_data_FrameStream_valid <= v_out_data_packed(0);
        out_out_data_FrameStream_channel <= v_out_data_packed(1 downto 1);
        out_out_data_FrameStream_data <= v_out_data_packed(513 downto 2);
        out_out_data_FrameStream_error <= v_out_data_packed(514 downto 514);
        out_out_data_FrameStream_empty <= v_out_data_packed(520 downto 515);
        out_out_data_FrameStream_startofpacket <= v_out_data_packed(521);
        out_out_data_FrameStream_endofpacket <= v_out_data_packed(522);
        out_out_data_Bit <= v_out_data_packed(523);
        s_get_plus_const_1 <= (v_get + "0000001");
        IF s_out_ready = '1' THEN
            v_m_get_2 := s_get_plus_const_1;
        ELSE
            v_m_get_2 := v_get;
        END IF;
        v_m_get := v_m_get_2;
        v_get_mux1 := v_m_get_2;
        s_unnamed <= v_get_mux1(5 downto 0);
        s_put_plus_const_1 <= (v_put + "0000001");
        IF s_in_valid = '1' THEN
            v_m_put_2 := s_put_plus_const_1;
        ELSE
            v_m_put_2 := v_put;
        END IF;
        v_m_put := v_m_put_2;
        v_eq := bool2stdlogic(v_m_put(5 downto 0) = v_m_get(5 downto 0));
        s_put_rewired <= v_put(5 downto 0);
        s_unnamed_2 <= (v_eq and bool2stdlogic(v_m_put(6) = v_m_get(6)));
        s_unnamed_3 <= (v_eq and bool2stdlogic(v_m_put(6) /= v_m_get(6)));
        s_in_data_packed_delayed1_2 <= s_in_data_packed_delayed1;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            IF (s_out_ready = '1') THEN
                s_get <= s_get_plus_const_1;
            END IF;
            s_full <= s_unnamed_3;
            s_empty <= s_unnamed_2;
            IF (s_in_valid = '1') THEN
                s_put <= s_put_plus_const_1;
            END IF;
            s_in_data_packed_delayed1 <= s_in_data_packed;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: FrameFifo
-- 
------------------------------------------------

ENTITY FrameFifo IS 
    PORT(
        clock : IN STD_LOGIC;
        in_in_frame_valid : IN STD_LOGIC;
        in_in_frame_startofpacket : IN STD_LOGIC;
        in_in_frame_endofpacket : IN STD_LOGIC;
        in_in_frame_channel : IN UNSIGNED(0 downto 0);
        in_in_frame_data : IN UNSIGNED(511 downto 0);
        in_in_frame_error : IN UNSIGNED(0 downto 0);
        in_in_frame_empty : IN UNSIGNED(5 downto 0);
        in_dbg_dataIn4 : IN STD_LOGIC;
        in_out_ready_mux1 : IN STD_LOGIC;
        out_empty_not_2 : OUT STD_LOGIC;
        out_out_data_FrameStream_valid : OUT STD_LOGIC;
        out_out_data_FrameStream_channel : OUT UNSIGNED(0 downto 0);
        out_out_data_FrameStream_data : OUT UNSIGNED(511 downto 0);
        out_out_data_FrameStream_error : OUT UNSIGNED(0 downto 0);
        out_out_data_FrameStream_empty : OUT UNSIGNED(5 downto 0);
        out_out_data_FrameStream_startofpacket : OUT STD_LOGIC;
        out_out_data_FrameStream_endofpacket : OUT STD_LOGIC;
        out_out_data_Bit : OUT STD_LOGIC;
        out_full_not : OUT STD_LOGIC;
        out_empty_not : OUT STD_LOGIC;
        in_unnamed : IN STD_LOGIC
    );
END FrameFifo;

ARCHITECTURE impl OF FrameFifo IS 
    SIGNAL s_full : STD_LOGIC; 
    SIGNAL s_in_valid : STD_LOGIC; 
    SIGNAL s_empty : STD_LOGIC; 
    SIGNAL s_out_ready : STD_LOGIC; 
BEGIN
    scl_fifo0 : entity work.scl_fifo(impl) port map (
        clock => clock,
        in_in_frame_valid => in_in_frame_valid,
        in_in_frame_startofpacket => in_in_frame_startofpacket,
        in_in_frame_endofpacket => in_in_frame_endofpacket,
        in_dbg_dataIn4 => in_dbg_dataIn4,
        in_in_frame_channel => in_in_frame_channel,
        in_in_frame_data => in_in_frame_data,
        in_in_frame_error => in_in_frame_error,
        in_in_frame_empty => in_in_frame_empty,
        in_in_valid => s_in_valid,
        in_out_ready => s_out_ready,
        out_full => s_full,
        out_empty => s_empty,
        out_out_data_FrameStream_startofpacket => out_out_data_FrameStream_startofpacket,
        out_out_data_FrameStream_valid => out_out_data_FrameStream_valid,
        out_out_data_FrameStream_empty => out_out_data_FrameStream_empty,
        out_out_data_Bit => out_out_data_Bit,
        out_out_data_FrameStream_endofpacket => out_out_data_FrameStream_endofpacket,
        out_out_data_FrameStream_channel => out_out_data_FrameStream_channel,
        out_out_data_FrameStream_data => out_out_data_FrameStream_data,
        out_out_data_FrameStream_error => out_out_data_FrameStream_error
    );
    default_comb : PROCESS(all)
    BEGIN
        out_empty_not_2 <= ( not s_empty);
        out_full_not <= ( not s_full);
        s_in_valid <= (out_full_not and in_unnamed);
        out_empty_not <= out_empty_not_2;
        s_out_ready <= (in_out_ready_mux1 and out_empty_not);
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: result
-- 
------------------------------------------------

ENTITY result IS 
    PORT(
        in_inVirtualWindow : IN STD_LOGIC;
        in_m_isReplay : IN STD_LOGIC;
        out_out_valid : OUT STD_LOGIC;
        out_out_code : OUT UNSIGNED(2 downto 0);
        in_unnamed_4 : IN UNSIGNED(2 downto 0);
        in_unnamed_5 : IN STD_LOGIC;
        in_unnamed_3 : IN STD_LOGIC;
        in_unnamed : IN STD_LOGIC;
        in_unnamed_2 : IN STD_LOGIC
    );
END result;

ARCHITECTURE impl OF result IS 
BEGIN
    default_comb : PROCESS(all)
        CONSTANT C_PASSINVIRTUALWINDOW : UNSIGNED(2 downto 0) := "101"; 
        CONSTANT C_PASSABOVEWINDOW : UNSIGNED(2 downto 0) := "001"; 
        CONSTANT C_FAILBELOWWINDOW : UNSIGNED(2 downto 0) := "000"; 
        CONSTANT C_FAILREPLAY : UNSIGNED(2 downto 0) := "010"; 
        CONSTANT C_PASSINPHYSICALWINDOW : UNSIGNED(2 downto 0) := "111"; 
        CONSTANT C_OPUPDATE : UNSIGNED(2 downto 0) := "000"; 
        CONSTANT C_PASSCHECKDISABLED : UNSIGNED(2 downto 0) := "011"; 
        VARIABLE v_PassInPhysicalWindow_mux1 : UNSIGNED(2 downto 0); 
        VARIABLE v_nested_condition_not_m_isReplay : STD_LOGIC; 
        VARIABLE v_nested_condition_m_isReplay : STD_LOGIC; 
        VARIABLE v_FailReplay_mux1 : UNSIGNED(2 downto 0); 
        VARIABLE v_nested_condition_inVirtualWindow : STD_LOGIC; 
        VARIABLE v_PassInVirtualWindow_mux1 : UNSIGNED(2 downto 0); 
        VARIABLE v_PassCheckDisabled_mux1 : UNSIGNED(2 downto 0); 
        VARIABLE v_not_m_isReplay : STD_LOGIC; 
        VARIABLE v_out_code : UNSIGNED(2 downto 0); 
    BEGIN
        IF (in_unnamed_2 = '1' and in_unnamed_3 = '1') THEN
            v_PassCheckDisabled_mux1 := C_PASSABOVEWINDOW;
        ELSE
            v_PassCheckDisabled_mux1 := C_PASSCHECKDISABLED;
        END IF;
        v_nested_condition_m_isReplay := (in_m_isReplay and ((in_unnamed and in_inVirtualWindow) and (( not in_unnamed_2) and in_unnamed_3)));
        IF v_nested_condition_m_isReplay = '1' THEN
            v_FailReplay_mux1 := C_FAILREPLAY;
        ELSE
            v_FailReplay_mux1 := v_PassCheckDisabled_mux1;
        END IF;
        v_not_m_isReplay := ( not in_m_isReplay);
        v_nested_condition_not_m_isReplay := (v_not_m_isReplay and ((in_unnamed and in_inVirtualWindow) and (( not in_unnamed_2) and in_unnamed_3)));
        IF v_nested_condition_not_m_isReplay = '1' THEN
            v_PassInPhysicalWindow_mux1 := C_PASSINPHYSICALWINDOW;
        ELSE
            v_PassInPhysicalWindow_mux1 := v_FailReplay_mux1;
        END IF;
        v_nested_condition_inVirtualWindow := (in_inVirtualWindow and (( not ((in_unnamed and in_inVirtualWindow) or ( not ( not in_unnamed_2)))) and in_unnamed_3));
        IF v_nested_condition_inVirtualWindow = '1' THEN
            v_PassInVirtualWindow_mux1 := C_PASSINVIRTUALWINDOW;
        ELSE
            v_PassInVirtualWindow_mux1 := v_PassInPhysicalWindow_mux1;
        END IF;
        IF (( not (in_inVirtualWindow = '1' or ( not ( not ((in_unnamed = '1' and in_inVirtualWindow = '1') or ( not ( not in_unnamed_2 = '1'))))))) and in_unnamed_3 = '1') THEN
            v_out_code := C_FAILBELOWWINDOW;
        ELSE
            v_out_code := v_PassInVirtualWindow_mux1;
        END IF;
        out_out_valid <= (in_unnamed_5 and bool2stdlogic(in_unnamed_4 = C_OPUPDATE));
        out_out_code <= v_out_code;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: flat_mux_3
-- 
------------------------------------------------

ENTITY flat_mux_3 IS 
    PORT(
        in_selector : IN UNSIGNED(5 downto 0);
        out_selected : OUT UNSIGNED(0 downto 0);
        in_flat_array : IN UNSIGNED(63 downto 0)
    );
END flat_mux_3;

ARCHITECTURE impl OF flat_mux_3 IS 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_flat_array_bit_5_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_4_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_selector : UNSIGNED(5 downto 0); 
        VARIABLE v_flat_array : UNSIGNED(63 downto 0); 
        VARIABLE v_flat_array_bit_2_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_3_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_11_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_14_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_15_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_8_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_6_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_10_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_13_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_9_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_16_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_17_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_12_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_7_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_18_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_22_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_24_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_26_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_25_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_29_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_27_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_28_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_21_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_19_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_20_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_23_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_30_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_37_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_36_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_31_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_39_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_34_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_41_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_32_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_45_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_35_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_33_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_40_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_38_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_44_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_43_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_42_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_46_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_47_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_49_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_54_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_53_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_48_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_58_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_57_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_60_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_62_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_52_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_51_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_59_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_61_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_56_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_55_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_50_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_selected : UNSIGNED(0 downto 0); 
    BEGIN
        v_selector := in_selector;
        v_flat_array := in_flat_array;
        IF (v_selector = "000001") THEN
            v_flat_array_bit_0_mux1 := v_flat_array(1 downto 1);
        ELSE
            v_flat_array_bit_0_mux1 := v_flat_array(0 downto 0);
        END IF;
        IF (v_selector = "000010") THEN
            v_flat_array_bit_2_mux1 := v_flat_array(2 downto 2);
        ELSE
            v_flat_array_bit_2_mux1 := v_flat_array_bit_0_mux1;
        END IF;
        IF (v_selector = "000011") THEN
            v_flat_array_bit_3_mux1 := v_flat_array(3 downto 3);
        ELSE
            v_flat_array_bit_3_mux1 := v_flat_array_bit_2_mux1;
        END IF;
        IF (v_selector = "000100") THEN
            v_flat_array_bit_4_mux1 := v_flat_array(4 downto 4);
        ELSE
            v_flat_array_bit_4_mux1 := v_flat_array_bit_3_mux1;
        END IF;
        IF (v_selector = "000101") THEN
            v_flat_array_bit_5_mux1 := v_flat_array(5 downto 5);
        ELSE
            v_flat_array_bit_5_mux1 := v_flat_array_bit_4_mux1;
        END IF;
        IF (v_selector = "000110") THEN
            v_flat_array_bit_6_mux1 := v_flat_array(6 downto 6);
        ELSE
            v_flat_array_bit_6_mux1 := v_flat_array_bit_5_mux1;
        END IF;
        IF (v_selector = "000111") THEN
            v_flat_array_bit_7_mux1 := v_flat_array(7 downto 7);
        ELSE
            v_flat_array_bit_7_mux1 := v_flat_array_bit_6_mux1;
        END IF;
        IF (v_selector = "001000") THEN
            v_flat_array_bit_8_mux1 := v_flat_array(8 downto 8);
        ELSE
            v_flat_array_bit_8_mux1 := v_flat_array_bit_7_mux1;
        END IF;
        IF (v_selector = "001001") THEN
            v_flat_array_bit_9_mux1 := v_flat_array(9 downto 9);
        ELSE
            v_flat_array_bit_9_mux1 := v_flat_array_bit_8_mux1;
        END IF;
        IF (v_selector = "001010") THEN
            v_flat_array_bit_10_mux1 := v_flat_array(10 downto 10);
        ELSE
            v_flat_array_bit_10_mux1 := v_flat_array_bit_9_mux1;
        END IF;
        IF (v_selector = "001011") THEN
            v_flat_array_bit_11_mux1 := v_flat_array(11 downto 11);
        ELSE
            v_flat_array_bit_11_mux1 := v_flat_array_bit_10_mux1;
        END IF;
        IF (v_selector = "001100") THEN
            v_flat_array_bit_12_mux1 := v_flat_array(12 downto 12);
        ELSE
            v_flat_array_bit_12_mux1 := v_flat_array_bit_11_mux1;
        END IF;
        IF (v_selector = "001101") THEN
            v_flat_array_bit_13_mux1 := v_flat_array(13 downto 13);
        ELSE
            v_flat_array_bit_13_mux1 := v_flat_array_bit_12_mux1;
        END IF;
        IF (v_selector = "001110") THEN
            v_flat_array_bit_14_mux1 := v_flat_array(14 downto 14);
        ELSE
            v_flat_array_bit_14_mux1 := v_flat_array_bit_13_mux1;
        END IF;
        IF (v_selector = "001111") THEN
            v_flat_array_bit_15_mux1 := v_flat_array(15 downto 15);
        ELSE
            v_flat_array_bit_15_mux1 := v_flat_array_bit_14_mux1;
        END IF;
        IF (v_selector = "010000") THEN
            v_flat_array_bit_16_mux1 := v_flat_array(16 downto 16);
        ELSE
            v_flat_array_bit_16_mux1 := v_flat_array_bit_15_mux1;
        END IF;
        IF (v_selector = "010001") THEN
            v_flat_array_bit_17_mux1 := v_flat_array(17 downto 17);
        ELSE
            v_flat_array_bit_17_mux1 := v_flat_array_bit_16_mux1;
        END IF;
        IF (v_selector = "010010") THEN
            v_flat_array_bit_18_mux1 := v_flat_array(18 downto 18);
        ELSE
            v_flat_array_bit_18_mux1 := v_flat_array_bit_17_mux1;
        END IF;
        IF (v_selector = "010011") THEN
            v_flat_array_bit_19_mux1 := v_flat_array(19 downto 19);
        ELSE
            v_flat_array_bit_19_mux1 := v_flat_array_bit_18_mux1;
        END IF;
        IF (v_selector = "010100") THEN
            v_flat_array_bit_20_mux1 := v_flat_array(20 downto 20);
        ELSE
            v_flat_array_bit_20_mux1 := v_flat_array_bit_19_mux1;
        END IF;
        IF (v_selector = "010101") THEN
            v_flat_array_bit_21_mux1 := v_flat_array(21 downto 21);
        ELSE
            v_flat_array_bit_21_mux1 := v_flat_array_bit_20_mux1;
        END IF;
        IF (v_selector = "010110") THEN
            v_flat_array_bit_22_mux1 := v_flat_array(22 downto 22);
        ELSE
            v_flat_array_bit_22_mux1 := v_flat_array_bit_21_mux1;
        END IF;
        IF (v_selector = "010111") THEN
            v_flat_array_bit_23_mux1 := v_flat_array(23 downto 23);
        ELSE
            v_flat_array_bit_23_mux1 := v_flat_array_bit_22_mux1;
        END IF;
        IF (v_selector = "011000") THEN
            v_flat_array_bit_24_mux1 := v_flat_array(24 downto 24);
        ELSE
            v_flat_array_bit_24_mux1 := v_flat_array_bit_23_mux1;
        END IF;
        IF (v_selector = "011001") THEN
            v_flat_array_bit_25_mux1 := v_flat_array(25 downto 25);
        ELSE
            v_flat_array_bit_25_mux1 := v_flat_array_bit_24_mux1;
        END IF;
        IF (v_selector = "011010") THEN
            v_flat_array_bit_26_mux1 := v_flat_array(26 downto 26);
        ELSE
            v_flat_array_bit_26_mux1 := v_flat_array_bit_25_mux1;
        END IF;
        IF (v_selector = "011011") THEN
            v_flat_array_bit_27_mux1 := v_flat_array(27 downto 27);
        ELSE
            v_flat_array_bit_27_mux1 := v_flat_array_bit_26_mux1;
        END IF;
        IF (v_selector = "011100") THEN
            v_flat_array_bit_28_mux1 := v_flat_array(28 downto 28);
        ELSE
            v_flat_array_bit_28_mux1 := v_flat_array_bit_27_mux1;
        END IF;
        IF (v_selector = "011101") THEN
            v_flat_array_bit_29_mux1 := v_flat_array(29 downto 29);
        ELSE
            v_flat_array_bit_29_mux1 := v_flat_array_bit_28_mux1;
        END IF;
        IF (v_selector = "011110") THEN
            v_flat_array_bit_30_mux1 := v_flat_array(30 downto 30);
        ELSE
            v_flat_array_bit_30_mux1 := v_flat_array_bit_29_mux1;
        END IF;
        IF (v_selector = "011111") THEN
            v_flat_array_bit_31_mux1 := v_flat_array(31 downto 31);
        ELSE
            v_flat_array_bit_31_mux1 := v_flat_array_bit_30_mux1;
        END IF;
        IF (v_selector = "100000") THEN
            v_flat_array_bit_32_mux1 := v_flat_array(32 downto 32);
        ELSE
            v_flat_array_bit_32_mux1 := v_flat_array_bit_31_mux1;
        END IF;
        IF (v_selector = "100001") THEN
            v_flat_array_bit_33_mux1 := v_flat_array(33 downto 33);
        ELSE
            v_flat_array_bit_33_mux1 := v_flat_array_bit_32_mux1;
        END IF;
        IF (v_selector = "100010") THEN
            v_flat_array_bit_34_mux1 := v_flat_array(34 downto 34);
        ELSE
            v_flat_array_bit_34_mux1 := v_flat_array_bit_33_mux1;
        END IF;
        IF (v_selector = "100011") THEN
            v_flat_array_bit_35_mux1 := v_flat_array(35 downto 35);
        ELSE
            v_flat_array_bit_35_mux1 := v_flat_array_bit_34_mux1;
        END IF;
        IF (v_selector = "100100") THEN
            v_flat_array_bit_36_mux1 := v_flat_array(36 downto 36);
        ELSE
            v_flat_array_bit_36_mux1 := v_flat_array_bit_35_mux1;
        END IF;
        IF (v_selector = "100101") THEN
            v_flat_array_bit_37_mux1 := v_flat_array(37 downto 37);
        ELSE
            v_flat_array_bit_37_mux1 := v_flat_array_bit_36_mux1;
        END IF;
        IF (v_selector = "100110") THEN
            v_flat_array_bit_38_mux1 := v_flat_array(38 downto 38);
        ELSE
            v_flat_array_bit_38_mux1 := v_flat_array_bit_37_mux1;
        END IF;
        IF (v_selector = "100111") THEN
            v_flat_array_bit_39_mux1 := v_flat_array(39 downto 39);
        ELSE
            v_flat_array_bit_39_mux1 := v_flat_array_bit_38_mux1;
        END IF;
        IF (v_selector = "101000") THEN
            v_flat_array_bit_40_mux1 := v_flat_array(40 downto 40);
        ELSE
            v_flat_array_bit_40_mux1 := v_flat_array_bit_39_mux1;
        END IF;
        IF (v_selector = "101001") THEN
            v_flat_array_bit_41_mux1 := v_flat_array(41 downto 41);
        ELSE
            v_flat_array_bit_41_mux1 := v_flat_array_bit_40_mux1;
        END IF;
        IF (v_selector = "101010") THEN
            v_flat_array_bit_42_mux1 := v_flat_array(42 downto 42);
        ELSE
            v_flat_array_bit_42_mux1 := v_flat_array_bit_41_mux1;
        END IF;
        IF (v_selector = "101011") THEN
            v_flat_array_bit_43_mux1 := v_flat_array(43 downto 43);
        ELSE
            v_flat_array_bit_43_mux1 := v_flat_array_bit_42_mux1;
        END IF;
        IF (v_selector = "101100") THEN
            v_flat_array_bit_44_mux1 := v_flat_array(44 downto 44);
        ELSE
            v_flat_array_bit_44_mux1 := v_flat_array_bit_43_mux1;
        END IF;
        IF (v_selector = "101101") THEN
            v_flat_array_bit_45_mux1 := v_flat_array(45 downto 45);
        ELSE
            v_flat_array_bit_45_mux1 := v_flat_array_bit_44_mux1;
        END IF;
        IF (v_selector = "101110") THEN
            v_flat_array_bit_46_mux1 := v_flat_array(46 downto 46);
        ELSE
            v_flat_array_bit_46_mux1 := v_flat_array_bit_45_mux1;
        END IF;
        IF (v_selector = "101111") THEN
            v_flat_array_bit_47_mux1 := v_flat_array(47 downto 47);
        ELSE
            v_flat_array_bit_47_mux1 := v_flat_array_bit_46_mux1;
        END IF;
        IF (v_selector = "110000") THEN
            v_flat_array_bit_48_mux1 := v_flat_array(48 downto 48);
        ELSE
            v_flat_array_bit_48_mux1 := v_flat_array_bit_47_mux1;
        END IF;
        IF (v_selector = "110001") THEN
            v_flat_array_bit_49_mux1 := v_flat_array(49 downto 49);
        ELSE
            v_flat_array_bit_49_mux1 := v_flat_array_bit_48_mux1;
        END IF;
        IF (v_selector = "110010") THEN
            v_flat_array_bit_50_mux1 := v_flat_array(50 downto 50);
        ELSE
            v_flat_array_bit_50_mux1 := v_flat_array_bit_49_mux1;
        END IF;
        IF (v_selector = "110011") THEN
            v_flat_array_bit_51_mux1 := v_flat_array(51 downto 51);
        ELSE
            v_flat_array_bit_51_mux1 := v_flat_array_bit_50_mux1;
        END IF;
        IF (v_selector = "110100") THEN
            v_flat_array_bit_52_mux1 := v_flat_array(52 downto 52);
        ELSE
            v_flat_array_bit_52_mux1 := v_flat_array_bit_51_mux1;
        END IF;
        IF (v_selector = "110101") THEN
            v_flat_array_bit_53_mux1 := v_flat_array(53 downto 53);
        ELSE
            v_flat_array_bit_53_mux1 := v_flat_array_bit_52_mux1;
        END IF;
        IF (v_selector = "110110") THEN
            v_flat_array_bit_54_mux1 := v_flat_array(54 downto 54);
        ELSE
            v_flat_array_bit_54_mux1 := v_flat_array_bit_53_mux1;
        END IF;
        IF (v_selector = "110111") THEN
            v_flat_array_bit_55_mux1 := v_flat_array(55 downto 55);
        ELSE
            v_flat_array_bit_55_mux1 := v_flat_array_bit_54_mux1;
        END IF;
        IF (v_selector = "111000") THEN
            v_flat_array_bit_56_mux1 := v_flat_array(56 downto 56);
        ELSE
            v_flat_array_bit_56_mux1 := v_flat_array_bit_55_mux1;
        END IF;
        IF (v_selector = "111001") THEN
            v_flat_array_bit_57_mux1 := v_flat_array(57 downto 57);
        ELSE
            v_flat_array_bit_57_mux1 := v_flat_array_bit_56_mux1;
        END IF;
        IF (v_selector = "111010") THEN
            v_flat_array_bit_58_mux1 := v_flat_array(58 downto 58);
        ELSE
            v_flat_array_bit_58_mux1 := v_flat_array_bit_57_mux1;
        END IF;
        IF (v_selector = "111011") THEN
            v_flat_array_bit_59_mux1 := v_flat_array(59 downto 59);
        ELSE
            v_flat_array_bit_59_mux1 := v_flat_array_bit_58_mux1;
        END IF;
        IF (v_selector = "111100") THEN
            v_flat_array_bit_60_mux1 := v_flat_array(60 downto 60);
        ELSE
            v_flat_array_bit_60_mux1 := v_flat_array_bit_59_mux1;
        END IF;
        IF (v_selector = "111101") THEN
            v_flat_array_bit_61_mux1 := v_flat_array(61 downto 61);
        ELSE
            v_flat_array_bit_61_mux1 := v_flat_array_bit_60_mux1;
        END IF;
        IF (v_selector = "111110") THEN
            v_flat_array_bit_62_mux1 := v_flat_array(62 downto 62);
        ELSE
            v_flat_array_bit_62_mux1 := v_flat_array_bit_61_mux1;
        END IF;
        IF (v_selector = "111111") THEN
            v_selected := v_flat_array(63 downto 63);
        ELSE
            v_selected := v_flat_array_bit_62_mux1;
        END IF;
        out_selected <= v_selected;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: flat_mux_2
-- 
------------------------------------------------

ENTITY flat_mux_2 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_data : IN UNSIGNED(255 downto 0);
        in_selector : IN UNSIGNED(1 downto 0);
        out_flat_array : OUT UNSIGNED(63 downto 0)
    );
END flat_mux_2;

ARCHITECTURE impl OF flat_mux_2 IS 
    SIGNAL s_selected : UNSIGNED(63 downto 0); 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_selected : UNSIGNED(63 downto 0); 
        VARIABLE v_flat_array_rewired_mux1 : UNSIGNED(63 downto 0); 
        VARIABLE v_selector : UNSIGNED(1 downto 0); 
        VARIABLE v_flat_array_rewired_mux1_2 : UNSIGNED(63 downto 0); 
        VARIABLE v_flat_array : UNSIGNED(255 downto 0); 
    BEGIN
        v_selector := in_selector;
        v_flat_array := in_data;
        IF (v_selector = "01") THEN
            v_flat_array_rewired_mux1_2 := v_flat_array(127 downto 64);
        ELSE
            v_flat_array_rewired_mux1_2 := v_flat_array(63 downto 0);
        END IF;
        IF (v_selector = "10") THEN
            v_flat_array_rewired_mux1 := v_flat_array(191 downto 128);
        ELSE
            v_flat_array_rewired_mux1 := v_flat_array_rewired_mux1_2;
        END IF;
        IF (v_selector = "11") THEN
            v_selected := v_flat_array(255 downto 192);
        ELSE
            v_selected := v_flat_array_rewired_mux1;
        END IF;
        s_selected <= v_selected;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            out_flat_array <= s_selected;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: mux_large_2
-- 
------------------------------------------------

ENTITY mux_large_2 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bitmap : IN UNSIGNED(255 downto 0);
        in_bitAddr : IN UNSIGNED(7 downto 0);
        out_out_bit_0 : OUT STD_LOGIC
    );
END mux_large_2;

ARCHITECTURE impl OF mux_large_2 IS 
    SIGNAL s_selector : UNSIGNED(1 downto 0); 
    SIGNAL s_data : UNSIGNED(255 downto 0); 
    SIGNAL s_selector_2 : UNSIGNED(7 downto 0); 
    SIGNAL s_selector_3 : UNSIGNED(5 downto 0); 
    SIGNAL s_selected : UNSIGNED(0 downto 0); 
    SIGNAL s_selector_delayed1 : UNSIGNED(7 downto 0); 
    SIGNAL s_flat_array : UNSIGNED(63 downto 0); 
BEGIN
    flat_mux0 : entity work.flat_mux_2(impl) port map (
        clock => clock,
        in_selector => s_selector,
        in_data => s_data,
        out_flat_array => s_flat_array
    );
    default_comb : PROCESS(all)
        VARIABLE v_selector_delayed1 : UNSIGNED(7 downto 0); 
        VARIABLE v_out : UNSIGNED(0 downto 0); 
    BEGIN
        s_selector_2 <= in_bitAddr;
        s_data <= in_bitmap;
        s_selector <= s_selector_2(7 downto 6);
        v_out := s_selected;
        out_out_bit_0 <= v_out(0);
        v_selector_delayed1 := s_selector_delayed1;
        s_selector_3 <= v_selector_delayed1(5 downto 0);
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_selector_delayed1 <= s_selector_2;
        END IF;
    END PROCESS;

    flat_mux1 : entity work.flat_mux_3(impl) port map (
        in_selector => s_selector_3,
        in_flat_array => s_flat_array,
        out_selected => s_selected
    );
END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: IsReplay
-- 
------------------------------------------------

ENTITY IsReplay IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bitmap : IN UNSIGNED(255 downto 0);
        out_isReplay : OUT STD_LOGIC;
        in_bitAddr : IN UNSIGNED(7 downto 0);
        in_secondStageValid_delayed6 : IN STD_LOGIC
    );
END IsReplay;

ARCHITECTURE impl OF IsReplay IS 
    SIGNAL s_bitAddr : UNSIGNED(7 downto 0); 
    SIGNAL s_out_bit_0 : STD_LOGIC; 
BEGIN
    mux_large0 : entity work.mux_large_2(impl) port map (
        clock => clock,
        in_bitmap => in_bitmap,
        in_bitAddr => s_bitAddr,
        out_out_bit_0 => s_out_bit_0
    );
    default_comb : PROCESS(all)
    BEGIN
        s_bitAddr <= in_bitAddr;
        out_isReplay <= (s_out_bit_0 and in_secondStageValid_delayed6);
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: secondStageUpdate
-- 
------------------------------------------------

ENTITY secondStageUpdate IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bitmap : IN UNSIGNED(255 downto 0);
        out_bitmap_next : OUT UNSIGNED(255 downto 0);
        out_bitAddr : OUT UNSIGNED(7 downto 0);
        in_aboveWindow_delayed3 : IN STD_LOGIC;
        in_lastSequenceNumber_delayed3 : IN UNSIGNED(63 downto 0);
        in_unnamed : IN UNSIGNED(63 downto 0);
        in_unnamed_2 : IN UNSIGNED(2 downto 0);
        in_m_cmd_valid_delayed9_and_updateState : IN STD_LOGIC;
        out_bitmap_next_delayed1 : OUT UNSIGNED(255 downto 0)
    );
END secondStageUpdate;

ARCHITECTURE impl OF secondStageUpdate IS 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_unnamed_3 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_4 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_5 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_6 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_7 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_8 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_9 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_10 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_11 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_12 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_13 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_14 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_15 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_16 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_17 : STD_LOGIC := '0'; 
    SIGNAL s_bitmap_next_delayed1 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_18 : UNSIGNED(7 downto 0); 
    SIGNAL s_unnamed_19 : UNSIGNED(7 downto 0); 
    SIGNAL s_unnamed_20 : UNSIGNED(7 downto 0); 
    SIGNAL s_unnamed_21 : UNSIGNED(7 downto 0); 
    SIGNAL s_unnamed_22 : UNSIGNED(7 downto 0); 
    SIGNAL s_unnamed_23 : STD_LOGIC; 
    SIGNAL s_unnamed_24 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_25 : STD_LOGIC; 
    SIGNAL s_unnamed_26 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_27 : STD_LOGIC; 
    SIGNAL s_unnamed_28 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_29 : STD_LOGIC; 
    SIGNAL s_unnamed_30 : STD_LOGIC; 
    SIGNAL s_unnamed_31 : STD_LOGIC; 
    SIGNAL s_unnamed_32 : STD_LOGIC; 
    SIGNAL s_unnamed_33 : STD_LOGIC; 
    SIGNAL s_unnamed_34 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_35 : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_36 : UNSIGNED(7 downto 0); 
    SIGNAL s_unnamed_37 : UNSIGNED(7 downto 0); 
    SIGNAL s_unnamed_38 : UNSIGNED(7 downto 0); 
    SIGNAL s_unnamed_39 : UNSIGNED(7 downto 0); 
BEGIN
    default_comb : PROCESS(all)
        CONSTANT C_OPSETSEQNR : UNSIGNED(2 downto 0) := "010"; 
        VARIABLE v_bitmap : UNSIGNED(255 downto 0); 
        VARIABLE v_const_0_mux1 : UNSIGNED(255 downto 0); 
        VARIABLE v_const_0_mux1_2 : UNSIGNED(255 downto 0); 
    BEGIN
        v_bitmap := in_bitmap;
        s_unnamed <= (bool2stdlogic(in_unnamed_2 = C_OPSETSEQNR) and in_m_cmd_valid_delayed9_and_updateState);
        IF s_unnamed_12 = '1' THEN
            v_const_0_mux1_2 := "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000";
        ELSE
            v_const_0_mux1_2 := v_bitmap;
        END IF;
        s_unnamed_2 <= ((in_aboveWindow_delayed3 and bool2stdlogic(in_unnamed(63 downto 8) /= in_lastSequenceNumber_delayed3(63 downto 8))) and in_m_cmd_valid_delayed9_and_updateState);
        IF s_unnamed_17 = '1' THEN
            v_const_0_mux1 := "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000";
        ELSE
            v_const_0_mux1 := v_const_0_mux1_2;
        END IF;
        out_bitmap_next <= (v_const_0_mux1 or s_unnamed_15);
        s_unnamed_19 <= in_unnamed(7 downto 0);
        s_unnamed_18 <= s_unnamed_39;
        s_unnamed_20 <= s_unnamed_38;
        s_unnamed_21 <= s_unnamed_37;
        s_unnamed_22 <= s_unnamed_36;
        out_bitmap_next_delayed1 <= s_bitmap_next_delayed1;
        s_unnamed_26 <= (bool2stdlogic(s_unnamed_19 = "11111111") & bool2stdlogic(s_unnamed_19 = "11111110") & bool2stdlogic(s_unnamed_19 = "11111101") & bool2stdlogic(s_unnamed_19 = "11111100") & bool2stdlogic(s_unnamed_19 = "11111011") & bool2stdlogic(s_unnamed_19 = "11111010") & bool2stdlogic(s_unnamed_19 = "11111001") & bool2stdlogic(s_unnamed_19 = "11111000") & bool2stdlogic(s_unnamed_19 = "11110111") & bool2stdlogic(s_unnamed_19 = "11110110") & bool2stdlogic(s_unnamed_19 = "11110101") & bool2stdlogic(s_unnamed_19 = "11110100") & bool2stdlogic(s_unnamed_19 = "11110011") & bool2stdlogic(s_unnamed_19 = "11110010") & bool2stdlogic(s_unnamed_19 = "11110001") & 
            bool2stdlogic(s_unnamed_19 = "11110000") & bool2stdlogic(s_unnamed_19 = "11101111") & bool2stdlogic(s_unnamed_19 = "11101110") & bool2stdlogic(s_unnamed_19 = "11101101") & bool2stdlogic(s_unnamed_19 = "11101100") & bool2stdlogic(s_unnamed_19 = "11101011") & bool2stdlogic(s_unnamed_19 = "11101010") & bool2stdlogic(s_unnamed_19 = "11101001") & bool2stdlogic(s_unnamed_19 = "11101000") & bool2stdlogic(s_unnamed_19 = "11100111") & bool2stdlogic(s_unnamed_19 = "11100110") & bool2stdlogic(s_unnamed_19 = "11100101") & bool2stdlogic(s_unnamed_19 = "11100100") & bool2stdlogic(s_unnamed_19 = "11100011") & bool2stdlogic(s_unnamed_19 = "11100010") & bool2stdlogic(s_unnamed_19 = "11100001") & 
            bool2stdlogic(s_unnamed_19 = "11100000") & bool2stdlogic(s_unnamed_19 = "11011111") & bool2stdlogic(s_unnamed_19 = "11011110") & bool2stdlogic(s_unnamed_19 = "11011101") & bool2stdlogic(s_unnamed_19 = "11011100") & bool2stdlogic(s_unnamed_19 = "11011011") & bool2stdlogic(s_unnamed_19 = "11011010") & bool2stdlogic(s_unnamed_19 = "11011001") & bool2stdlogic(s_unnamed_19 = "11011000") & bool2stdlogic(s_unnamed_19 = "11010111") & bool2stdlogic(s_unnamed_19 = "11010110") & bool2stdlogic(s_unnamed_19 = "11010101") & bool2stdlogic(s_unnamed_19 = "11010100") & bool2stdlogic(s_unnamed_19 = "11010011") & bool2stdlogic(s_unnamed_19 = "11010010") & bool2stdlogic(s_unnamed_19 = "11010001") & 
            bool2stdlogic(s_unnamed_19 = "11010000") & bool2stdlogic(s_unnamed_19 = "11001111") & bool2stdlogic(s_unnamed_19 = "11001110") & bool2stdlogic(s_unnamed_19 = "11001101") & bool2stdlogic(s_unnamed_19 = "11001100") & bool2stdlogic(s_unnamed_19 = "11001011") & bool2stdlogic(s_unnamed_19 = "11001010") & bool2stdlogic(s_unnamed_19 = "11001001") & bool2stdlogic(s_unnamed_19 = "11001000") & bool2stdlogic(s_unnamed_19 = "11000111") & bool2stdlogic(s_unnamed_19 = "11000110") & bool2stdlogic(s_unnamed_19 = "11000101") & bool2stdlogic(s_unnamed_19 = "11000100") & bool2stdlogic(s_unnamed_19 = "11000011") & bool2stdlogic(s_unnamed_19 = "11000010") & bool2stdlogic(s_unnamed_19 = "11000001") & 
            bool2stdlogic(s_unnamed_19 = "11000000") & bool2stdlogic(s_unnamed_19 = "10111111") & bool2stdlogic(s_unnamed_19 = "10111110") & bool2stdlogic(s_unnamed_19 = "10111101") & bool2stdlogic(s_unnamed_19 = "10111100") & bool2stdlogic(s_unnamed_19 = "10111011") & bool2stdlogic(s_unnamed_19 = "10111010") & bool2stdlogic(s_unnamed_19 = "10111001") & bool2stdlogic(s_unnamed_19 = "10111000") & bool2stdlogic(s_unnamed_19 = "10110111") & bool2stdlogic(s_unnamed_19 = "10110110") & bool2stdlogic(s_unnamed_19 = "10110101") & bool2stdlogic(s_unnamed_19 = "10110100") & bool2stdlogic(s_unnamed_19 = "10110011") & bool2stdlogic(s_unnamed_19 = "10110010") & bool2stdlogic(s_unnamed_19 = "10110001") & 
            bool2stdlogic(s_unnamed_19 = "10110000") & bool2stdlogic(s_unnamed_19 = "10101111") & bool2stdlogic(s_unnamed_19 = "10101110") & bool2stdlogic(s_unnamed_19 = "10101101") & bool2stdlogic(s_unnamed_19 = "10101100") & bool2stdlogic(s_unnamed_19 = "10101011") & bool2stdlogic(s_unnamed_19 = "10101010") & bool2stdlogic(s_unnamed_19 = "10101001") & bool2stdlogic(s_unnamed_19 = "10101000") & bool2stdlogic(s_unnamed_19 = "10100111") & bool2stdlogic(s_unnamed_19 = "10100110") & bool2stdlogic(s_unnamed_19 = "10100101") & bool2stdlogic(s_unnamed_19 = "10100100") & bool2stdlogic(s_unnamed_19 = "10100011") & bool2stdlogic(s_unnamed_19 = "10100010") & bool2stdlogic(s_unnamed_19 = "10100001") & 
            bool2stdlogic(s_unnamed_19 = "10100000") & bool2stdlogic(s_unnamed_19 = "10011111") & bool2stdlogic(s_unnamed_19 = "10011110") & bool2stdlogic(s_unnamed_19 = "10011101") & bool2stdlogic(s_unnamed_19 = "10011100") & bool2stdlogic(s_unnamed_19 = "10011011") & bool2stdlogic(s_unnamed_19 = "10011010") & bool2stdlogic(s_unnamed_19 = "10011001") & bool2stdlogic(s_unnamed_19 = "10011000") & bool2stdlogic(s_unnamed_19 = "10010111") & bool2stdlogic(s_unnamed_19 = "10010110") & bool2stdlogic(s_unnamed_19 = "10010101") & bool2stdlogic(s_unnamed_19 = "10010100") & bool2stdlogic(s_unnamed_19 = "10010011") & bool2stdlogic(s_unnamed_19 = "10010010") & bool2stdlogic(s_unnamed_19 = "10010001") & 
            bool2stdlogic(s_unnamed_19 = "10010000") & bool2stdlogic(s_unnamed_19 = "10001111") & bool2stdlogic(s_unnamed_19 = "10001110") & bool2stdlogic(s_unnamed_19 = "10001101") & bool2stdlogic(s_unnamed_19 = "10001100") & bool2stdlogic(s_unnamed_19 = "10001011") & bool2stdlogic(s_unnamed_19 = "10001010") & bool2stdlogic(s_unnamed_19 = "10001001") & bool2stdlogic(s_unnamed_19 = "10001000") & bool2stdlogic(s_unnamed_19 = "10000111") & bool2stdlogic(s_unnamed_19 = "10000110") & bool2stdlogic(s_unnamed_19 = "10000101") & bool2stdlogic(s_unnamed_19 = "10000100") & bool2stdlogic(s_unnamed_19 = "10000011") & bool2stdlogic(s_unnamed_19 = "10000010") & bool2stdlogic(s_unnamed_19 = "10000001") & 
            bool2stdlogic(s_unnamed_19 = "10000000") & bool2stdlogic(s_unnamed_19 = "01111111") & bool2stdlogic(s_unnamed_19 = "01111110") & bool2stdlogic(s_unnamed_19 = "01111101") & bool2stdlogic(s_unnamed_19 = "01111100") & bool2stdlogic(s_unnamed_19 = "01111011") & bool2stdlogic(s_unnamed_19 = "01111010") & bool2stdlogic(s_unnamed_19 = "01111001") & bool2stdlogic(s_unnamed_19 = "01111000") & bool2stdlogic(s_unnamed_19 = "01110111") & bool2stdlogic(s_unnamed_19 = "01110110") & bool2stdlogic(s_unnamed_19 = "01110101") & bool2stdlogic(s_unnamed_19 = "01110100") & bool2stdlogic(s_unnamed_19 = "01110011") & bool2stdlogic(s_unnamed_19 = "01110010") & bool2stdlogic(s_unnamed_19 = "01110001") & 
            bool2stdlogic(s_unnamed_19 = "01110000") & bool2stdlogic(s_unnamed_19 = "01101111") & bool2stdlogic(s_unnamed_19 = "01101110") & bool2stdlogic(s_unnamed_19 = "01101101") & bool2stdlogic(s_unnamed_19 = "01101100") & bool2stdlogic(s_unnamed_19 = "01101011") & bool2stdlogic(s_unnamed_19 = "01101010") & bool2stdlogic(s_unnamed_19 = "01101001") & bool2stdlogic(s_unnamed_19 = "01101000") & bool2stdlogic(s_unnamed_19 = "01100111") & bool2stdlogic(s_unnamed_19 = "01100110") & bool2stdlogic(s_unnamed_19 = "01100101") & bool2stdlogic(s_unnamed_19 = "01100100") & bool2stdlogic(s_unnamed_19 = "01100011") & bool2stdlogic(s_unnamed_19 = "01100010") & bool2stdlogic(s_unnamed_19 = "01100001") & 
            bool2stdlogic(s_unnamed_19 = "01100000") & bool2stdlogic(s_unnamed_19 = "01011111") & bool2stdlogic(s_unnamed_19 = "01011110") & bool2stdlogic(s_unnamed_19 = "01011101") & bool2stdlogic(s_unnamed_19 = "01011100") & bool2stdlogic(s_unnamed_19 = "01011011") & bool2stdlogic(s_unnamed_19 = "01011010") & bool2stdlogic(s_unnamed_19 = "01011001") & bool2stdlogic(s_unnamed_19 = "01011000") & bool2stdlogic(s_unnamed_19 = "01010111") & bool2stdlogic(s_unnamed_19 = "01010110") & bool2stdlogic(s_unnamed_19 = "01010101") & bool2stdlogic(s_unnamed_19 = "01010100") & bool2stdlogic(s_unnamed_19 = "01010011") & bool2stdlogic(s_unnamed_19 = "01010010") & bool2stdlogic(s_unnamed_19 = "01010001") & 
            bool2stdlogic(s_unnamed_19 = "01010000") & bool2stdlogic(s_unnamed_19 = "01001111") & bool2stdlogic(s_unnamed_19 = "01001110") & bool2stdlogic(s_unnamed_19 = "01001101") & bool2stdlogic(s_unnamed_19 = "01001100") & bool2stdlogic(s_unnamed_19 = "01001011") & bool2stdlogic(s_unnamed_19 = "01001010") & bool2stdlogic(s_unnamed_19 = "01001001") & bool2stdlogic(s_unnamed_19 = "01001000") & bool2stdlogic(s_unnamed_19 = "01000111") & bool2stdlogic(s_unnamed_19 = "01000110") & bool2stdlogic(s_unnamed_19 = "01000101") & bool2stdlogic(s_unnamed_19 = "01000100") & bool2stdlogic(s_unnamed_19 = "01000011") & bool2stdlogic(s_unnamed_19 = "01000010") & bool2stdlogic(s_unnamed_19 = "01000001") & 
            bool2stdlogic(s_unnamed_19 = "01000000") & bool2stdlogic(s_unnamed_19 = "00111111") & bool2stdlogic(s_unnamed_19 = "00111110") & bool2stdlogic(s_unnamed_19 = "00111101") & bool2stdlogic(s_unnamed_19 = "00111100") & bool2stdlogic(s_unnamed_19 = "00111011") & bool2stdlogic(s_unnamed_19 = "00111010") & bool2stdlogic(s_unnamed_19 = "00111001") & bool2stdlogic(s_unnamed_19 = "00111000") & bool2stdlogic(s_unnamed_19 = "00110111") & bool2stdlogic(s_unnamed_19 = "00110110") & bool2stdlogic(s_unnamed_19 = "00110101") & bool2stdlogic(s_unnamed_19 = "00110100") & bool2stdlogic(s_unnamed_19 = "00110011") & bool2stdlogic(s_unnamed_19 = "00110010") & bool2stdlogic(s_unnamed_19 = "00110001") & 
            bool2stdlogic(s_unnamed_19 = "00110000") & bool2stdlogic(s_unnamed_19 = "00101111") & bool2stdlogic(s_unnamed_19 = "00101110") & bool2stdlogic(s_unnamed_19 = "00101101") & bool2stdlogic(s_unnamed_19 = "00101100") & bool2stdlogic(s_unnamed_19 = "00101011") & bool2stdlogic(s_unnamed_19 = "00101010") & bool2stdlogic(s_unnamed_19 = "00101001") & bool2stdlogic(s_unnamed_19 = "00101000") & bool2stdlogic(s_unnamed_19 = "00100111") & bool2stdlogic(s_unnamed_19 = "00100110") & bool2stdlogic(s_unnamed_19 = "00100101") & bool2stdlogic(s_unnamed_19 = "00100100") & bool2stdlogic(s_unnamed_19 = "00100011") & bool2stdlogic(s_unnamed_19 = "00100010") & bool2stdlogic(s_unnamed_19 = "00100001") & 
            bool2stdlogic(s_unnamed_19 = "00100000") & bool2stdlogic(s_unnamed_19 = "00011111") & bool2stdlogic(s_unnamed_19 = "00011110") & bool2stdlogic(s_unnamed_19 = "00011101") & bool2stdlogic(s_unnamed_19 = "00011100") & bool2stdlogic(s_unnamed_19 = "00011011") & bool2stdlogic(s_unnamed_19 = "00011010") & bool2stdlogic(s_unnamed_19 = "00011001") & bool2stdlogic(s_unnamed_19 = "00011000") & bool2stdlogic(s_unnamed_19 = "00010111") & bool2stdlogic(s_unnamed_19 = "00010110") & bool2stdlogic(s_unnamed_19 = "00010101") & bool2stdlogic(s_unnamed_19 = "00010100") & bool2stdlogic(s_unnamed_19 = "00010011") & bool2stdlogic(s_unnamed_19 = "00010010") & bool2stdlogic(s_unnamed_19 = "00010001") & 
            bool2stdlogic(s_unnamed_19 = "00010000") & bool2stdlogic(s_unnamed_19 = "00001111") & bool2stdlogic(s_unnamed_19 = "00001110") & bool2stdlogic(s_unnamed_19 = "00001101") & bool2stdlogic(s_unnamed_19 = "00001100") & bool2stdlogic(s_unnamed_19 = "00001011") & bool2stdlogic(s_unnamed_19 = "00001010") & bool2stdlogic(s_unnamed_19 = "00001001") & bool2stdlogic(s_unnamed_19 = "00001000") & bool2stdlogic(s_unnamed_19 = "00000111") & bool2stdlogic(s_unnamed_19 = "00000110") & bool2stdlogic(s_unnamed_19 = "00000101") & bool2stdlogic(s_unnamed_19 = "00000100") & bool2stdlogic(s_unnamed_19 = "00000011") & bool2stdlogic(s_unnamed_19 = "00000010") & bool2stdlogic(s_unnamed_19 = "00000001") & 
            bool2stdlogic(s_unnamed_19 = "00000000"));
        s_unnamed_27 <= s_unnamed_3;
        s_unnamed_30 <= s_unnamed_4;
        s_unnamed_28 <= s_unnamed_5;
        s_unnamed_29 <= s_unnamed_6;
        s_unnamed_25 <= s_unnamed_7;
        s_unnamed_24 <= s_unnamed_9;
        s_unnamed_23 <= s_unnamed_13;
        s_unnamed_35 <= s_unnamed_14;
        s_unnamed_31 <= s_unnamed_16;
        s_unnamed_33 <= s_unnamed_8;
        s_unnamed_32 <= s_unnamed_10;
        s_unnamed_34 <= s_unnamed_11;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_unnamed_39 <= s_unnamed_19;
            s_unnamed_38 <= s_unnamed_18;
            out_bitAddr <= s_unnamed_22;
            s_unnamed_37 <= s_unnamed_20;
            s_unnamed_36 <= s_unnamed_21;
            s_unnamed_4 <= s_unnamed;
            s_unnamed_3 <= s_unnamed_2;
            s_unnamed_5 <= s_unnamed_26;
            s_unnamed_6 <= s_unnamed_27;
            s_unnamed_7 <= s_unnamed_30;
            s_unnamed_9 <= s_unnamed_28;
            s_unnamed_16 <= s_unnamed_29;
            s_unnamed_13 <= s_unnamed_25;
            s_unnamed_14 <= s_unnamed_24;
            s_unnamed_10 <= s_unnamed_23;
            s_unnamed_11 <= s_unnamed_35;
            s_unnamed_8 <= s_unnamed_31;
            s_unnamed_17 <= s_unnamed_33;
            s_unnamed_12 <= s_unnamed_32;
            s_unnamed_15 <= s_unnamed_34;
            s_bitmap_next_delayed1 <= out_bitmap_next;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: read_write_hazard_bypass_ringbuffer_Memory_Helper
-- Auto generated to handle various memory access issues such as read during write and read modify write hazards.
------------------------------------------------

ENTITY read_write_hazard_bypass_ringbuffer_Memory_Helper IS 
    PORT(
        clock : IN STD_LOGIC;
        in_ringbuffer_write_pointer : IN UNSIGNED(2 downto 0);
        in_bypass_data_word_0_stage_4 : IN UNSIGNED(2 downto 0);
        out_conflict_bypass_mux : OUT UNSIGNED(255 downto 0);
        in_unnamed : IN UNSIGNED(255 downto 0);
        in_bitmap_next_delayed1 : IN UNSIGNED(255 downto 0)
    );
END read_write_hazard_bypass_ringbuffer_Memory_Helper;

ARCHITECTURE impl OF read_write_hazard_bypass_ringbuffer_Memory_Helper IS 
    SIGNAL s_conflict : STD_LOGIC; 
    SIGNAL s_conflict_delayed1 : STD_LOGIC; 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_conflict_bypass_mux : UNSIGNED(255 downto 0); 
    BEGIN

        -- Compare read and write addr for conflicts
        s_conflict <= bool2stdlogic(in_bypass_data_word_0_stage_4 = in_ringbuffer_write_pointer);

        -- If read and write addr match and read and write are enabled, forward write data to read output.
        IF s_conflict_delayed1 = '1' THEN
            v_conflict_bypass_mux := in_bitmap_next_delayed1;
        ELSE
            v_conflict_bypass_mux := in_unnamed;
        END IF;

        -- If read and write addr match and read and write are enabled, forward write data to read output.
        out_conflict_bypass_mux <= v_conflict_bypass_mux;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_conflict_delayed1 <= s_conflict;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY altera_mf;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: bitmap_Memory_Helper
-- Auto generated to handle various memory access issues such as read during write and read modify write hazards.
------------------------------------------------

ENTITY bitmap_Memory_Helper IS 
    PORT(
        clock : IN STD_LOGIC;
        in_address : IN UNSIGNED(19 downto 0);
        in_bitmap_next : IN UNSIGNED(255 downto 0);
        out_hazard_corrected_data : OUT UNSIGNED(255 downto 0);
        in_unnamed : IN STD_LOGIC;
        in_unnamed_2 : IN UNSIGNED(19 downto 0);
        in_bitmap_next_delayed1 : IN UNSIGNED(255 downto 0);
        in_bitmap_rd_readdata : IN UNSIGNED(255 downto 0)
    );
END bitmap_Memory_Helper;

ARCHITECTURE impl OF bitmap_Memory_Helper IS 
    SIGNAL s_altdpram_q : UNSIGNED(255 downto 0); 
    SIGNAL s_ringbuffer_write_pointer : UNSIGNED(2 downto 0) := "000"; 
    SIGNAL s_ringbuffer_write_pointer_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_address_delayed2 : UNSIGNED(19 downto 0); 
    SIGNAL s_address_delayed3 : UNSIGNED(19 downto 0); 
    SIGNAL s_conflict_word_0_stage_0 : STD_LOGIC; 
    SIGNAL s_address_delayed1 : UNSIGNED(19 downto 0); 
    SIGNAL s_unnamed : UNSIGNED(19 downto 0); 
    SIGNAL s_final_conflict_word_0 : STD_LOGIC; 
    SIGNAL s_final_conflict_word_0_delayed1 : STD_LOGIC; 
    SIGNAL s_conflict_word_0_stage_1_delayed1 : STD_LOGIC; 
    SIGNAL s_conflict_word_0_stage_0_delayed1 : STD_LOGIC; 
    SIGNAL s_bypass_data_word_0_stage_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_conflict_word_0_stage_2_delayed1 : STD_LOGIC; 
    SIGNAL s_bypass_data_word_0_stage_1_delayed1 : UNSIGNED(2 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_1 : UNSIGNED(2 downto 0); 
    SIGNAL s_conflict_word_0_stage_1 : STD_LOGIC; 
    SIGNAL s_conflict_word_0_stage_2 : STD_LOGIC; 
    SIGNAL s_conflict_word_0_stage_3 : STD_LOGIC; 
    SIGNAL s_bypass_data_word_0_stage_3 : UNSIGNED(2 downto 0); 
    SIGNAL s_conflict_word_0_stage_3_delayed1 : STD_LOGIC; 
    SIGNAL s_bypass_data_word_0_stage_3_delayed1 : UNSIGNED(2 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_2_delayed1 : UNSIGNED(2 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_0 : UNSIGNED(2 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_0_delayed1 : UNSIGNED(2 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_4 : UNSIGNED(2 downto 0); 
    SIGNAL s_conflict_bypass_mux : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed_2 : UNSIGNED(255 downto 0); 
    SIGNAL s_const_1 : STD_LOGIC; 
    SIGNAL s_unnamed_3 : UNSIGNED(19 downto 0); 
    SIGNAL s_unnamed_4 : UNSIGNED(2 downto 0); 
    SIGNAL s_address_delayed1_2 : UNSIGNED(19 downto 0); 
    SIGNAL s_address_delayed2_2 : UNSIGNED(19 downto 0); 
BEGIN
    altdpram : altera_mf.altera_mf_components.altdpram
     generic map (
        INDATA_ACLR => "OFF",
        INDATA_REG => "INCLOCK",
        OUTDATA_ACLR => "OFF",
        OUTDATA_REG => "OUTCLOCK",
        RDADDRESS_ACLR => "OFF",
        RDADDRESS_REG => "UNREGISTERED",
        RDCONTROL_ACLR => "OFF",
        RDCONTROL_REG => "UNREGISTERED",
        WIDTHAD => 3,
        WRADDRESS_ACLR => "OFF",
        WRADDRESS_REG => "INCLOCK",
        WRCONTROL_ACLR => "OFF",
        WRCONTROL_REG => "INCLOCK",
        intended_device_family => "Arria 10",
        numwords => 8,
        ram_block_type => "MLAB",
        read_during_write_mode_mixed_ports => "NEW_DATA",
        width => 256
    )
     port map (
        inclock => clock,
        outclock => clock,
        wren => s_const_1,
        data => STD_LOGIC_VECTOR(in_bitmap_next),
        rdaddress => STD_LOGIC_VECTOR(s_bypass_data_word_0_stage_4),
        wraddress => STD_LOGIC_VECTOR(s_ringbuffer_write_pointer_2),
        UNSIGNED(q) => s_altdpram_q
    );
    read_write_hazard_bypass_ringbuffer_Memory_Helper0 : entity work.read_write_hazard_bypass_ringbuffer_Memory_Helper(impl) port map (
        clock => clock,
        in_ringbuffer_write_pointer => s_ringbuffer_write_pointer_2,
        in_bypass_data_word_0_stage_4 => s_bypass_data_word_0_stage_4,
        in_unnamed => s_unnamed_2,
        in_bitmap_next_delayed1 => in_bitmap_next_delayed1,
        out_conflict_bypass_mux => s_conflict_bypass_mux
    );
    default_comb : PROCESS(all)
        VARIABLE v_hazard_corrected_data : UNSIGNED(255 downto 0); 
        VARIABLE v_bypass_data_word_0_stage_2 : UNSIGNED(2 downto 0); 
        VARIABLE v_bypass_data_word_0_stage_3 : UNSIGNED(2 downto 0); 
        VARIABLE v_bypass_data_word_0_stage_1 : UNSIGNED(2 downto 0); 
        VARIABLE v_bypass_data_word_0_stage_4 : UNSIGNED(2 downto 0); 
        VARIABLE v_conflict_word_0_stage_4 : STD_LOGIC; 
        VARIABLE v_final_override_data_word_0 : UNSIGNED(255 downto 0); 
    BEGIN
        s_ringbuffer_write_pointer_2 <= s_ringbuffer_write_pointer;

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_0 <= (bool2stdlogic(in_address = in_unnamed_2) and in_unnamed);
        s_bypass_data_word_0_stage_0 <= s_ringbuffer_write_pointer_2;

        -- Compare read and write addr for conflicts
        IF ((s_unnamed = in_unnamed_2) and in_unnamed = '1') THEN
            v_bypass_data_word_0_stage_4 := s_ringbuffer_write_pointer_2;
        ELSE
            v_bypass_data_word_0_stage_4 := s_bypass_data_word_0_stage_3_delayed1;
        END IF;

        -- Compare read and write addr for conflicts
        v_conflict_word_0_stage_4 := (s_conflict_word_0_stage_3_delayed1 or (bool2stdlogic(s_unnamed = in_unnamed_2) and in_unnamed));
        s_bypass_data_word_0_stage_4 <= v_bypass_data_word_0_stage_4;
        s_final_conflict_word_0 <= v_conflict_word_0_stage_4;
        v_final_override_data_word_0 := s_conflict_bypass_mux;

        -- If read and write addr match and read and write are enabled and write is not masked, forward write data to read output.
        IF s_final_conflict_word_0_delayed1 = '1' THEN
            v_hazard_corrected_data := v_final_override_data_word_0;
        ELSE
            v_hazard_corrected_data := in_bitmap_rd_readdata;
        END IF;

        -- If read and write addr match and read and write are enabled and write is not masked, forward write data to read output.
        out_hazard_corrected_data <= v_hazard_corrected_data;
        s_unnamed_2 <= s_altdpram_q;
        s_unnamed_3 <= s_address_delayed3;

        -- Compare read and write addr for conflicts
        IF ((s_unnamed_3 = in_unnamed_2) and in_unnamed = '1') THEN
            v_bypass_data_word_0_stage_3 := s_ringbuffer_write_pointer_2;
        ELSE
            v_bypass_data_word_0_stage_3 := s_bypass_data_word_0_stage_2_delayed1;
        END IF;

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_3 <= (s_conflict_word_0_stage_2_delayed1 or (bool2stdlogic(s_unnamed_3 = in_unnamed_2) and in_unnamed));
        s_bypass_data_word_0_stage_3 <= v_bypass_data_word_0_stage_3;
        s_const_1 <= '1';
        s_unnamed_4 <= (s_ringbuffer_write_pointer_2 + "001");
        s_address_delayed1_2 <= s_address_delayed1;

        -- Compare read and write addr for conflicts
        IF ((s_address_delayed1_2 = in_unnamed_2) and in_unnamed = '1') THEN
            v_bypass_data_word_0_stage_1 := s_ringbuffer_write_pointer_2;
        ELSE
            v_bypass_data_word_0_stage_1 := s_bypass_data_word_0_stage_0_delayed1;
        END IF;

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_1 <= (s_conflict_word_0_stage_0_delayed1 or (bool2stdlogic(s_address_delayed1_2 = in_unnamed_2) and in_unnamed));
        s_bypass_data_word_0_stage_1 <= v_bypass_data_word_0_stage_1;
        s_address_delayed2_2 <= s_address_delayed2;

        -- Compare read and write addr for conflicts
        IF ((s_address_delayed2_2 = in_unnamed_2) and in_unnamed = '1') THEN
            v_bypass_data_word_0_stage_2 := s_ringbuffer_write_pointer_2;
        ELSE
            v_bypass_data_word_0_stage_2 := s_bypass_data_word_0_stage_1_delayed1;
        END IF;

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_2 <= (s_conflict_word_0_stage_1_delayed1 or (bool2stdlogic(s_address_delayed2_2 = in_unnamed_2) and in_unnamed));
        s_bypass_data_word_0_stage_2 <= v_bypass_data_word_0_stage_2;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_ringbuffer_write_pointer <= s_unnamed_4;
            s_address_delayed1 <= in_address;
            s_address_delayed2 <= s_address_delayed1_2;
            s_address_delayed3 <= s_address_delayed2_2;
            s_unnamed <= s_unnamed_3;
            s_conflict_word_0_stage_0_delayed1 <= s_conflict_word_0_stage_0;
            s_bypass_data_word_0_stage_0_delayed1 <= s_bypass_data_word_0_stage_0;
            s_conflict_word_0_stage_1_delayed1 <= s_conflict_word_0_stage_1;
            s_bypass_data_word_0_stage_1_delayed1 <= s_bypass_data_word_0_stage_1;
            s_conflict_word_0_stage_2_delayed1 <= s_conflict_word_0_stage_2;
            s_bypass_data_word_0_stage_2_delayed1 <= s_bypass_data_word_0_stage_2;
            s_conflict_word_0_stage_3_delayed1 <= s_conflict_word_0_stage_3;
            s_bypass_data_word_0_stage_3_delayed1 <= s_bypass_data_word_0_stage_3;
            s_final_conflict_word_0_delayed1 <= s_final_conflict_word_0;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_memory_5
-- 
------------------------------------------------

ENTITY scl_memory_5 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_address : IN UNSIGNED(19 downto 0);
        in_bitmap_next : IN UNSIGNED(255 downto 0);
        out_hazard_corrected_data : OUT UNSIGNED(255 downto 0);
        bitmap_rd_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_rd_read : OUT STD_LOGIC;
        bitmap_rd_readdata : IN STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_wr_writedata : OUT STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_write : OUT STD_LOGIC;
        in_m_cmd_valid_delayed9 : IN STD_LOGIC;
        in_unnamed : IN STD_LOGIC;
        in_unnamed_2 : IN UNSIGNED(19 downto 0);
        in_bitmap_next_delayed1 : IN UNSIGNED(255 downto 0)
    );
END scl_memory_5;

ARCHITECTURE impl OF scl_memory_5 IS 
    SIGNAL s_bitmap_rd_readdata : UNSIGNED(255 downto 0); 
BEGIN
    bitmap_Memory_Helper0 : entity work.bitmap_Memory_Helper(impl) port map (
        clock => clock,
        in_address => in_address,
        in_bitmap_next => in_bitmap_next,
        in_unnamed => in_unnamed,
        in_unnamed_2 => in_unnamed_2,
        in_bitmap_next_delayed1 => in_bitmap_next_delayed1,
        in_bitmap_rd_readdata => s_bitmap_rd_readdata,
        out_hazard_corrected_data => out_hazard_corrected_data
    );
    default_comb : PROCESS(all)
    BEGIN
        bitmap_rd_address <= STD_LOGIC_VECTOR(in_address);
        bitmap_rd_read <= in_m_cmd_valid_delayed9;
        bitmap_wr_address <= STD_LOGIC_VECTOR(in_unnamed_2);
        bitmap_wr_writedata <= STD_LOGIC_VECTOR(in_bitmap_next);
        bitmap_wr_write <= in_unnamed;
        s_bitmap_rd_readdata <= UNSIGNED(bitmap_rd_readdata);
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: SecondLookupStage
-- 
------------------------------------------------

ENTITY SecondLookupStage IS 
    PORT(
        clock : IN STD_LOGIC;
        in_secondStageValid : IN STD_LOGIC;
        out_bitmap : OUT UNSIGNED(255 downto 0);
        out_bitAddr : OUT UNSIGNED(7 downto 0);
        in_updateState : IN STD_LOGIC;
        bitmap_rd_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_rd_read : OUT STD_LOGIC;
        bitmap_rd_readdata : IN STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_wr_writedata : OUT STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_write : OUT STD_LOGIC;
        in_aboveWindow_delayed3 : IN STD_LOGIC;
        in_lastSequenceNumber_delayed3 : IN UNSIGNED(63 downto 0);
        in_m_cmd_valid_delayed9 : IN STD_LOGIC;
        in_unnamed : IN UNSIGNED(63 downto 0);
        in_unnamed_2 : IN UNSIGNED(13 downto 0);
        in_unnamed_3 : IN UNSIGNED(2 downto 0);
        in_unnamed_4 : IN UNSIGNED(5 downto 0);
        in_unnamed_5 : IN STD_LOGIC
    );
END SecondLookupStage;

ARCHITECTURE impl OF SecondLookupStage IS 
    SIGNAL s_address : UNSIGNED(19 downto 0); 
    SIGNAL s_bitmap : UNSIGNED(255 downto 0); 
    SIGNAL s_secondStageValid : UNSIGNED(255 downto 0); 
    SIGNAL s_address_delayed1 : UNSIGNED(19 downto 0); 
    SIGNAL s_secondStageValid_delayed1 : UNSIGNED(255 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed1 : STD_LOGIC := '0'; 
    SIGNAL s_address_delayed2 : UNSIGNED(19 downto 0); 
    SIGNAL s_secondStageValid_delayed2 : UNSIGNED(255 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed2 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed5 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed3 : STD_LOGIC := '0'; 
    SIGNAL s_address_delayed5 : UNSIGNED(19 downto 0); 
    SIGNAL s_address_delayed4 : UNSIGNED(19 downto 0); 
    SIGNAL s_secondStageValid_delayed5 : UNSIGNED(255 downto 0); 
    SIGNAL s_secondStageValid_delayed4 : UNSIGNED(255 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed4 : STD_LOGIC := '0'; 
    SIGNAL s_secondStageValid_delayed3 : UNSIGNED(255 downto 0); 
    SIGNAL s_address_delayed3 : UNSIGNED(19 downto 0); 
    SIGNAL s_hazard_corrected_data : UNSIGNED(255 downto 0); 
    SIGNAL s_bitmap_next : UNSIGNED(255 downto 0); 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState : STD_LOGIC; 
    SIGNAL s_unnamed_2 : UNSIGNED(19 downto 0); 
    SIGNAL s_bitmap_next_delayed1 : UNSIGNED(255 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed3_2 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed1_2 : UNSIGNED(255 downto 0); 
    SIGNAL s_address_delayed2_2 : UNSIGNED(19 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed2_2 : STD_LOGIC; 
    SIGNAL s_address_delayed1_2 : UNSIGNED(19 downto 0); 
    SIGNAL s_secondStageValid_delayed2_2 : UNSIGNED(255 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed1_2 : STD_LOGIC; 
    SIGNAL s_address_delayed3_2 : UNSIGNED(19 downto 0); 
    SIGNAL s_address_delayed4_2 : UNSIGNED(19 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9_and_updateState_delayed4_2 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed4_2 : UNSIGNED(255 downto 0); 
    SIGNAL s_secondStageValid_delayed3_2 : UNSIGNED(255 downto 0); 
BEGIN
    scl_memory0 : entity work.scl_memory_5(impl) port map (
        clock => clock,
        bitmap_wr_writedata => bitmap_wr_writedata,
        bitmap_rd_readdata => bitmap_rd_readdata,
        bitmap_wr_address => bitmap_wr_address,
        bitmap_rd_address => bitmap_rd_address,
        bitmap_rd_read => bitmap_rd_read,
        bitmap_wr_write => bitmap_wr_write,
        in_address => s_address,
        in_bitmap_next => s_bitmap_next,
        in_m_cmd_valid_delayed9 => in_m_cmd_valid_delayed9,
        in_unnamed => s_unnamed,
        in_unnamed_2 => s_unnamed_2,
        in_bitmap_next_delayed1 => s_bitmap_next_delayed1,
        out_hazard_corrected_data => s_hazard_corrected_data
    );
    default_comb : PROCESS(all)
        VARIABLE v_secondStageValid : UNSIGNED(0 downto 0); 
        VARIABLE v_updateState : STD_LOGIC; 
        VARIABLE v_bitmap : UNSIGNED(255 downto 0); 
    BEGIN
        s_address <= (in_unnamed_2 & in_unnamed_4);
        IF in_unnamed_5 = '1' THEN
            v_bitmap := s_hazard_corrected_data;
        ELSE
            v_bitmap := "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX";
        END IF;
        out_bitmap <= v_bitmap;
        v_updateState := in_updateState;
        v_secondStageValid := (0 => in_secondStageValid);
        s_secondStageValid <= (v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & v_secondStageValid & 
            v_secondStageValid);
        s_bitmap <= (out_bitmap and s_secondStageValid_delayed5);
        s_unnamed <= s_m_cmd_valid_delayed9_and_updateState_delayed5;
        s_m_cmd_valid_delayed9_and_updateState <= (in_m_cmd_valid_delayed9 and v_updateState);
        s_unnamed_2 <= s_address_delayed5;
        s_address_delayed1_2 <= s_address_delayed1;
        s_secondStageValid_delayed1_2 <= s_secondStageValid_delayed1;
        s_m_cmd_valid_delayed9_and_updateState_delayed1_2 <= s_m_cmd_valid_delayed9_and_updateState_delayed1;
        s_address_delayed2_2 <= s_address_delayed2;
        s_secondStageValid_delayed2_2 <= s_secondStageValid_delayed2;
        s_m_cmd_valid_delayed9_and_updateState_delayed2_2 <= s_m_cmd_valid_delayed9_and_updateState_delayed2;
        s_m_cmd_valid_delayed9_and_updateState_delayed3_2 <= s_m_cmd_valid_delayed9_and_updateState_delayed3;
        s_secondStageValid_delayed3_2 <= s_secondStageValid_delayed3;
        s_address_delayed3_2 <= s_address_delayed3;
        s_address_delayed4_2 <= s_address_delayed4;
        s_secondStageValid_delayed4_2 <= s_secondStageValid_delayed4;
        s_m_cmd_valid_delayed9_and_updateState_delayed4_2 <= s_m_cmd_valid_delayed9_and_updateState_delayed4;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_address_delayed5 <= s_address_delayed4_2;
            s_secondStageValid_delayed5 <= s_secondStageValid_delayed4_2;
            s_m_cmd_valid_delayed9_and_updateState_delayed5 <= s_m_cmd_valid_delayed9_and_updateState_delayed4_2;
            s_address_delayed4 <= s_address_delayed3_2;
            s_address_delayed1 <= s_address;
            s_secondStageValid_delayed1 <= s_secondStageValid;
            s_m_cmd_valid_delayed9_and_updateState_delayed1 <= s_m_cmd_valid_delayed9_and_updateState;
            s_address_delayed2 <= s_address_delayed1_2;
            s_secondStageValid_delayed2 <= s_secondStageValid_delayed1_2;
            s_m_cmd_valid_delayed9_and_updateState_delayed2 <= s_m_cmd_valid_delayed9_and_updateState_delayed1_2;
            s_address_delayed3 <= s_address_delayed2_2;
            s_secondStageValid_delayed3 <= s_secondStageValid_delayed2_2;
            s_m_cmd_valid_delayed9_and_updateState_delayed3 <= s_m_cmd_valid_delayed9_and_updateState_delayed2_2;
            s_m_cmd_valid_delayed9_and_updateState_delayed4 <= s_m_cmd_valid_delayed9_and_updateState_delayed3_2;
            s_secondStageValid_delayed4 <= s_secondStageValid_delayed3_2;
        END IF;
    END PROCESS;

    secondStageUpdate0 : entity work.secondStageUpdate(impl) port map (
        clock => clock,
        in_bitmap => s_bitmap,
        in_aboveWindow_delayed3 => in_aboveWindow_delayed3,
        in_unnamed => in_unnamed,
        in_lastSequenceNumber_delayed3 => in_lastSequenceNumber_delayed3,
        in_unnamed_2 => in_unnamed_3,
        in_m_cmd_valid_delayed9_and_updateState => s_m_cmd_valid_delayed9_and_updateState,
        out_bitmap_next => s_bitmap_next,
        out_bitAddr => out_bitAddr,
        out_bitmap_next_delayed1 => s_bitmap_next_delayed1
    );
END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: ringBufferMask
-- 
------------------------------------------------

ENTITY ringBufferMask IS 
    PORT(
        in_bitAddr : IN UNSIGNED(5 downto 0);
        in_get : IN UNSIGNED(5 downto 0);
        out_mask : OUT UNSIGNED(63 downto 0);
        in_unnamed : IN STD_LOGIC
    );
END ringBufferMask;

ARCHITECTURE impl OF ringBufferMask IS 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_put : UNSIGNED(5 downto 0); 
        VARIABLE v_get : UNSIGNED(5 downto 0); 
        VARIABLE v_overrun : STD_LOGIC; 
        VARIABLE v_const_0_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_0_or_put_gt_const_0_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_2 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1_or_put_gt_const_1_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_3 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_4 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10_or_put_gt_const_10_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11_or_put_gt_const_11_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100_or_put_gt_const_100_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_5 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_6 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110_or_put_gt_const_110_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_7 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101_or_put_gt_const_101_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1000_or_put_gt_const_1000_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_8 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_9 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111_or_put_gt_const_111_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_10 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_11 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1001_or_put_gt_const_1001_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_12 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1011_or_put_gt_const_1011_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_13 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1010_or_put_gt_const_1010_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_14 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_15 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1110_or_put_gt_const_1110_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1101_or_put_gt_const_1101_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1100_or_put_gt_const_1100_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_16 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_17 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10001_or_put_gt_const_10001_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_18 : STD_LOGIC; 
        VARIABLE v_get_lt_const_1111_or_put_gt_const_1111_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10000_or_put_gt_const_10000_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_19 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10010_or_put_gt_const_10010_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10011_or_put_gt_const_10011_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_20 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_21 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10100_or_put_gt_const_10100_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10110_or_put_gt_const_10110_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_22 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10101_or_put_gt_const_10101_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_23 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_24 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_25 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_26 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11001_or_put_gt_const_11001_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_10111_or_put_gt_const_10111_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11000_or_put_gt_const_11000_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_27 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_28 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11011_or_put_gt_const_11011_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11010_or_put_gt_const_11010_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_29 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_30 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11110_or_put_gt_const_11110_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_31 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11101_or_put_gt_const_11101_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11100_or_put_gt_const_11100_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_32 : STD_LOGIC; 
        VARIABLE v_get_lt_const_11111_or_put_gt_const_11111_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_33 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100000_or_put_gt_const_100000_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100001_or_put_gt_const_100001_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_34 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100010_or_put_gt_const_100010_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_35 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_36 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100100_or_put_gt_const_100100_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_37 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100011_or_put_gt_const_100011_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_38 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100110_or_put_gt_const_100110_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100101_or_put_gt_const_100101_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_39 : STD_LOGIC; 
        VARIABLE v_get_lt_const_100111_or_put_gt_const_100111_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_40 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_41 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_42 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_43 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101000_or_put_gt_const_101000_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101010_or_put_gt_const_101010_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101001_or_put_gt_const_101001_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_44 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101011_or_put_gt_const_101011_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_45 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_46 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110111_or_put_gt_const_110111_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_47 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_48 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110110_or_put_gt_const_110110_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_49 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101100_or_put_gt_const_101100_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_50 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101110_or_put_gt_const_101110_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101101_or_put_gt_const_101101_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_51 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_52 : STD_LOGIC; 
        VARIABLE v_get_lt_const_101111_or_put_gt_const_101111_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110000_or_put_gt_const_110000_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_53 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110001_or_put_gt_const_110001_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_54 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_55 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110011_or_put_gt_const_110011_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110010_or_put_gt_const_110010_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_56 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110101_or_put_gt_const_110101_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_110100_or_put_gt_const_110100_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_57 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111001_or_put_gt_const_111001_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_58 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111010_or_put_gt_const_111010_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111000_or_put_gt_const_111000_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_59 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111011_or_put_gt_const_111011_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111100_or_put_gt_const_111100_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_60 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111101_or_put_gt_const_111101_mux1 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_61 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_62 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_63 : STD_LOGIC; 
        VARIABLE v_const_0_mux1_64 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111111_or_put_gt_const_111111_mux1 : STD_LOGIC; 
        VARIABLE v_get_lt_const_111110_or_put_gt_const_111110_mux1 : STD_LOGIC; 
    BEGIN
        v_put := in_bitAddr;
        v_get := in_get;
        v_overrun := bool2stdlogic(v_get > v_put);
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1 := (bool2stdlogic(v_get < "000000") and bool2stdlogic(v_put > "000000"));
        ELSE
            v_const_0_mux1 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_0_or_put_gt_const_0_mux1 := (bool2stdlogic(v_get < "000000") or bool2stdlogic(v_put > "000000"));
        ELSE
            v_get_lt_const_0_or_put_gt_const_0_mux1 := v_const_0_mux1;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_3 := (bool2stdlogic(v_get < "000001") and bool2stdlogic(v_put > "000001"));
        ELSE
            v_const_0_mux1_3 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1_or_put_gt_const_1_mux1 := (bool2stdlogic(v_get < "000001") or bool2stdlogic(v_put > "000001"));
        ELSE
            v_get_lt_const_1_or_put_gt_const_1_mux1 := v_const_0_mux1_3;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_2 := (bool2stdlogic(v_get < "000010") and bool2stdlogic(v_put > "000010"));
        ELSE
            v_const_0_mux1_2 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10_or_put_gt_const_10_mux1 := (bool2stdlogic(v_get < "000010") or bool2stdlogic(v_put > "000010"));
        ELSE
            v_get_lt_const_10_or_put_gt_const_10_mux1 := v_const_0_mux1_2;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_4 := (bool2stdlogic(v_get < "000011") and bool2stdlogic(v_put > "000011"));
        ELSE
            v_const_0_mux1_4 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11_or_put_gt_const_11_mux1 := (bool2stdlogic(v_get < "000011") or bool2stdlogic(v_put > "000011"));
        ELSE
            v_get_lt_const_11_or_put_gt_const_11_mux1 := v_const_0_mux1_4;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_6 := (bool2stdlogic(v_get < "000100") and bool2stdlogic(v_put > "000100"));
        ELSE
            v_const_0_mux1_6 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100_or_put_gt_const_100_mux1 := (bool2stdlogic(v_get < "000100") or bool2stdlogic(v_put > "000100"));
        ELSE
            v_get_lt_const_100_or_put_gt_const_100_mux1 := v_const_0_mux1_6;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_7 := (bool2stdlogic(v_get < "000101") and bool2stdlogic(v_put > "000101"));
        ELSE
            v_const_0_mux1_7 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101_or_put_gt_const_101_mux1 := (bool2stdlogic(v_get < "000101") or bool2stdlogic(v_put > "000101"));
        ELSE
            v_get_lt_const_101_or_put_gt_const_101_mux1 := v_const_0_mux1_7;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_5 := (bool2stdlogic(v_get < "000110") and bool2stdlogic(v_put > "000110"));
        ELSE
            v_const_0_mux1_5 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110_or_put_gt_const_110_mux1 := (bool2stdlogic(v_get < "000110") or bool2stdlogic(v_put > "000110"));
        ELSE
            v_get_lt_const_110_or_put_gt_const_110_mux1 := v_const_0_mux1_5;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_8 := (bool2stdlogic(v_get < "000111") and bool2stdlogic(v_put > "000111"));
        ELSE
            v_const_0_mux1_8 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111_or_put_gt_const_111_mux1 := (bool2stdlogic(v_get < "000111") or bool2stdlogic(v_put > "000111"));
        ELSE
            v_get_lt_const_111_or_put_gt_const_111_mux1 := v_const_0_mux1_8;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_10 := (bool2stdlogic(v_get < "001000") and bool2stdlogic(v_put > "001000"));
        ELSE
            v_const_0_mux1_10 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1000_or_put_gt_const_1000_mux1 := (bool2stdlogic(v_get < "001000") or bool2stdlogic(v_put > "001000"));
        ELSE
            v_get_lt_const_1000_or_put_gt_const_1000_mux1 := v_const_0_mux1_10;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_11 := (bool2stdlogic(v_get < "001001") and bool2stdlogic(v_put > "001001"));
        ELSE
            v_const_0_mux1_11 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1001_or_put_gt_const_1001_mux1 := (bool2stdlogic(v_get < "001001") or bool2stdlogic(v_put > "001001"));
        ELSE
            v_get_lt_const_1001_or_put_gt_const_1001_mux1 := v_const_0_mux1_11;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_12 := (bool2stdlogic(v_get < "001010") and bool2stdlogic(v_put > "001010"));
        ELSE
            v_const_0_mux1_12 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1010_or_put_gt_const_1010_mux1 := (bool2stdlogic(v_get < "001010") or bool2stdlogic(v_put > "001010"));
        ELSE
            v_get_lt_const_1010_or_put_gt_const_1010_mux1 := v_const_0_mux1_12;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_13 := (bool2stdlogic(v_get < "001011") and bool2stdlogic(v_put > "001011"));
        ELSE
            v_const_0_mux1_13 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1011_or_put_gt_const_1011_mux1 := (bool2stdlogic(v_get < "001011") or bool2stdlogic(v_put > "001011"));
        ELSE
            v_get_lt_const_1011_or_put_gt_const_1011_mux1 := v_const_0_mux1_13;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_16 := (bool2stdlogic(v_get < "001100") and bool2stdlogic(v_put > "001100"));
        ELSE
            v_const_0_mux1_16 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1100_or_put_gt_const_1100_mux1 := (bool2stdlogic(v_get < "001100") or bool2stdlogic(v_put > "001100"));
        ELSE
            v_get_lt_const_1100_or_put_gt_const_1100_mux1 := v_const_0_mux1_16;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_14 := (bool2stdlogic(v_get < "001101") and bool2stdlogic(v_put > "001101"));
        ELSE
            v_const_0_mux1_14 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1101_or_put_gt_const_1101_mux1 := (bool2stdlogic(v_get < "001101") or bool2stdlogic(v_put > "001101"));
        ELSE
            v_get_lt_const_1101_or_put_gt_const_1101_mux1 := v_const_0_mux1_14;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_15 := (bool2stdlogic(v_get < "001110") and bool2stdlogic(v_put > "001110"));
        ELSE
            v_const_0_mux1_15 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1110_or_put_gt_const_1110_mux1 := (bool2stdlogic(v_get < "001110") or bool2stdlogic(v_put > "001110"));
        ELSE
            v_get_lt_const_1110_or_put_gt_const_1110_mux1 := v_const_0_mux1_15;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_9 := (bool2stdlogic(v_get < "001111") and bool2stdlogic(v_put > "001111"));
        ELSE
            v_const_0_mux1_9 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_1111_or_put_gt_const_1111_mux1 := (bool2stdlogic(v_get < "001111") or bool2stdlogic(v_put > "001111"));
        ELSE
            v_get_lt_const_1111_or_put_gt_const_1111_mux1 := v_const_0_mux1_9;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_18 := (bool2stdlogic(v_get < "010000") and bool2stdlogic(v_put > "010000"));
        ELSE
            v_const_0_mux1_18 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10000_or_put_gt_const_10000_mux1 := (bool2stdlogic(v_get < "010000") or bool2stdlogic(v_put > "010000"));
        ELSE
            v_get_lt_const_10000_or_put_gt_const_10000_mux1 := v_const_0_mux1_18;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_17 := (bool2stdlogic(v_get < "010001") and bool2stdlogic(v_put > "010001"));
        ELSE
            v_const_0_mux1_17 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10001_or_put_gt_const_10001_mux1 := (bool2stdlogic(v_get < "010001") or bool2stdlogic(v_put > "010001"));
        ELSE
            v_get_lt_const_10001_or_put_gt_const_10001_mux1 := v_const_0_mux1_17;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_19 := (bool2stdlogic(v_get < "010010") and bool2stdlogic(v_put > "010010"));
        ELSE
            v_const_0_mux1_19 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10010_or_put_gt_const_10010_mux1 := (bool2stdlogic(v_get < "010010") or bool2stdlogic(v_put > "010010"));
        ELSE
            v_get_lt_const_10010_or_put_gt_const_10010_mux1 := v_const_0_mux1_19;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_20 := (bool2stdlogic(v_get < "010011") and bool2stdlogic(v_put > "010011"));
        ELSE
            v_const_0_mux1_20 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10011_or_put_gt_const_10011_mux1 := (bool2stdlogic(v_get < "010011") or bool2stdlogic(v_put > "010011"));
        ELSE
            v_get_lt_const_10011_or_put_gt_const_10011_mux1 := v_const_0_mux1_20;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_21 := (bool2stdlogic(v_get < "010100") and bool2stdlogic(v_put > "010100"));
        ELSE
            v_const_0_mux1_21 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10100_or_put_gt_const_10100_mux1 := (bool2stdlogic(v_get < "010100") or bool2stdlogic(v_put > "010100"));
        ELSE
            v_get_lt_const_10100_or_put_gt_const_10100_mux1 := v_const_0_mux1_21;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_23 := (bool2stdlogic(v_get < "010101") and bool2stdlogic(v_put > "010101"));
        ELSE
            v_const_0_mux1_23 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10101_or_put_gt_const_10101_mux1 := (bool2stdlogic(v_get < "010101") or bool2stdlogic(v_put > "010101"));
        ELSE
            v_get_lt_const_10101_or_put_gt_const_10101_mux1 := v_const_0_mux1_23;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_24 := (bool2stdlogic(v_get < "010110") and bool2stdlogic(v_put > "010110"));
        ELSE
            v_const_0_mux1_24 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10110_or_put_gt_const_10110_mux1 := (bool2stdlogic(v_get < "010110") or bool2stdlogic(v_put > "010110"));
        ELSE
            v_get_lt_const_10110_or_put_gt_const_10110_mux1 := v_const_0_mux1_24;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_22 := (bool2stdlogic(v_get < "010111") and bool2stdlogic(v_put > "010111"));
        ELSE
            v_const_0_mux1_22 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_10111_or_put_gt_const_10111_mux1 := (bool2stdlogic(v_get < "010111") or bool2stdlogic(v_put > "010111"));
        ELSE
            v_get_lt_const_10111_or_put_gt_const_10111_mux1 := v_const_0_mux1_22;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_26 := (bool2stdlogic(v_get < "011000") and bool2stdlogic(v_put > "011000"));
        ELSE
            v_const_0_mux1_26 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11000_or_put_gt_const_11000_mux1 := (bool2stdlogic(v_get < "011000") or bool2stdlogic(v_put > "011000"));
        ELSE
            v_get_lt_const_11000_or_put_gt_const_11000_mux1 := v_const_0_mux1_26;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_25 := (bool2stdlogic(v_get < "011001") and bool2stdlogic(v_put > "011001"));
        ELSE
            v_const_0_mux1_25 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11001_or_put_gt_const_11001_mux1 := (bool2stdlogic(v_get < "011001") or bool2stdlogic(v_put > "011001"));
        ELSE
            v_get_lt_const_11001_or_put_gt_const_11001_mux1 := v_const_0_mux1_25;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_27 := (bool2stdlogic(v_get < "011010") and bool2stdlogic(v_put > "011010"));
        ELSE
            v_const_0_mux1_27 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11010_or_put_gt_const_11010_mux1 := (bool2stdlogic(v_get < "011010") or bool2stdlogic(v_put > "011010"));
        ELSE
            v_get_lt_const_11010_or_put_gt_const_11010_mux1 := v_const_0_mux1_27;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_28 := (bool2stdlogic(v_get < "011011") and bool2stdlogic(v_put > "011011"));
        ELSE
            v_const_0_mux1_28 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11011_or_put_gt_const_11011_mux1 := (bool2stdlogic(v_get < "011011") or bool2stdlogic(v_put > "011011"));
        ELSE
            v_get_lt_const_11011_or_put_gt_const_11011_mux1 := v_const_0_mux1_28;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_29 := (bool2stdlogic(v_get < "011100") and bool2stdlogic(v_put > "011100"));
        ELSE
            v_const_0_mux1_29 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11100_or_put_gt_const_11100_mux1 := (bool2stdlogic(v_get < "011100") or bool2stdlogic(v_put > "011100"));
        ELSE
            v_get_lt_const_11100_or_put_gt_const_11100_mux1 := v_const_0_mux1_29;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_31 := (bool2stdlogic(v_get < "011101") and bool2stdlogic(v_put > "011101"));
        ELSE
            v_const_0_mux1_31 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11101_or_put_gt_const_11101_mux1 := (bool2stdlogic(v_get < "011101") or bool2stdlogic(v_put > "011101"));
        ELSE
            v_get_lt_const_11101_or_put_gt_const_11101_mux1 := v_const_0_mux1_31;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_30 := (bool2stdlogic(v_get < "011110") and bool2stdlogic(v_put > "011110"));
        ELSE
            v_const_0_mux1_30 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11110_or_put_gt_const_11110_mux1 := (bool2stdlogic(v_get < "011110") or bool2stdlogic(v_put > "011110"));
        ELSE
            v_get_lt_const_11110_or_put_gt_const_11110_mux1 := v_const_0_mux1_30;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_32 := (bool2stdlogic(v_get < "011111") and bool2stdlogic(v_put > "011111"));
        ELSE
            v_const_0_mux1_32 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_11111_or_put_gt_const_11111_mux1 := (bool2stdlogic(v_get < "011111") or bool2stdlogic(v_put > "011111"));
        ELSE
            v_get_lt_const_11111_or_put_gt_const_11111_mux1 := v_const_0_mux1_32;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_34 := (bool2stdlogic(v_get < "100000") and bool2stdlogic(v_put > "100000"));
        ELSE
            v_const_0_mux1_34 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100000_or_put_gt_const_100000_mux1 := (bool2stdlogic(v_get < "100000") or bool2stdlogic(v_put > "100000"));
        ELSE
            v_get_lt_const_100000_or_put_gt_const_100000_mux1 := v_const_0_mux1_34;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_33 := (bool2stdlogic(v_get < "100001") and bool2stdlogic(v_put > "100001"));
        ELSE
            v_const_0_mux1_33 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100001_or_put_gt_const_100001_mux1 := (bool2stdlogic(v_get < "100001") or bool2stdlogic(v_put > "100001"));
        ELSE
            v_get_lt_const_100001_or_put_gt_const_100001_mux1 := v_const_0_mux1_33;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_35 := (bool2stdlogic(v_get < "100010") and bool2stdlogic(v_put > "100010"));
        ELSE
            v_const_0_mux1_35 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100010_or_put_gt_const_100010_mux1 := (bool2stdlogic(v_get < "100010") or bool2stdlogic(v_put > "100010"));
        ELSE
            v_get_lt_const_100010_or_put_gt_const_100010_mux1 := v_const_0_mux1_35;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_36 := (bool2stdlogic(v_get < "100011") and bool2stdlogic(v_put > "100011"));
        ELSE
            v_const_0_mux1_36 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100011_or_put_gt_const_100011_mux1 := (bool2stdlogic(v_get < "100011") or bool2stdlogic(v_put > "100011"));
        ELSE
            v_get_lt_const_100011_or_put_gt_const_100011_mux1 := v_const_0_mux1_36;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_38 := (bool2stdlogic(v_get < "100100") and bool2stdlogic(v_put > "100100"));
        ELSE
            v_const_0_mux1_38 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100100_or_put_gt_const_100100_mux1 := (bool2stdlogic(v_get < "100100") or bool2stdlogic(v_put > "100100"));
        ELSE
            v_get_lt_const_100100_or_put_gt_const_100100_mux1 := v_const_0_mux1_38;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_37 := (bool2stdlogic(v_get < "100101") and bool2stdlogic(v_put > "100101"));
        ELSE
            v_const_0_mux1_37 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100101_or_put_gt_const_100101_mux1 := (bool2stdlogic(v_get < "100101") or bool2stdlogic(v_put > "100101"));
        ELSE
            v_get_lt_const_100101_or_put_gt_const_100101_mux1 := v_const_0_mux1_37;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_39 := (bool2stdlogic(v_get < "100110") and bool2stdlogic(v_put > "100110"));
        ELSE
            v_const_0_mux1_39 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100110_or_put_gt_const_100110_mux1 := (bool2stdlogic(v_get < "100110") or bool2stdlogic(v_put > "100110"));
        ELSE
            v_get_lt_const_100110_or_put_gt_const_100110_mux1 := v_const_0_mux1_39;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_40 := (bool2stdlogic(v_get < "100111") and bool2stdlogic(v_put > "100111"));
        ELSE
            v_const_0_mux1_40 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_100111_or_put_gt_const_100111_mux1 := (bool2stdlogic(v_get < "100111") or bool2stdlogic(v_put > "100111"));
        ELSE
            v_get_lt_const_100111_or_put_gt_const_100111_mux1 := v_const_0_mux1_40;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_42 := (bool2stdlogic(v_get < "101000") and bool2stdlogic(v_put > "101000"));
        ELSE
            v_const_0_mux1_42 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101000_or_put_gt_const_101000_mux1 := (bool2stdlogic(v_get < "101000") or bool2stdlogic(v_put > "101000"));
        ELSE
            v_get_lt_const_101000_or_put_gt_const_101000_mux1 := v_const_0_mux1_42;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_43 := (bool2stdlogic(v_get < "101001") and bool2stdlogic(v_put > "101001"));
        ELSE
            v_const_0_mux1_43 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101001_or_put_gt_const_101001_mux1 := (bool2stdlogic(v_get < "101001") or bool2stdlogic(v_put > "101001"));
        ELSE
            v_get_lt_const_101001_or_put_gt_const_101001_mux1 := v_const_0_mux1_43;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_41 := (bool2stdlogic(v_get < "101010") and bool2stdlogic(v_put > "101010"));
        ELSE
            v_const_0_mux1_41 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101010_or_put_gt_const_101010_mux1 := (bool2stdlogic(v_get < "101010") or bool2stdlogic(v_put > "101010"));
        ELSE
            v_get_lt_const_101010_or_put_gt_const_101010_mux1 := v_const_0_mux1_41;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_45 := (bool2stdlogic(v_get < "101011") and bool2stdlogic(v_put > "101011"));
        ELSE
            v_const_0_mux1_45 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101011_or_put_gt_const_101011_mux1 := (bool2stdlogic(v_get < "101011") or bool2stdlogic(v_put > "101011"));
        ELSE
            v_get_lt_const_101011_or_put_gt_const_101011_mux1 := v_const_0_mux1_45;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_44 := (bool2stdlogic(v_get < "101100") and bool2stdlogic(v_put > "101100"));
        ELSE
            v_const_0_mux1_44 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101100_or_put_gt_const_101100_mux1 := (bool2stdlogic(v_get < "101100") or bool2stdlogic(v_put > "101100"));
        ELSE
            v_get_lt_const_101100_or_put_gt_const_101100_mux1 := v_const_0_mux1_44;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_50 := (bool2stdlogic(v_get < "101101") and bool2stdlogic(v_put > "101101"));
        ELSE
            v_const_0_mux1_50 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101101_or_put_gt_const_101101_mux1 := (bool2stdlogic(v_get < "101101") or bool2stdlogic(v_put > "101101"));
        ELSE
            v_get_lt_const_101101_or_put_gt_const_101101_mux1 := v_const_0_mux1_50;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_49 := (bool2stdlogic(v_get < "101110") and bool2stdlogic(v_put > "101110"));
        ELSE
            v_const_0_mux1_49 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101110_or_put_gt_const_101110_mux1 := (bool2stdlogic(v_get < "101110") or bool2stdlogic(v_put > "101110"));
        ELSE
            v_get_lt_const_101110_or_put_gt_const_101110_mux1 := v_const_0_mux1_49;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_52 := (bool2stdlogic(v_get < "101111") and bool2stdlogic(v_put > "101111"));
        ELSE
            v_const_0_mux1_52 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_101111_or_put_gt_const_101111_mux1 := (bool2stdlogic(v_get < "101111") or bool2stdlogic(v_put > "101111"));
        ELSE
            v_get_lt_const_101111_or_put_gt_const_101111_mux1 := v_const_0_mux1_52;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_51 := (bool2stdlogic(v_get < "110000") and bool2stdlogic(v_put > "110000"));
        ELSE
            v_const_0_mux1_51 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110000_or_put_gt_const_110000_mux1 := (bool2stdlogic(v_get < "110000") or bool2stdlogic(v_put > "110000"));
        ELSE
            v_get_lt_const_110000_or_put_gt_const_110000_mux1 := v_const_0_mux1_51;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_53 := (bool2stdlogic(v_get < "110001") and bool2stdlogic(v_put > "110001"));
        ELSE
            v_const_0_mux1_53 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110001_or_put_gt_const_110001_mux1 := (bool2stdlogic(v_get < "110001") or bool2stdlogic(v_put > "110001"));
        ELSE
            v_get_lt_const_110001_or_put_gt_const_110001_mux1 := v_const_0_mux1_53;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_54 := (bool2stdlogic(v_get < "110010") and bool2stdlogic(v_put > "110010"));
        ELSE
            v_const_0_mux1_54 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110010_or_put_gt_const_110010_mux1 := (bool2stdlogic(v_get < "110010") or bool2stdlogic(v_put > "110010"));
        ELSE
            v_get_lt_const_110010_or_put_gt_const_110010_mux1 := v_const_0_mux1_54;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_56 := (bool2stdlogic(v_get < "110011") and bool2stdlogic(v_put > "110011"));
        ELSE
            v_const_0_mux1_56 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110011_or_put_gt_const_110011_mux1 := (bool2stdlogic(v_get < "110011") or bool2stdlogic(v_put > "110011"));
        ELSE
            v_get_lt_const_110011_or_put_gt_const_110011_mux1 := v_const_0_mux1_56;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_55 := (bool2stdlogic(v_get < "110100") and bool2stdlogic(v_put > "110100"));
        ELSE
            v_const_0_mux1_55 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110100_or_put_gt_const_110100_mux1 := (bool2stdlogic(v_get < "110100") or bool2stdlogic(v_put > "110100"));
        ELSE
            v_get_lt_const_110100_or_put_gt_const_110100_mux1 := v_const_0_mux1_55;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_57 := (bool2stdlogic(v_get < "110101") and bool2stdlogic(v_put > "110101"));
        ELSE
            v_const_0_mux1_57 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110101_or_put_gt_const_110101_mux1 := (bool2stdlogic(v_get < "110101") or bool2stdlogic(v_put > "110101"));
        ELSE
            v_get_lt_const_110101_or_put_gt_const_110101_mux1 := v_const_0_mux1_57;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_48 := (bool2stdlogic(v_get < "110110") and bool2stdlogic(v_put > "110110"));
        ELSE
            v_const_0_mux1_48 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110110_or_put_gt_const_110110_mux1 := (bool2stdlogic(v_get < "110110") or bool2stdlogic(v_put > "110110"));
        ELSE
            v_get_lt_const_110110_or_put_gt_const_110110_mux1 := v_const_0_mux1_48;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_46 := (bool2stdlogic(v_get < "110111") and bool2stdlogic(v_put > "110111"));
        ELSE
            v_const_0_mux1_46 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_110111_or_put_gt_const_110111_mux1 := (bool2stdlogic(v_get < "110111") or bool2stdlogic(v_put > "110111"));
        ELSE
            v_get_lt_const_110111_or_put_gt_const_110111_mux1 := v_const_0_mux1_46;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_47 := (bool2stdlogic(v_get < "111000") and bool2stdlogic(v_put > "111000"));
        ELSE
            v_const_0_mux1_47 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111000_or_put_gt_const_111000_mux1 := (bool2stdlogic(v_get < "111000") or bool2stdlogic(v_put > "111000"));
        ELSE
            v_get_lt_const_111000_or_put_gt_const_111000_mux1 := v_const_0_mux1_47;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_58 := (bool2stdlogic(v_get < "111001") and bool2stdlogic(v_put > "111001"));
        ELSE
            v_const_0_mux1_58 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111001_or_put_gt_const_111001_mux1 := (bool2stdlogic(v_get < "111001") or bool2stdlogic(v_put > "111001"));
        ELSE
            v_get_lt_const_111001_or_put_gt_const_111001_mux1 := v_const_0_mux1_58;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_59 := (bool2stdlogic(v_get < "111010") and bool2stdlogic(v_put > "111010"));
        ELSE
            v_const_0_mux1_59 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111010_or_put_gt_const_111010_mux1 := (bool2stdlogic(v_get < "111010") or bool2stdlogic(v_put > "111010"));
        ELSE
            v_get_lt_const_111010_or_put_gt_const_111010_mux1 := v_const_0_mux1_59;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_60 := (bool2stdlogic(v_get < "111011") and bool2stdlogic(v_put > "111011"));
        ELSE
            v_const_0_mux1_60 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111011_or_put_gt_const_111011_mux1 := (bool2stdlogic(v_get < "111011") or bool2stdlogic(v_put > "111011"));
        ELSE
            v_get_lt_const_111011_or_put_gt_const_111011_mux1 := v_const_0_mux1_60;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_62 := (bool2stdlogic(v_get < "111100") and bool2stdlogic(v_put > "111100"));
        ELSE
            v_const_0_mux1_62 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111100_or_put_gt_const_111100_mux1 := (bool2stdlogic(v_get < "111100") or bool2stdlogic(v_put > "111100"));
        ELSE
            v_get_lt_const_111100_or_put_gt_const_111100_mux1 := v_const_0_mux1_62;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_61 := (bool2stdlogic(v_get < "111101") and bool2stdlogic(v_put > "111101"));
        ELSE
            v_const_0_mux1_61 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111101_or_put_gt_const_111101_mux1 := (bool2stdlogic(v_get < "111101") or bool2stdlogic(v_put > "111101"));
        ELSE
            v_get_lt_const_111101_or_put_gt_const_111101_mux1 := v_const_0_mux1_61;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_63 := (bool2stdlogic(v_get < "111110") and bool2stdlogic(v_put > "111110"));
        ELSE
            v_const_0_mux1_63 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111110_or_put_gt_const_111110_mux1 := (bool2stdlogic(v_get < "111110") or bool2stdlogic(v_put > "111110"));
        ELSE
            v_get_lt_const_111110_or_put_gt_const_111110_mux1 := v_const_0_mux1_63;
        END IF;
        IF (( not v_overrun = '1') and in_unnamed = '1') THEN
            v_const_0_mux1_64 := (bool2stdlogic(v_get < "111111") and bool2stdlogic(v_put > "111111"));
        ELSE
            v_const_0_mux1_64 := '0';
        END IF;
        IF (( not ( not v_overrun = '1')) and in_unnamed = '1') THEN
            v_get_lt_const_111111_or_put_gt_const_111111_mux1 := (bool2stdlogic(v_get < "111111") or bool2stdlogic(v_put > "111111"));
        ELSE
            v_get_lt_const_111111_or_put_gt_const_111111_mux1 := v_const_0_mux1_64;
        END IF;
        out_mask <= (v_get_lt_const_111111_or_put_gt_const_111111_mux1 & v_get_lt_const_111110_or_put_gt_const_111110_mux1 & v_get_lt_const_111101_or_put_gt_const_111101_mux1 & v_get_lt_const_111100_or_put_gt_const_111100_mux1 & v_get_lt_const_111011_or_put_gt_const_111011_mux1 & v_get_lt_const_111010_or_put_gt_const_111010_mux1 & v_get_lt_const_111001_or_put_gt_const_111001_mux1 & v_get_lt_const_111000_or_put_gt_const_111000_mux1 & v_get_lt_const_110111_or_put_gt_const_110111_mux1 & v_get_lt_const_110110_or_put_gt_const_110110_mux1 & v_get_lt_const_110101_or_put_gt_const_110101_mux1 & v_get_lt_const_110100_or_put_gt_const_110100_mux1 & v_get_lt_const_110011_or_put_gt_const_110011_mux1 & v_get_lt_const_110010_or_put_gt_const_110010_mux1 & v_get_lt_const_110001_or_put_gt_const_110001_mux1 & 
            v_get_lt_const_110000_or_put_gt_const_110000_mux1 & v_get_lt_const_101111_or_put_gt_const_101111_mux1 & v_get_lt_const_101110_or_put_gt_const_101110_mux1 & v_get_lt_const_101101_or_put_gt_const_101101_mux1 & v_get_lt_const_101100_or_put_gt_const_101100_mux1 & v_get_lt_const_101011_or_put_gt_const_101011_mux1 & v_get_lt_const_101010_or_put_gt_const_101010_mux1 & v_get_lt_const_101001_or_put_gt_const_101001_mux1 & v_get_lt_const_101000_or_put_gt_const_101000_mux1 & v_get_lt_const_100111_or_put_gt_const_100111_mux1 & v_get_lt_const_100110_or_put_gt_const_100110_mux1 & v_get_lt_const_100101_or_put_gt_const_100101_mux1 & v_get_lt_const_100100_or_put_gt_const_100100_mux1 & v_get_lt_const_100011_or_put_gt_const_100011_mux1 & v_get_lt_const_100010_or_put_gt_const_100010_mux1 & v_get_lt_const_100001_or_put_gt_const_100001_mux1 & 
            v_get_lt_const_100000_or_put_gt_const_100000_mux1 & v_get_lt_const_11111_or_put_gt_const_11111_mux1 & v_get_lt_const_11110_or_put_gt_const_11110_mux1 & v_get_lt_const_11101_or_put_gt_const_11101_mux1 & v_get_lt_const_11100_or_put_gt_const_11100_mux1 & v_get_lt_const_11011_or_put_gt_const_11011_mux1 & v_get_lt_const_11010_or_put_gt_const_11010_mux1 & v_get_lt_const_11001_or_put_gt_const_11001_mux1 & v_get_lt_const_11000_or_put_gt_const_11000_mux1 & v_get_lt_const_10111_or_put_gt_const_10111_mux1 & v_get_lt_const_10110_or_put_gt_const_10110_mux1 & v_get_lt_const_10101_or_put_gt_const_10101_mux1 & v_get_lt_const_10100_or_put_gt_const_10100_mux1 & v_get_lt_const_10011_or_put_gt_const_10011_mux1 & v_get_lt_const_10010_or_put_gt_const_10010_mux1 & v_get_lt_const_10001_or_put_gt_const_10001_mux1 & 
            v_get_lt_const_10000_or_put_gt_const_10000_mux1 & v_get_lt_const_1111_or_put_gt_const_1111_mux1 & v_get_lt_const_1110_or_put_gt_const_1110_mux1 & v_get_lt_const_1101_or_put_gt_const_1101_mux1 & v_get_lt_const_1100_or_put_gt_const_1100_mux1 & v_get_lt_const_1011_or_put_gt_const_1011_mux1 & v_get_lt_const_1010_or_put_gt_const_1010_mux1 & v_get_lt_const_1001_or_put_gt_const_1001_mux1 & v_get_lt_const_1000_or_put_gt_const_1000_mux1 & v_get_lt_const_111_or_put_gt_const_111_mux1 & v_get_lt_const_110_or_put_gt_const_110_mux1 & v_get_lt_const_101_or_put_gt_const_101_mux1 & v_get_lt_const_100_or_put_gt_const_100_mux1 & v_get_lt_const_11_or_put_gt_const_11_mux1 & v_get_lt_const_10_or_put_gt_const_10_mux1 & v_get_lt_const_1_or_put_gt_const_1_mux1 & 
            v_get_lt_const_0_or_put_gt_const_0_mux1);
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: flat_mux
-- 
------------------------------------------------

ENTITY flat_mux IS 
    PORT(
        in_selector : IN UNSIGNED(5 downto 0);
        in_data : IN UNSIGNED(63 downto 0);
        out_selected : OUT UNSIGNED(0 downto 0)
    );
END flat_mux;

ARCHITECTURE impl OF flat_mux IS 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_selector : UNSIGNED(5 downto 0); 
        VARIABLE v_flat_array : UNSIGNED(63 downto 0); 
        VARIABLE v_flat_array_bit_0_mux1 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux2 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux8 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux4 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux24 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux6 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux7 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux5 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux9 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux3 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux10 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux16 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux21 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux20 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux15 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux18 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux19 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux13 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux22 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux11 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux14 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux12 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux17 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux23 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux25 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux34 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux26 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux30 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux35 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux36 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux29 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux37 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux28 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux32 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux38 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux31 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux33 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux27 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux40 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux39 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux52 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux53 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux54 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux44 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux41 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux42 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux55 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux56 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux45 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux47 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux49 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux51 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux48 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux46 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux50 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux43 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux57 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux58 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux61 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux60 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux59 : UNSIGNED(0 downto 0); 
        VARIABLE v_flat_array_bit_0_mux62 : UNSIGNED(0 downto 0); 
        VARIABLE v_selected : UNSIGNED(0 downto 0); 
    BEGIN
        v_selector := in_selector;
        v_flat_array := in_data;
        IF (v_selector = "000001") THEN
            v_flat_array_bit_0_mux1 := v_flat_array(1 downto 1);
        ELSE
            v_flat_array_bit_0_mux1 := v_flat_array(0 downto 0);
        END IF;
        IF (v_selector = "000010") THEN
            v_flat_array_bit_0_mux2 := v_flat_array(2 downto 2);
        ELSE
            v_flat_array_bit_0_mux2 := v_flat_array_bit_0_mux1;
        END IF;
        IF (v_selector = "000011") THEN
            v_flat_array_bit_0_mux3 := v_flat_array(3 downto 3);
        ELSE
            v_flat_array_bit_0_mux3 := v_flat_array_bit_0_mux2;
        END IF;
        IF (v_selector = "000100") THEN
            v_flat_array_bit_0_mux4 := v_flat_array(4 downto 4);
        ELSE
            v_flat_array_bit_0_mux4 := v_flat_array_bit_0_mux3;
        END IF;
        IF (v_selector = "000101") THEN
            v_flat_array_bit_0_mux5 := v_flat_array(5 downto 5);
        ELSE
            v_flat_array_bit_0_mux5 := v_flat_array_bit_0_mux4;
        END IF;
        IF (v_selector = "000110") THEN
            v_flat_array_bit_0_mux6 := v_flat_array(6 downto 6);
        ELSE
            v_flat_array_bit_0_mux6 := v_flat_array_bit_0_mux5;
        END IF;
        IF (v_selector = "000111") THEN
            v_flat_array_bit_0_mux7 := v_flat_array(7 downto 7);
        ELSE
            v_flat_array_bit_0_mux7 := v_flat_array_bit_0_mux6;
        END IF;
        IF (v_selector = "001000") THEN
            v_flat_array_bit_0_mux8 := v_flat_array(8 downto 8);
        ELSE
            v_flat_array_bit_0_mux8 := v_flat_array_bit_0_mux7;
        END IF;
        IF (v_selector = "001001") THEN
            v_flat_array_bit_0_mux9 := v_flat_array(9 downto 9);
        ELSE
            v_flat_array_bit_0_mux9 := v_flat_array_bit_0_mux8;
        END IF;
        IF (v_selector = "001010") THEN
            v_flat_array_bit_0_mux10 := v_flat_array(10 downto 10);
        ELSE
            v_flat_array_bit_0_mux10 := v_flat_array_bit_0_mux9;
        END IF;
        IF (v_selector = "001011") THEN
            v_flat_array_bit_0_mux11 := v_flat_array(11 downto 11);
        ELSE
            v_flat_array_bit_0_mux11 := v_flat_array_bit_0_mux10;
        END IF;
        IF (v_selector = "001100") THEN
            v_flat_array_bit_0_mux12 := v_flat_array(12 downto 12);
        ELSE
            v_flat_array_bit_0_mux12 := v_flat_array_bit_0_mux11;
        END IF;
        IF (v_selector = "001101") THEN
            v_flat_array_bit_0_mux13 := v_flat_array(13 downto 13);
        ELSE
            v_flat_array_bit_0_mux13 := v_flat_array_bit_0_mux12;
        END IF;
        IF (v_selector = "001110") THEN
            v_flat_array_bit_0_mux14 := v_flat_array(14 downto 14);
        ELSE
            v_flat_array_bit_0_mux14 := v_flat_array_bit_0_mux13;
        END IF;
        IF (v_selector = "001111") THEN
            v_flat_array_bit_0_mux15 := v_flat_array(15 downto 15);
        ELSE
            v_flat_array_bit_0_mux15 := v_flat_array_bit_0_mux14;
        END IF;
        IF (v_selector = "010000") THEN
            v_flat_array_bit_0_mux16 := v_flat_array(16 downto 16);
        ELSE
            v_flat_array_bit_0_mux16 := v_flat_array_bit_0_mux15;
        END IF;
        IF (v_selector = "010001") THEN
            v_flat_array_bit_0_mux17 := v_flat_array(17 downto 17);
        ELSE
            v_flat_array_bit_0_mux17 := v_flat_array_bit_0_mux16;
        END IF;
        IF (v_selector = "010010") THEN
            v_flat_array_bit_0_mux18 := v_flat_array(18 downto 18);
        ELSE
            v_flat_array_bit_0_mux18 := v_flat_array_bit_0_mux17;
        END IF;
        IF (v_selector = "010011") THEN
            v_flat_array_bit_0_mux19 := v_flat_array(19 downto 19);
        ELSE
            v_flat_array_bit_0_mux19 := v_flat_array_bit_0_mux18;
        END IF;
        IF (v_selector = "010100") THEN
            v_flat_array_bit_0_mux20 := v_flat_array(20 downto 20);
        ELSE
            v_flat_array_bit_0_mux20 := v_flat_array_bit_0_mux19;
        END IF;
        IF (v_selector = "010101") THEN
            v_flat_array_bit_0_mux21 := v_flat_array(21 downto 21);
        ELSE
            v_flat_array_bit_0_mux21 := v_flat_array_bit_0_mux20;
        END IF;
        IF (v_selector = "010110") THEN
            v_flat_array_bit_0_mux22 := v_flat_array(22 downto 22);
        ELSE
            v_flat_array_bit_0_mux22 := v_flat_array_bit_0_mux21;
        END IF;
        IF (v_selector = "010111") THEN
            v_flat_array_bit_0_mux23 := v_flat_array(23 downto 23);
        ELSE
            v_flat_array_bit_0_mux23 := v_flat_array_bit_0_mux22;
        END IF;
        IF (v_selector = "011000") THEN
            v_flat_array_bit_0_mux24 := v_flat_array(24 downto 24);
        ELSE
            v_flat_array_bit_0_mux24 := v_flat_array_bit_0_mux23;
        END IF;
        IF (v_selector = "011001") THEN
            v_flat_array_bit_0_mux25 := v_flat_array(25 downto 25);
        ELSE
            v_flat_array_bit_0_mux25 := v_flat_array_bit_0_mux24;
        END IF;
        IF (v_selector = "011010") THEN
            v_flat_array_bit_0_mux26 := v_flat_array(26 downto 26);
        ELSE
            v_flat_array_bit_0_mux26 := v_flat_array_bit_0_mux25;
        END IF;
        IF (v_selector = "011011") THEN
            v_flat_array_bit_0_mux27 := v_flat_array(27 downto 27);
        ELSE
            v_flat_array_bit_0_mux27 := v_flat_array_bit_0_mux26;
        END IF;
        IF (v_selector = "011100") THEN
            v_flat_array_bit_0_mux28 := v_flat_array(28 downto 28);
        ELSE
            v_flat_array_bit_0_mux28 := v_flat_array_bit_0_mux27;
        END IF;
        IF (v_selector = "011101") THEN
            v_flat_array_bit_0_mux29 := v_flat_array(29 downto 29);
        ELSE
            v_flat_array_bit_0_mux29 := v_flat_array_bit_0_mux28;
        END IF;
        IF (v_selector = "011110") THEN
            v_flat_array_bit_0_mux30 := v_flat_array(30 downto 30);
        ELSE
            v_flat_array_bit_0_mux30 := v_flat_array_bit_0_mux29;
        END IF;
        IF (v_selector = "011111") THEN
            v_flat_array_bit_0_mux31 := v_flat_array(31 downto 31);
        ELSE
            v_flat_array_bit_0_mux31 := v_flat_array_bit_0_mux30;
        END IF;
        IF (v_selector = "100000") THEN
            v_flat_array_bit_0_mux32 := v_flat_array(32 downto 32);
        ELSE
            v_flat_array_bit_0_mux32 := v_flat_array_bit_0_mux31;
        END IF;
        IF (v_selector = "100001") THEN
            v_flat_array_bit_0_mux33 := v_flat_array(33 downto 33);
        ELSE
            v_flat_array_bit_0_mux33 := v_flat_array_bit_0_mux32;
        END IF;
        IF (v_selector = "100010") THEN
            v_flat_array_bit_0_mux34 := v_flat_array(34 downto 34);
        ELSE
            v_flat_array_bit_0_mux34 := v_flat_array_bit_0_mux33;
        END IF;
        IF (v_selector = "100011") THEN
            v_flat_array_bit_0_mux35 := v_flat_array(35 downto 35);
        ELSE
            v_flat_array_bit_0_mux35 := v_flat_array_bit_0_mux34;
        END IF;
        IF (v_selector = "100100") THEN
            v_flat_array_bit_0_mux36 := v_flat_array(36 downto 36);
        ELSE
            v_flat_array_bit_0_mux36 := v_flat_array_bit_0_mux35;
        END IF;
        IF (v_selector = "100101") THEN
            v_flat_array_bit_0_mux37 := v_flat_array(37 downto 37);
        ELSE
            v_flat_array_bit_0_mux37 := v_flat_array_bit_0_mux36;
        END IF;
        IF (v_selector = "100110") THEN
            v_flat_array_bit_0_mux38 := v_flat_array(38 downto 38);
        ELSE
            v_flat_array_bit_0_mux38 := v_flat_array_bit_0_mux37;
        END IF;
        IF (v_selector = "100111") THEN
            v_flat_array_bit_0_mux39 := v_flat_array(39 downto 39);
        ELSE
            v_flat_array_bit_0_mux39 := v_flat_array_bit_0_mux38;
        END IF;
        IF (v_selector = "101000") THEN
            v_flat_array_bit_0_mux40 := v_flat_array(40 downto 40);
        ELSE
            v_flat_array_bit_0_mux40 := v_flat_array_bit_0_mux39;
        END IF;
        IF (v_selector = "101001") THEN
            v_flat_array_bit_0_mux41 := v_flat_array(41 downto 41);
        ELSE
            v_flat_array_bit_0_mux41 := v_flat_array_bit_0_mux40;
        END IF;
        IF (v_selector = "101010") THEN
            v_flat_array_bit_0_mux42 := v_flat_array(42 downto 42);
        ELSE
            v_flat_array_bit_0_mux42 := v_flat_array_bit_0_mux41;
        END IF;
        IF (v_selector = "101011") THEN
            v_flat_array_bit_0_mux43 := v_flat_array(43 downto 43);
        ELSE
            v_flat_array_bit_0_mux43 := v_flat_array_bit_0_mux42;
        END IF;
        IF (v_selector = "101100") THEN
            v_flat_array_bit_0_mux44 := v_flat_array(44 downto 44);
        ELSE
            v_flat_array_bit_0_mux44 := v_flat_array_bit_0_mux43;
        END IF;
        IF (v_selector = "101101") THEN
            v_flat_array_bit_0_mux45 := v_flat_array(45 downto 45);
        ELSE
            v_flat_array_bit_0_mux45 := v_flat_array_bit_0_mux44;
        END IF;
        IF (v_selector = "101110") THEN
            v_flat_array_bit_0_mux46 := v_flat_array(46 downto 46);
        ELSE
            v_flat_array_bit_0_mux46 := v_flat_array_bit_0_mux45;
        END IF;
        IF (v_selector = "101111") THEN
            v_flat_array_bit_0_mux47 := v_flat_array(47 downto 47);
        ELSE
            v_flat_array_bit_0_mux47 := v_flat_array_bit_0_mux46;
        END IF;
        IF (v_selector = "110000") THEN
            v_flat_array_bit_0_mux48 := v_flat_array(48 downto 48);
        ELSE
            v_flat_array_bit_0_mux48 := v_flat_array_bit_0_mux47;
        END IF;
        IF (v_selector = "110001") THEN
            v_flat_array_bit_0_mux49 := v_flat_array(49 downto 49);
        ELSE
            v_flat_array_bit_0_mux49 := v_flat_array_bit_0_mux48;
        END IF;
        IF (v_selector = "110010") THEN
            v_flat_array_bit_0_mux50 := v_flat_array(50 downto 50);
        ELSE
            v_flat_array_bit_0_mux50 := v_flat_array_bit_0_mux49;
        END IF;
        IF (v_selector = "110011") THEN
            v_flat_array_bit_0_mux51 := v_flat_array(51 downto 51);
        ELSE
            v_flat_array_bit_0_mux51 := v_flat_array_bit_0_mux50;
        END IF;
        IF (v_selector = "110100") THEN
            v_flat_array_bit_0_mux52 := v_flat_array(52 downto 52);
        ELSE
            v_flat_array_bit_0_mux52 := v_flat_array_bit_0_mux51;
        END IF;
        IF (v_selector = "110101") THEN
            v_flat_array_bit_0_mux53 := v_flat_array(53 downto 53);
        ELSE
            v_flat_array_bit_0_mux53 := v_flat_array_bit_0_mux52;
        END IF;
        IF (v_selector = "110110") THEN
            v_flat_array_bit_0_mux54 := v_flat_array(54 downto 54);
        ELSE
            v_flat_array_bit_0_mux54 := v_flat_array_bit_0_mux53;
        END IF;
        IF (v_selector = "110111") THEN
            v_flat_array_bit_0_mux55 := v_flat_array(55 downto 55);
        ELSE
            v_flat_array_bit_0_mux55 := v_flat_array_bit_0_mux54;
        END IF;
        IF (v_selector = "111000") THEN
            v_flat_array_bit_0_mux56 := v_flat_array(56 downto 56);
        ELSE
            v_flat_array_bit_0_mux56 := v_flat_array_bit_0_mux55;
        END IF;
        IF (v_selector = "111001") THEN
            v_flat_array_bit_0_mux57 := v_flat_array(57 downto 57);
        ELSE
            v_flat_array_bit_0_mux57 := v_flat_array_bit_0_mux56;
        END IF;
        IF (v_selector = "111010") THEN
            v_flat_array_bit_0_mux58 := v_flat_array(58 downto 58);
        ELSE
            v_flat_array_bit_0_mux58 := v_flat_array_bit_0_mux57;
        END IF;
        IF (v_selector = "111011") THEN
            v_flat_array_bit_0_mux59 := v_flat_array(59 downto 59);
        ELSE
            v_flat_array_bit_0_mux59 := v_flat_array_bit_0_mux58;
        END IF;
        IF (v_selector = "111100") THEN
            v_flat_array_bit_0_mux60 := v_flat_array(60 downto 60);
        ELSE
            v_flat_array_bit_0_mux60 := v_flat_array_bit_0_mux59;
        END IF;
        IF (v_selector = "111101") THEN
            v_flat_array_bit_0_mux61 := v_flat_array(61 downto 61);
        ELSE
            v_flat_array_bit_0_mux61 := v_flat_array_bit_0_mux60;
        END IF;
        IF (v_selector = "111110") THEN
            v_flat_array_bit_0_mux62 := v_flat_array(62 downto 62);
        ELSE
            v_flat_array_bit_0_mux62 := v_flat_array_bit_0_mux61;
        END IF;
        IF (v_selector = "111111") THEN
            v_selected := v_flat_array(63 downto 63);
        ELSE
            v_selected := v_flat_array_bit_0_mux62;
        END IF;
        out_selected <= v_selected;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: mux_large
-- 
------------------------------------------------

ENTITY mux_large IS 
    PORT(
        clock : IN STD_LOGIC;
        out_secondStageValid : OUT STD_LOGIC;
        in_selector : IN UNSIGNED(5 downto 0);
        in_hazard_corrected_data : IN UNSIGNED(63 downto 0)
    );
END mux_large;

ARCHITECTURE impl OF mux_large IS 
    SIGNAL s_selector : UNSIGNED(5 downto 0); 
    SIGNAL s_data : UNSIGNED(63 downto 0); 
    SIGNAL s_selected : UNSIGNED(0 downto 0); 
    SIGNAL s_out_bit_0 : STD_LOGIC; 
BEGIN
    flat_mux0 : entity work.flat_mux(impl) port map (
        in_selector => s_selector,
        in_data => s_data,
        out_selected => s_selected
    );
    default_comb : PROCESS(all)
        VARIABLE v_out : UNSIGNED(0 downto 0); 
    BEGIN
        s_selector <= in_selector;
        s_data <= in_hazard_corrected_data;
        v_out := s_selected;
        s_out_bit_0 <= v_out(0);
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            out_secondStageValid <= s_out_bit_0;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: Memory_Helper_3
-- Auto generated to handle various memory access issues such as read during write and read modify write hazards.
------------------------------------------------

ENTITY Memory_Helper_3 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bitmap_next : IN UNSIGNED(63 downto 0);
        out_hazard_corrected_data : OUT UNSIGNED(63 downto 0);
        in_unnamed : IN UNSIGNED(63 downto 0);
        in_unnamed_2 : IN UNSIGNED(13 downto 0);
        in_unnamed_4 : IN STD_LOGIC;
        in_unnamed_3 : IN UNSIGNED(13 downto 0)
    );
END Memory_Helper_3;

ARCHITECTURE impl OF Memory_Helper_3 IS 
    SIGNAL s_final_override_data_word_0 : UNSIGNED(63 downto 0); 
    SIGNAL s_conflict_word_0_stage_1 : STD_LOGIC; 
    SIGNAL s_bypass_data_word_0_stage_1 : UNSIGNED(63 downto 0); 
    SIGNAL s_final_conflict_word_0 : STD_LOGIC; 
    SIGNAL s_unnamed : UNSIGNED(13 downto 0); 
    SIGNAL s_conflict_word_0_stage_0 : STD_LOGIC; 
    SIGNAL s_conflict_word_0_stage_0_delayed1 : STD_LOGIC; 
    SIGNAL s_bypass_data_word_0_stage_0_delayed1 : UNSIGNED(63 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_0 : UNSIGNED(63 downto 0); 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_final_conflict_word_0 : STD_LOGIC; 
        VARIABLE v_bypass_data_word_0_stage_1 : UNSIGNED(63 downto 0); 
        VARIABLE v_final_override_data_word_0 : UNSIGNED(63 downto 0); 
        VARIABLE v_hazard_corrected_data : UNSIGNED(63 downto 0); 
    BEGIN

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_0 <= (bool2stdlogic(in_unnamed_2 = in_unnamed_3) and in_unnamed_4);
        s_bypass_data_word_0_stage_0 <= in_bitmap_next;

        -- Compare read and write addr for conflicts
        IF ((s_unnamed = in_unnamed_3) and in_unnamed_4 = '1') THEN
            v_bypass_data_word_0_stage_1 := in_bitmap_next;
        ELSE
            v_bypass_data_word_0_stage_1 := s_bypass_data_word_0_stage_0_delayed1;
        END IF;

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_1 <= (s_conflict_word_0_stage_0_delayed1 or (bool2stdlogic(s_unnamed = in_unnamed_3) and in_unnamed_4));
        s_bypass_data_word_0_stage_1 <= v_bypass_data_word_0_stage_1;
        v_final_conflict_word_0 := s_final_conflict_word_0;
        v_final_override_data_word_0 := s_final_override_data_word_0;

        -- If read and write addr match and read and write are enabled and write is not masked, forward write data to read output.
        IF v_final_conflict_word_0 = '1' THEN
            v_hazard_corrected_data := v_final_override_data_word_0;
        ELSE
            v_hazard_corrected_data := in_unnamed;
        END IF;

        -- If read and write addr match and read and write are enabled and write is not masked, forward write data to read output.
        out_hazard_corrected_data <= v_hazard_corrected_data;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_unnamed <= in_unnamed_2;
            s_conflict_word_0_stage_0_delayed1 <= s_conflict_word_0_stage_0;
            s_bypass_data_word_0_stage_0_delayed1 <= s_bypass_data_word_0_stage_0;
            s_final_conflict_word_0 <= s_conflict_word_0_stage_1;
            s_final_override_data_word_0 <= s_bypass_data_word_0_stage_1;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY altera_mf;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_memory_4
-- 
------------------------------------------------

ENTITY scl_memory_4 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bitmap_next : IN UNSIGNED(63 downto 0);
        out_hazard_corrected_data : OUT UNSIGNED(63 downto 0);
        in_unnamed : IN UNSIGNED(13 downto 0);
        in_unnamed_3 : IN STD_LOGIC;
        in_unnamed_2 : IN UNSIGNED(13 downto 0)
    );
END scl_memory_4;

ARCHITECTURE impl OF scl_memory_4 IS 
    SIGNAL s_unnamed : UNSIGNED(63 downto 0); 
    SIGNAL s_altsyncram_q_b_delayed1 : UNSIGNED(63 downto 0) := "0000000000000000000000000000000000000000000000000000000000000000"; 
    SIGNAL s_altsyncram_q_b : UNSIGNED(63 downto 0); 
    SIGNAL s_altsyncram_q_b_2 : UNSIGNED(63 downto 0); 
BEGIN
    altsyncram : altera_mf.altera_mf_components.altsyncram
     generic map (
        address_aclr_a => "NONE",
        address_aclr_b => "NONE",
        address_reg_b => "CLOCK0",
        byteena_reg_b => "CLOCK0",
        indata_reg_b => "CLOCK0",
        intended_device_family => "Arria 10",
        numwords_a => 16384,
        numwords_b => 16384,
        operation_mode => "DUAL_PORT",
        outdata_aclr_a => "NONE",
        outdata_aclr_b => "NONE",
        outdata_reg_a => "UNREGISTERED",
        outdata_reg_b => "UNREGISTERED",
        ram_block_type => "M20K",
        rdcontrol_reg_b => "CLOCK0",
        read_during_write_mode_mixed_ports => "DONT_CARE",
        read_during_write_mode_port_a => "DONT_CARE",
        read_during_write_mode_port_b => "DONT_CARE",
        width_a => 64,
        width_b => 64,
        widthad_a => 14,
        widthad_b => 14,
        wrcontrol_aclr_a => "NONE",
        wrcontrol_aclr_b => "NONE",
        wrcontrol_wraddress_reg_b => "CLOCK0"
    )
     port map (
        clock0 => clock,
        wren_a => in_unnamed_3,
        data_a => STD_LOGIC_VECTOR(in_bitmap_next),
        address_a => STD_LOGIC_VECTOR(in_unnamed_2),
        address_b => STD_LOGIC_VECTOR(in_unnamed),
        UNSIGNED(q_b) => s_altsyncram_q_b
    );
    Memory_Helper0 : entity work.Memory_Helper_3(impl) port map (
        clock => clock,
        in_bitmap_next => in_bitmap_next,
        in_unnamed => s_unnamed,
        in_unnamed_2 => in_unnamed,
        in_unnamed_3 => in_unnamed_2,
        in_unnamed_4 => in_unnamed_3,
        out_hazard_corrected_data => out_hazard_corrected_data
    );
    default_comb : PROCESS(all)
    BEGIN
        s_unnamed <= s_altsyncram_q_b_delayed1;
        s_altsyncram_q_b_2 <= s_altsyncram_q_b;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_altsyncram_q_b_delayed1 <= s_altsyncram_q_b_2;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: FirstLookupStage
-- 
------------------------------------------------

ENTITY FirstLookupStage IS 
    PORT(
        clock : IN STD_LOGIC;
        in_lastSequenceNumber : IN UNSIGNED(63 downto 0);
        out_secondStageValid : OUT STD_LOGIC;
        in_unnamed_4 : IN UNSIGNED(63 downto 0);
        in_unnamed_5 : IN UNSIGNED(13 downto 0);
        in_unnamed_3 : IN STD_LOGIC;
        in_unnamed : IN STD_LOGIC;
        out_secondStageValid_delayed6 : OUT STD_LOGIC;
        in_unnamed_2 : IN STD_LOGIC;
        in_unnamed_8 : IN STD_LOGIC;
        in_unnamed_7 : IN STD_LOGIC;
        in_unnamed_6 : IN UNSIGNED(13 downto 0);
        out_unnamed : OUT UNSIGNED(5 downto 0)
    );
END FirstLookupStage;

ARCHITECTURE impl OF FirstLookupStage IS 
    SIGNAL s_bitAddr : UNSIGNED(5 downto 0); 
    SIGNAL s_get : UNSIGNED(5 downto 0); 
    SIGNAL s_mask : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_bitmap_next : UNSIGNED(63 downto 0); 
    SIGNAL s_hazard_corrected_data : UNSIGNED(63 downto 0); 
    SIGNAL s_mask_not_delayed1 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_delayed1 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_mask_not_delayed2 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_delayed2 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_3 : STD_LOGIC; 
    SIGNAL s_unnamed_4 : UNSIGNED(5 downto 0); 
    SIGNAL s_unnamed_5 : UNSIGNED(5 downto 0); 
    SIGNAL s_unnamed_6 : UNSIGNED(5 downto 0); 
    SIGNAL s_secondStageValid_delayed1 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed3 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed2 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed4 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed5 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed6 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed5_2 : STD_LOGIC; 
    SIGNAL s_mask_not_delayed1_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_delayed1_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_7 : STD_LOGIC; 
    SIGNAL s_unnamed_8 : UNSIGNED(63 downto 0); 
    SIGNAL s_mask_not : UNSIGNED(63 downto 0); 
    SIGNAL s_secondStageValid_delayed4_2 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed3_2 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed2_2 : STD_LOGIC; 
    SIGNAL s_secondStageValid_delayed1_2 : STD_LOGIC; 
    SIGNAL s_unnamed_9 : UNSIGNED(5 downto 0); 
    SIGNAL s_secondStageValid : STD_LOGIC; 
    SIGNAL s_selector : UNSIGNED(5 downto 0); 
    SIGNAL s_unnamed_10 : UNSIGNED(5 downto 0); 
BEGIN
    scl_memory0 : entity work.scl_memory_4(impl) port map (
        clock => clock,
        in_bitmap_next => s_bitmap_next,
        in_unnamed => in_unnamed_5,
        in_unnamed_2 => in_unnamed_6,
        in_unnamed_3 => in_unnamed_7,
        out_hazard_corrected_data => s_hazard_corrected_data
    );
    default_comb : PROCESS(all)
        VARIABLE v_bitAddr : UNSIGNED(5 downto 0); 
        VARIABLE v_const_0_mux1 : UNSIGNED(63 downto 0); 
        VARIABLE v_bitmap : UNSIGNED(63 downto 0); 
        VARIABLE v_const_0_mux1_2 : UNSIGNED(63 downto 0); 
        VARIABLE v_const_0_mux3 : UNSIGNED(63 downto 0); 
        VARIABLE v_const_0_mux2 : UNSIGNED(63 downto 0); 
        VARIABLE v_bitmap_next : UNSIGNED(63 downto 0); 
        VARIABLE v_const_0_mux3_2 : UNSIGNED(63 downto 0); 
    BEGIN
        s_bitAddr <= in_unnamed_4(13 downto 8);
        v_bitmap := s_hazard_corrected_data;
        out_secondStageValid <= s_secondStageValid;
        v_bitAddr := s_bitAddr;
        IF in_unnamed_3 = '1' THEN
            v_const_0_mux1 := "0000000000000000000000000000000000000000000000000000000000000000";
        ELSE
            v_const_0_mux1 := v_bitmap;
        END IF;
        s_get <= in_lastSequenceNumber(13 downto 8);
        s_unnamed <= (bool2stdlogic((in_unnamed_4 - in_lastSequenceNumber) >= "0000000000000000000000000000000000000000000000000011111100000000") and in_unnamed);
        s_unnamed_6 <= s_bitAddr;
        s_unnamed_5 <= s_unnamed_10;
        s_unnamed_4 <= s_selector;
        s_secondStageValid_delayed1 <= s_secondStageValid_delayed1_2;
        s_secondStageValid_delayed2 <= s_secondStageValid_delayed2_2;
        s_secondStageValid_delayed3 <= s_secondStageValid_delayed3_2;
        s_secondStageValid_delayed4 <= s_secondStageValid_delayed4_2;
        s_secondStageValid_delayed5 <= s_secondStageValid_delayed5_2;
        out_secondStageValid_delayed6 <= s_secondStageValid_delayed6;
        v_const_0_mux1_2 := v_const_0_mux1;
        IF in_unnamed_2 = '1' THEN
            v_const_0_mux2 := (v_const_0_mux1_2 and s_mask_not_delayed2);
        ELSE
            v_const_0_mux2 := v_const_0_mux1_2;
        END IF;
        IF s_unnamed_3 = '1' THEN
            v_const_0_mux3 := "0000000000000000000000000000000000000000000000000000000000000000";
        ELSE
            v_const_0_mux3 := v_const_0_mux2;
        END IF;
        v_const_0_mux3_2 := v_const_0_mux3;
        IF in_unnamed_8 = '1' THEN
            v_bitmap_next := (v_const_0_mux3_2 or s_unnamed_delayed2);
        ELSE
            v_bitmap_next := v_const_0_mux3_2;
        END IF;
        s_bitmap_next <= v_bitmap_next;
        out_unnamed <= s_unnamed_9;
        s_unnamed_8 <= (bool2stdlogic(v_bitAddr = "111111") & bool2stdlogic(v_bitAddr = "111110") & bool2stdlogic(v_bitAddr = "111101") & bool2stdlogic(v_bitAddr = "111100") & bool2stdlogic(v_bitAddr = "111011") & bool2stdlogic(v_bitAddr = "111010") & bool2stdlogic(v_bitAddr = "111001") & bool2stdlogic(v_bitAddr = "111000") & bool2stdlogic(v_bitAddr = "110111") & bool2stdlogic(v_bitAddr = "110110") & bool2stdlogic(v_bitAddr = "110101") & bool2stdlogic(v_bitAddr = "110100") & bool2stdlogic(v_bitAddr = "110011") & bool2stdlogic(v_bitAddr = "110010") & bool2stdlogic(v_bitAddr = "110001") & 
            bool2stdlogic(v_bitAddr = "110000") & bool2stdlogic(v_bitAddr = "101111") & bool2stdlogic(v_bitAddr = "101110") & bool2stdlogic(v_bitAddr = "101101") & bool2stdlogic(v_bitAddr = "101100") & bool2stdlogic(v_bitAddr = "101011") & bool2stdlogic(v_bitAddr = "101010") & bool2stdlogic(v_bitAddr = "101001") & bool2stdlogic(v_bitAddr = "101000") & bool2stdlogic(v_bitAddr = "100111") & bool2stdlogic(v_bitAddr = "100110") & bool2stdlogic(v_bitAddr = "100101") & bool2stdlogic(v_bitAddr = "100100") & bool2stdlogic(v_bitAddr = "100011") & bool2stdlogic(v_bitAddr = "100010") & bool2stdlogic(v_bitAddr = "100001") & 
            bool2stdlogic(v_bitAddr = "100000") & bool2stdlogic(v_bitAddr = "011111") & bool2stdlogic(v_bitAddr = "011110") & bool2stdlogic(v_bitAddr = "011101") & bool2stdlogic(v_bitAddr = "011100") & bool2stdlogic(v_bitAddr = "011011") & bool2stdlogic(v_bitAddr = "011010") & bool2stdlogic(v_bitAddr = "011001") & bool2stdlogic(v_bitAddr = "011000") & bool2stdlogic(v_bitAddr = "010111") & bool2stdlogic(v_bitAddr = "010110") & bool2stdlogic(v_bitAddr = "010101") & bool2stdlogic(v_bitAddr = "010100") & bool2stdlogic(v_bitAddr = "010011") & bool2stdlogic(v_bitAddr = "010010") & bool2stdlogic(v_bitAddr = "010001") & 
            bool2stdlogic(v_bitAddr = "010000") & bool2stdlogic(v_bitAddr = "001111") & bool2stdlogic(v_bitAddr = "001110") & bool2stdlogic(v_bitAddr = "001101") & bool2stdlogic(v_bitAddr = "001100") & bool2stdlogic(v_bitAddr = "001011") & bool2stdlogic(v_bitAddr = "001010") & bool2stdlogic(v_bitAddr = "001001") & bool2stdlogic(v_bitAddr = "001000") & bool2stdlogic(v_bitAddr = "000111") & bool2stdlogic(v_bitAddr = "000110") & bool2stdlogic(v_bitAddr = "000101") & bool2stdlogic(v_bitAddr = "000100") & bool2stdlogic(v_bitAddr = "000011") & bool2stdlogic(v_bitAddr = "000010") & bool2stdlogic(v_bitAddr = "000001") & 
            bool2stdlogic(v_bitAddr = "000000"));
        s_mask_not <= ( not s_mask);
        s_mask_not_delayed1_2 <= s_mask_not_delayed1;
        s_unnamed_delayed1_2 <= s_unnamed_delayed1;
        s_unnamed_7 <= s_unnamed_2;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_selector <= s_unnamed_5;
            s_unnamed_delayed2 <= s_unnamed_delayed1_2;
            s_secondStageValid_delayed4_2 <= s_secondStageValid_delayed3;
            s_unnamed_10 <= s_unnamed_6;
            s_unnamed_3 <= s_unnamed_7;
            s_unnamed_9 <= s_unnamed_4;
            s_secondStageValid_delayed5_2 <= s_secondStageValid_delayed4;
            s_secondStageValid_delayed3_2 <= s_secondStageValid_delayed2;
            s_secondStageValid_delayed1_2 <= out_secondStageValid;
            s_secondStageValid_delayed2_2 <= s_secondStageValid_delayed1;
            s_secondStageValid_delayed6 <= s_secondStageValid_delayed5;
            s_unnamed_delayed1 <= s_unnamed_8;
            s_mask_not_delayed1 <= s_mask_not;
            s_unnamed_2 <= s_unnamed;
            s_mask_not_delayed2 <= s_mask_not_delayed1_2;
        END IF;
    END PROCESS;

    mux_large0 : entity work.mux_large(impl) port map (
        clock => clock,
        in_hazard_corrected_data => s_hazard_corrected_data,
        in_selector => s_selector,
        out_secondStageValid => s_secondStageValid
    );
    ringBufferMask0 : entity work.ringBufferMask(impl) port map (
        in_bitAddr => s_bitAddr,
        in_unnamed => in_unnamed,
        in_get => s_get,
        out_mask => s_mask
    );
END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: windowBounds
-- 
------------------------------------------------

ENTITY windowBounds IS 
    PORT(
        clock : IN STD_LOGIC;
        in_m_cmd_windowSize : IN UNSIGNED(23 downto 0);
        in_lastSequenceNumber : IN UNSIGNED(63 downto 0);
        out_inPhysicalWindow : OUT STD_LOGIC;
        out_inVirtualWindow : OUT STD_LOGIC;
        in_unnamed_3 : IN UNSIGNED(63 downto 0);
        in_aboveWindow_not : IN STD_LOGIC;
        in_unnamed : IN UNSIGNED(2 downto 0);
        in_unnamed_2 : IN STD_LOGIC;
        in_unnamed_4 : IN STD_LOGIC;
        out_unnamed : OUT STD_LOGIC
    );
END windowBounds;

ARCHITECTURE impl OF windowBounds IS 
    SIGNAL s_windowOffset : UNSIGNED(63 downto 0); 
    SIGNAL s_virtualWindowSize : UNSIGNED(23 downto 0) := "000000000011111100000000"; 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_inPhysicalWindow_delayed1 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed1 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed2 : STD_LOGIC; 
    SIGNAL s_inPhysicalWindow_delayed3 : STD_LOGIC; 
    SIGNAL s_inPhysicalWindow_delayed4 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed3 : UNSIGNED(63 downto 0); 
    SIGNAL s_windowOffset_delayed4 : UNSIGNED(63 downto 0); 
    SIGNAL s_windowOffset_delayed2 : UNSIGNED(63 downto 0); 
    SIGNAL s_windowOffset_delayed5 : UNSIGNED(63 downto 0); 
    SIGNAL s_windowOffset_delayed6 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed5 : STD_LOGIC; 
    SIGNAL s_inPhysicalWindow_delayed6 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed8 : UNSIGNED(63 downto 0); 
    SIGNAL s_windowOffset_delayed7 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed7 : STD_LOGIC; 
    SIGNAL s_inPhysicalWindow_delayed8 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed9 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed9 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed8_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed8_2 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed7_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed7_2 : STD_LOGIC; 
    SIGNAL s_inPhysicalWindow_delayed6_2 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed6_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_windowOffset_delayed5_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed5_2 : STD_LOGIC; 
    SIGNAL s_inPhysicalWindow_delayed4_2 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed4_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_windowOffset_delayed3_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed3_2 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed2_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed2_2 : STD_LOGIC; 
    SIGNAL s_windowOffset_delayed1_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow_delayed1_2 : STD_LOGIC; 
BEGIN
    default_comb : PROCESS(all)
        CONSTANT C_OPSETWINDOWSIZE : UNSIGNED(2 downto 0) := "011"; 
        VARIABLE v_virtualWindowSize : UNSIGNED(63 downto 0); 
        VARIABLE v_virtualWindowSize_2 : UNSIGNED(23 downto 0); 
    BEGIN
        s_windowOffset <= (in_lastSequenceNumber - in_unnamed_3);
        out_inPhysicalWindow <= (in_aboveWindow_not and bool2stdlogic(s_windowOffset < "0000000000000000000000000000000000000000000000000011111100000000"));
        v_virtualWindowSize_2 := s_virtualWindowSize;
        v_virtualWindowSize := ("0000000000000000000000000000000000000000" & v_virtualWindowSize_2);
        out_inVirtualWindow <= (in_unnamed_4 and bool2stdlogic(s_windowOffset_delayed9 < v_virtualWindowSize));
        s_unnamed <= (in_unnamed_2 and bool2stdlogic(in_unnamed = C_OPSETWINDOWSIZE));
        out_unnamed <= s_inPhysicalWindow_delayed9;
        s_inPhysicalWindow_delayed1 <= s_inPhysicalWindow_delayed1_2;
        s_windowOffset_delayed1 <= s_windowOffset_delayed1_2;
        s_inPhysicalWindow_delayed2 <= s_inPhysicalWindow_delayed2_2;
        s_windowOffset_delayed2 <= s_windowOffset_delayed2_2;
        s_inPhysicalWindow_delayed3 <= s_inPhysicalWindow_delayed3_2;
        s_windowOffset_delayed3 <= s_windowOffset_delayed3_2;
        s_inPhysicalWindow_delayed4 <= s_inPhysicalWindow_delayed4_2;
        s_windowOffset_delayed4 <= s_windowOffset_delayed4_2;
        s_inPhysicalWindow_delayed5 <= s_inPhysicalWindow_delayed5_2;
        s_windowOffset_delayed5 <= s_windowOffset_delayed5_2;
        s_inPhysicalWindow_delayed6 <= s_inPhysicalWindow_delayed6_2;
        s_windowOffset_delayed6 <= s_windowOffset_delayed6_2;
        s_inPhysicalWindow_delayed7 <= s_inPhysicalWindow_delayed7_2;
        s_windowOffset_delayed7 <= s_windowOffset_delayed7_2;
        s_inPhysicalWindow_delayed8 <= s_inPhysicalWindow_delayed8_2;
        s_windowOffset_delayed8 <= s_windowOffset_delayed8_2;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_inPhysicalWindow_delayed1_2 <= out_inPhysicalWindow;
            s_windowOffset_delayed1_2 <= s_windowOffset;
            s_windowOffset_delayed9 <= s_windowOffset_delayed8;
            s_windowOffset_delayed4_2 <= s_windowOffset_delayed3;
            s_inPhysicalWindow_delayed6_2 <= s_inPhysicalWindow_delayed5;
            s_windowOffset_delayed6_2 <= s_windowOffset_delayed5;
            s_inPhysicalWindow_delayed2_2 <= s_inPhysicalWindow_delayed1;
            s_windowOffset_delayed2_2 <= s_windowOffset_delayed1;
            s_windowOffset_delayed7_2 <= s_windowOffset_delayed6;
            s_inPhysicalWindow_delayed9 <= s_inPhysicalWindow_delayed8;
            s_inPhysicalWindow_delayed4_2 <= s_inPhysicalWindow_delayed3;
            s_inPhysicalWindow_delayed5_2 <= s_inPhysicalWindow_delayed4;
            s_inPhysicalWindow_delayed7_2 <= s_inPhysicalWindow_delayed6;
            IF (s_unnamed = '1') THEN
                s_virtualWindowSize <= in_m_cmd_windowSize;
            END IF;
            s_windowOffset_delayed5_2 <= s_windowOffset_delayed4;
            s_inPhysicalWindow_delayed3_2 <= s_inPhysicalWindow_delayed2;
            s_windowOffset_delayed3_2 <= s_windowOffset_delayed2;
            s_windowOffset_delayed8_2 <= s_windowOffset_delayed7;
            s_inPhysicalWindow_delayed8_2 <= s_inPhysicalWindow_delayed7;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: Memory_Helper_2
-- Auto generated to handle various memory access issues such as read during write and read modify write hazards.
------------------------------------------------

ENTITY Memory_Helper_2 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bypass_data_word_0_stage_0 : IN UNSIGNED(31 downto 0);
        out_hazard_corrected_data : OUT UNSIGNED(31 downto 0);
        in_unnamed_6 : IN UNSIGNED(31 downto 0);
        in_unnamed : IN UNSIGNED(13 downto 0);
        in_unnamed_3 : IN UNSIGNED(31 downto 0);
        in_unnamed_4 : IN STD_LOGIC;
        in_unnamed_2 : IN STD_LOGIC;
        out_unnamed : OUT STD_LOGIC;
        in_unnamed_5 : IN UNSIGNED(13 downto 0)
    );
END Memory_Helper_2;

ARCHITECTURE impl OF Memory_Helper_2 IS 
    SIGNAL s_bypass_data_word_0_stage_1 : UNSIGNED(31 downto 0); 
    SIGNAL s_conflict_word_0_stage_0 : STD_LOGIC; 
    SIGNAL s_conflict_word_0_stage_0_delayed1 : STD_LOGIC; 
    SIGNAL s_final_conflict_word_0 : STD_LOGIC; 
    SIGNAL s_delayedWrEnableCounter : UNSIGNED(1 downto 0) := "01"; 
    SIGNAL s_bypass_data_word_0_stage_0 : UNSIGNED(31 downto 0); 
    SIGNAL s_unnamed : UNSIGNED(13 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_0_delayed1 : UNSIGNED(31 downto 0); 
    SIGNAL s_final_override_data_word_0 : UNSIGNED(31 downto 0); 
    SIGNAL s_conflict_word_0_stage_1 : STD_LOGIC; 
    SIGNAL s_unnamed_2 : UNSIGNED(1 downto 0); 
    SIGNAL s_delayedWrEnableCounterExpired_not : STD_LOGIC; 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_bypass_data_word_0_stage_1 : UNSIGNED(31 downto 0); 
        VARIABLE v_delayedWrEnableCounterExpired : STD_LOGIC; 
        VARIABLE v_final_conflict_word_0 : STD_LOGIC; 
        VARIABLE v_delayedWrEnableCounter : UNSIGNED(1 downto 0); 
        VARIABLE v_final_override_data_word_0 : UNSIGNED(31 downto 0); 
        VARIABLE v_hazard_corrected_data : UNSIGNED(31 downto 0); 
    BEGIN
        v_delayedWrEnableCounter := s_delayedWrEnableCounter;
        v_delayedWrEnableCounterExpired := v_delayedWrEnableCounter(1);
        s_bypass_data_word_0_stage_0 <= in_bypass_data_word_0_stage_0;
        v_final_conflict_word_0 := s_final_conflict_word_0;
        v_final_override_data_word_0 := s_final_override_data_word_0;

        -- If read and write addr match and read and write are enabled and write is not masked, forward write data to read output.
        IF v_final_conflict_word_0 = '1' THEN
            v_hazard_corrected_data := v_final_override_data_word_0;
        ELSE
            v_hazard_corrected_data := in_unnamed_6;
        END IF;

        -- If read and write addr match and read and write are enabled and write is not masked, forward write data to read output.
        out_hazard_corrected_data <= v_hazard_corrected_data;
        out_unnamed <= (v_delayedWrEnableCounterExpired and (in_unnamed_4 or in_unnamed_2));

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_0 <= (bool2stdlogic(in_unnamed = in_unnamed_5) and out_unnamed);

        -- Compare read and write addr for conflicts
        IF ((s_unnamed = in_unnamed_5) and out_unnamed = '1') THEN
            v_bypass_data_word_0_stage_1 := in_unnamed_3;
        ELSE
            v_bypass_data_word_0_stage_1 := s_bypass_data_word_0_stage_0_delayed1;
        END IF;

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_1 <= (s_conflict_word_0_stage_0_delayed1 or (bool2stdlogic(s_unnamed = in_unnamed_5) and out_unnamed));
        s_bypass_data_word_0_stage_1 <= v_bypass_data_word_0_stage_1;
        s_unnamed_2 <= (v_delayedWrEnableCounter - "01");
        s_delayedWrEnableCounterExpired_not <= ( not v_delayedWrEnableCounterExpired);
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            IF (s_delayedWrEnableCounterExpired_not = '1') THEN
                s_delayedWrEnableCounter <= s_unnamed_2;
            END IF;
            s_unnamed <= in_unnamed;
            s_conflict_word_0_stage_0_delayed1 <= s_conflict_word_0_stage_0;
            s_bypass_data_word_0_stage_0_delayed1 <= s_bypass_data_word_0_stage_0;
            s_final_conflict_word_0 <= s_conflict_word_0_stage_1;
            s_final_override_data_word_0 <= s_bypass_data_word_0_stage_1;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY altera_mf;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_memory_3
-- 
------------------------------------------------

ENTITY scl_memory_3 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bypass_data_word_0_stage_0 : IN UNSIGNED(31 downto 0);
        out_hazard_corrected_data : OUT UNSIGNED(31 downto 0);
        in_unnamed : IN UNSIGNED(13 downto 0);
        in_unnamed_3 : IN UNSIGNED(31 downto 0);
        in_unnamed_4 : IN STD_LOGIC;
        in_unnamed_2 : IN STD_LOGIC;
        in_unnamed_5 : IN UNSIGNED(13 downto 0)
    );
END scl_memory_3;

ARCHITECTURE impl OF scl_memory_3 IS 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_unnamed_2 : UNSIGNED(31 downto 0); 
    SIGNAL s_altsyncram_q_b_delayed1 : UNSIGNED(31 downto 0) := "00000000000000000000000000000000"; 
    SIGNAL s_altsyncram_q_b : UNSIGNED(31 downto 0); 
    SIGNAL s_altsyncram_q_b_2 : UNSIGNED(31 downto 0); 
BEGIN
    altsyncram : altera_mf.altera_mf_components.altsyncram
     generic map (
        address_aclr_a => "NONE",
        address_aclr_b => "NONE",
        address_reg_b => "CLOCK0",
        byteena_reg_b => "CLOCK0",
        indata_reg_b => "CLOCK0",
        intended_device_family => "Arria 10",
        numwords_a => 16384,
        numwords_b => 16384,
        operation_mode => "DUAL_PORT",
        outdata_aclr_a => "NONE",
        outdata_aclr_b => "NONE",
        outdata_reg_a => "UNREGISTERED",
        outdata_reg_b => "UNREGISTERED",
        ram_block_type => "M20K",
        rdcontrol_reg_b => "CLOCK0",
        read_during_write_mode_mixed_ports => "DONT_CARE",
        read_during_write_mode_port_a => "DONT_CARE",
        read_during_write_mode_port_b => "DONT_CARE",
        width_a => 32,
        width_b => 32,
        widthad_a => 14,
        widthad_b => 14,
        wrcontrol_aclr_a => "NONE",
        wrcontrol_aclr_b => "NONE",
        wrcontrol_wraddress_reg_b => "CLOCK0"
    )
     port map (
        clock0 => clock,
        wren_a => s_unnamed,
        data_a => STD_LOGIC_VECTOR(in_unnamed_3),
        address_a => STD_LOGIC_VECTOR(in_unnamed_5),
        address_b => STD_LOGIC_VECTOR(in_unnamed),
        UNSIGNED(q_b) => s_altsyncram_q_b
    );
    Memory_Helper0 : entity work.Memory_Helper_2(impl) port map (
        clock => clock,
        in_unnamed => in_unnamed,
        in_unnamed_2 => in_unnamed_2,
        in_unnamed_3 => in_unnamed_3,
        in_unnamed_4 => in_unnamed_4,
        in_unnamed_5 => in_unnamed_5,
        in_bypass_data_word_0_stage_0 => in_bypass_data_word_0_stage_0,
        in_unnamed_6 => s_unnamed_2,
        out_unnamed => s_unnamed,
        out_hazard_corrected_data => out_hazard_corrected_data
    );
    default_comb : PROCESS(all)
    BEGIN
        s_unnamed_2 <= s_altsyncram_q_b_delayed1;
        s_altsyncram_q_b_2 <= s_altsyncram_q_b;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_altsyncram_q_b_delayed1 <= s_altsyncram_q_b_2;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: stage_2
-- 
------------------------------------------------

ENTITY stage_2 IS 
    PORT(
        clock : IN STD_LOGIC;
        out_lastSequenceNumber : OUT UNSIGNED(63 downto 0);
        out_aboveWindow : OUT STD_LOGIC;
        in_unnamed : IN UNSIGNED(13 downto 0);
        in_unnamed_8 : IN UNSIGNED(63 downto 0);
        in_unnamed_3 : IN UNSIGNED(63 downto 0);
        in_unnamed_2 : IN STD_LOGIC;
        in_unnamed_5 : IN STD_LOGIC;
        in_unnamed_4 : IN STD_LOGIC;
        in_unnamed_6 : IN STD_LOGIC;
        in_unnamed_7 : IN UNSIGNED(13 downto 0)
    );
END stage_2;

ARCHITECTURE impl OF stage_2 IS 
    SIGNAL s_greater : STD_LOGIC; 
    SIGNAL s_lastSeq : UNSIGNED(31 downto 0); 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_unnamed_2 : UNSIGNED(31 downto 0); 
    SIGNAL s_hazard_corrected_data : UNSIGNED(31 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_0 : UNSIGNED(31 downto 0); 
    SIGNAL s_unnamed_3 : UNSIGNED(31 downto 0); 
    SIGNAL s_unnamed_4 : UNSIGNED(31 downto 0); 
    SIGNAL s_unnamed_5 : UNSIGNED(31 downto 0); 
    SIGNAL s_lastSeq_delayed1 : UNSIGNED(31 downto 0); 
BEGIN
    scl_memory0 : entity work.scl_memory_3(impl) port map (
        clock => clock,
        in_unnamed => in_unnamed,
        in_unnamed_2 => s_unnamed,
        in_unnamed_3 => s_unnamed_2,
        in_unnamed_4 => in_unnamed_6,
        in_unnamed_5 => in_unnamed_7,
        in_bypass_data_word_0_stage_0 => s_bypass_data_word_0_stage_0,
        out_hazard_corrected_data => s_hazard_corrected_data
    );
    default_comb : PROCESS(all)
    BEGIN
        s_lastSeq <= s_hazard_corrected_data;
        out_lastSequenceNumber <= (in_unnamed_8(63 downto 32) & s_lastSeq_delayed1);
        s_unnamed_2 <= s_bypass_data_word_0_stage_0;
        s_greater <= (in_unnamed_5 or (in_unnamed_2 and bool2stdlogic(s_unnamed_2 > s_lastSeq)));
        s_unnamed <= (in_unnamed_4 and s_greater);
        s_unnamed_5 <= in_unnamed_3(31 downto 0);
        s_unnamed_4 <= s_unnamed_3;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_bypass_data_word_0_stage_0 <= s_unnamed_4;
            out_aboveWindow <= s_greater;
            s_lastSeq_delayed1 <= s_lastSeq;
            s_unnamed_3 <= s_unnamed_5;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: Memory_Helper
-- Auto generated to handle various memory access issues such as read during write and read modify write hazards.
------------------------------------------------

ENTITY Memory_Helper IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bypass_data_word_0_stage_0 : IN UNSIGNED(31 downto 0);
        out_hazard_corrected_data : OUT UNSIGNED(31 downto 0);
        in_unnamed : IN UNSIGNED(31 downto 0);
        in_m_cmd_connectionIndex_delayed1 : IN UNSIGNED(13 downto 0);
        in_unnamed_2 : IN UNSIGNED(31 downto 0);
        in_unnamed_5 : IN STD_LOGIC;
        in_unnamed_3 : IN STD_LOGIC;
        out_unnamed : OUT STD_LOGIC;
        in_unnamed_4 : IN UNSIGNED(13 downto 0)
    );
END Memory_Helper;

ARCHITECTURE impl OF Memory_Helper IS 
    SIGNAL s_conflict_word_0_stage_0 : STD_LOGIC; 
    SIGNAL s_bypass_data_word_0_stage_1 : UNSIGNED(31 downto 0); 
    SIGNAL s_final_conflict_word_0 : STD_LOGIC; 
    SIGNAL s_final_override_data_word_0 : UNSIGNED(31 downto 0); 
    SIGNAL s_delayedWrEnableCounter : UNSIGNED(1 downto 0) := "01"; 
    SIGNAL s_m_cmd_connectionIndex_delayed2 : UNSIGNED(13 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_0_delayed1 : UNSIGNED(31 downto 0); 
    SIGNAL s_conflict_word_0_stage_1 : STD_LOGIC; 
    SIGNAL s_bypass_data_word_0_stage_0 : UNSIGNED(31 downto 0); 
    SIGNAL s_conflict_word_0_stage_0_delayed1 : STD_LOGIC; 
    SIGNAL s_delayedWrEnableCounterExpired_not : STD_LOGIC; 
    SIGNAL s_unnamed : UNSIGNED(1 downto 0); 
BEGIN
    default_comb : PROCESS(all)
        VARIABLE v_delayedWrEnableCounter : UNSIGNED(1 downto 0); 
        VARIABLE v_final_override_data_word_0 : UNSIGNED(31 downto 0); 
        VARIABLE v_bypass_data_word_0_stage_1 : UNSIGNED(31 downto 0); 
        VARIABLE v_hazard_corrected_data : UNSIGNED(31 downto 0); 
        VARIABLE v_delayedWrEnableCounterExpired : STD_LOGIC; 
        VARIABLE v_final_conflict_word_0 : STD_LOGIC; 
    BEGIN
        v_delayedWrEnableCounter := s_delayedWrEnableCounter;
        v_delayedWrEnableCounterExpired := v_delayedWrEnableCounter(1);
        s_bypass_data_word_0_stage_0 <= in_bypass_data_word_0_stage_0;
        v_final_conflict_word_0 := s_final_conflict_word_0;
        v_final_override_data_word_0 := s_final_override_data_word_0;

        -- If read and write addr match and read and write are enabled and write is not masked, forward write data to read output.
        IF v_final_conflict_word_0 = '1' THEN
            v_hazard_corrected_data := v_final_override_data_word_0;
        ELSE
            v_hazard_corrected_data := in_unnamed;
        END IF;

        -- If read and write addr match and read and write are enabled and write is not masked, forward write data to read output.
        out_hazard_corrected_data <= v_hazard_corrected_data;
        out_unnamed <= (v_delayedWrEnableCounterExpired and (in_unnamed_5 or in_unnamed_3));

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_0 <= (bool2stdlogic(in_m_cmd_connectionIndex_delayed1 = in_unnamed_4) and out_unnamed);

        -- Compare read and write addr for conflicts
        IF ((s_m_cmd_connectionIndex_delayed2 = in_unnamed_4) and out_unnamed = '1') THEN
            v_bypass_data_word_0_stage_1 := in_unnamed_2;
        ELSE
            v_bypass_data_word_0_stage_1 := s_bypass_data_word_0_stage_0_delayed1;
        END IF;

        -- Compare read and write addr for conflicts
        s_conflict_word_0_stage_1 <= (s_conflict_word_0_stage_0_delayed1 or (bool2stdlogic(s_m_cmd_connectionIndex_delayed2 = in_unnamed_4) and out_unnamed));
        s_bypass_data_word_0_stage_1 <= v_bypass_data_word_0_stage_1;
        s_unnamed <= (v_delayedWrEnableCounter - "01");
        s_delayedWrEnableCounterExpired_not <= ( not v_delayedWrEnableCounterExpired);
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            IF (s_delayedWrEnableCounterExpired_not = '1') THEN
                s_delayedWrEnableCounter <= s_unnamed;
            END IF;
            s_m_cmd_connectionIndex_delayed2 <= in_m_cmd_connectionIndex_delayed1;
            s_conflict_word_0_stage_0_delayed1 <= s_conflict_word_0_stage_0;
            s_bypass_data_word_0_stage_0_delayed1 <= s_bypass_data_word_0_stage_0;
            s_final_conflict_word_0 <= s_conflict_word_0_stage_1;
            s_final_override_data_word_0 <= s_bypass_data_word_0_stage_1;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY altera_mf;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_memory_2
-- 
------------------------------------------------

ENTITY scl_memory_2 IS 
    PORT(
        clock : IN STD_LOGIC;
        in_bypass_data_word_0_stage_0 : IN UNSIGNED(31 downto 0);
        out_hazard_corrected_data : OUT UNSIGNED(31 downto 0);
        in_m_cmd_connectionIndex_delayed1 : IN UNSIGNED(13 downto 0);
        in_unnamed : IN UNSIGNED(31 downto 0);
        in_unnamed_4 : IN STD_LOGIC;
        in_unnamed_2 : IN STD_LOGIC;
        in_unnamed_3 : IN UNSIGNED(13 downto 0)
    );
END scl_memory_2;

ARCHITECTURE impl OF scl_memory_2 IS 
    SIGNAL s_unnamed : UNSIGNED(31 downto 0); 
    SIGNAL s_altsyncram_q_b_delayed1 : UNSIGNED(31 downto 0) := "00000000000000000000000000000000"; 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_altsyncram_q_b : UNSIGNED(31 downto 0); 
    SIGNAL s_altsyncram_q_b_2 : UNSIGNED(31 downto 0); 
BEGIN
    altsyncram : altera_mf.altera_mf_components.altsyncram
     generic map (
        address_aclr_a => "NONE",
        address_aclr_b => "NONE",
        address_reg_b => "CLOCK0",
        byteena_reg_b => "CLOCK0",
        indata_reg_b => "CLOCK0",
        intended_device_family => "Arria 10",
        numwords_a => 16384,
        numwords_b => 16384,
        operation_mode => "DUAL_PORT",
        outdata_aclr_a => "NONE",
        outdata_aclr_b => "NONE",
        outdata_reg_a => "UNREGISTERED",
        outdata_reg_b => "UNREGISTERED",
        ram_block_type => "M20K",
        rdcontrol_reg_b => "CLOCK0",
        read_during_write_mode_mixed_ports => "DONT_CARE",
        read_during_write_mode_port_a => "DONT_CARE",
        read_during_write_mode_port_b => "DONT_CARE",
        width_a => 32,
        width_b => 32,
        widthad_a => 14,
        widthad_b => 14,
        wrcontrol_aclr_a => "NONE",
        wrcontrol_aclr_b => "NONE",
        wrcontrol_wraddress_reg_b => "CLOCK0"
    )
     port map (
        clock0 => clock,
        wren_a => s_unnamed_2,
        data_a => STD_LOGIC_VECTOR(in_unnamed),
        address_a => STD_LOGIC_VECTOR(in_unnamed_3),
        address_b => STD_LOGIC_VECTOR(in_m_cmd_connectionIndex_delayed1),
        UNSIGNED(q_b) => s_altsyncram_q_b
    );
    Memory_Helper0 : entity work.Memory_Helper(impl) port map (
        clock => clock,
        in_bypass_data_word_0_stage_0 => in_bypass_data_word_0_stage_0,
        in_unnamed => s_unnamed,
        in_unnamed_2 => in_unnamed,
        in_m_cmd_connectionIndex_delayed1 => in_m_cmd_connectionIndex_delayed1,
        in_unnamed_3 => in_unnamed_2,
        in_unnamed_4 => in_unnamed_3,
        in_unnamed_5 => in_unnamed_4,
        out_hazard_corrected_data => out_hazard_corrected_data,
        out_unnamed => s_unnamed_2
    );
    default_comb : PROCESS(all)
    BEGIN
        s_unnamed <= s_altsyncram_q_b_delayed1;
        s_altsyncram_q_b_2 <= s_altsyncram_q_b;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_altsyncram_q_b_delayed1 <= s_altsyncram_q_b_2;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: stage
-- 
------------------------------------------------

ENTITY stage IS 
    PORT(
        clock : IN STD_LOGIC;
        in_m_cmd_sequenceNumber : IN UNSIGNED(63 downto 0);
        in_const_0 : IN UNSIGNED(63 downto 0);
        in_m_cmd_connectionIndex_delayed1 : IN UNSIGNED(13 downto 0);
        in_const_1 : IN STD_LOGIC;
        in_const_0_2 : IN STD_LOGIC;
        in_unnamed_2 : IN STD_LOGIC;
        in_unnamed_3 : IN STD_LOGIC;
        in_unnamed : IN UNSIGNED(13 downto 0);
        out_unnamed_3 : OUT UNSIGNED(63 downto 0);
        out_unnamed : OUT STD_LOGIC;
        out_unnamed_2 : OUT STD_LOGIC
    );
END stage;

ARCHITECTURE impl OF stage IS 
    SIGNAL s_equal : STD_LOGIC; 
    SIGNAL s_lastSeq : UNSIGNED(31 downto 0); 
    SIGNAL s_greater : STD_LOGIC; 
    SIGNAL s_m_cmd_sequenceNumber_rewired_delayed1 : UNSIGNED(31 downto 0); 
    SIGNAL s_bypass_data_word_0_stage_0 : UNSIGNED(31 downto 0); 
    SIGNAL s_hazard_corrected_data : UNSIGNED(31 downto 0); 
    SIGNAL s_equal_delayed1 : STD_LOGIC; 
    SIGNAL s_greater_delayed1 : STD_LOGIC; 
    SIGNAL s_unnamed : UNSIGNED(31 downto 0); 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_equal_delayed2 : STD_LOGIC; 
    SIGNAL s_greater_delayed2 : STD_LOGIC; 
    SIGNAL s_m_cmd_sequenceNumber_rewired : UNSIGNED(31 downto 0); 
    SIGNAL s_m_cmd_sequenceNumber_rewired_delayed1_2 : UNSIGNED(31 downto 0); 
    SIGNAL s_lastSeq_delayed1 : UNSIGNED(31 downto 0); 
    SIGNAL s_lastSeq_delayed2 : UNSIGNED(31 downto 0); 
    SIGNAL s_m_cmd_sequenceNumber_rewired_delayed2 : UNSIGNED(31 downto 0); 
    SIGNAL s_m_cmd_sequenceNumber_rewired_delayed1_3 : UNSIGNED(31 downto 0); 
    SIGNAL s_greater_delayed1_2 : STD_LOGIC; 
    SIGNAL s_m_cmd_sequenceNumber_rewired_delayed1_4 : UNSIGNED(31 downto 0); 
    SIGNAL s_equal_delayed1_2 : STD_LOGIC; 
    SIGNAL s_lastSeq_delayed1_2 : UNSIGNED(31 downto 0); 
    SIGNAL s_lastSeq_delayed3 : UNSIGNED(31 downto 0); 
    SIGNAL s_lastSeq_delayed2_2 : UNSIGNED(31 downto 0); 
BEGIN
    scl_memory0 : entity work.scl_memory_2(impl) port map (
        clock => clock,
        in_bypass_data_word_0_stage_0 => s_bypass_data_word_0_stage_0,
        in_unnamed => s_unnamed,
        in_m_cmd_connectionIndex_delayed1 => in_m_cmd_connectionIndex_delayed1,
        in_unnamed_2 => s_unnamed_2,
        in_unnamed_3 => in_unnamed,
        in_unnamed_4 => in_unnamed_3,
        out_hazard_corrected_data => s_hazard_corrected_data
    );
    default_comb : PROCESS(all)
    BEGIN
        s_lastSeq <= s_hazard_corrected_data;
        s_equal <= (in_const_1 and bool2stdlogic(s_m_cmd_sequenceNumber_rewired_delayed2 = s_lastSeq));
        s_unnamed <= s_bypass_data_word_0_stage_0;
        s_greater <= (in_const_0_2 or (in_const_1 and bool2stdlogic(s_unnamed > s_lastSeq)));
        s_unnamed_2 <= (in_unnamed_2 and s_greater);
        out_unnamed_3 <= (s_lastSeq_delayed3 & in_const_0(31 downto 0));
        out_unnamed <= s_equal_delayed2;
        out_unnamed_2 <= s_greater_delayed2;
        s_m_cmd_sequenceNumber_rewired <= in_m_cmd_sequenceNumber(63 downto 32);
        s_m_cmd_sequenceNumber_rewired_delayed1_2 <= s_m_cmd_sequenceNumber_rewired_delayed1_3;
        s_lastSeq_delayed1 <= s_lastSeq_delayed1_2;
        s_lastSeq_delayed2 <= s_lastSeq_delayed2_2;
        s_m_cmd_sequenceNumber_rewired_delayed1_4 <= s_m_cmd_sequenceNumber_rewired_delayed1;
        s_equal_delayed1_2 <= s_equal_delayed1;
        s_greater_delayed1_2 <= s_greater_delayed1;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_m_cmd_sequenceNumber_rewired_delayed1_3 <= s_m_cmd_sequenceNumber_rewired;
            s_equal_delayed1 <= s_equal;
            s_lastSeq_delayed3 <= s_lastSeq_delayed2;
            s_greater_delayed2 <= s_greater_delayed1_2;
            s_lastSeq_delayed2_2 <= s_lastSeq_delayed1;
            s_equal_delayed2 <= s_equal_delayed1_2;
            s_bypass_data_word_0_stage_0 <= s_m_cmd_sequenceNumber_rewired_delayed1_4;
            s_lastSeq_delayed1_2 <= s_lastSeq;
            s_greater_delayed1 <= s_greater;
            s_m_cmd_sequenceNumber_rewired_delayed2 <= s_m_cmd_sequenceNumber_rewired_delayed1_2;
            s_m_cmd_sequenceNumber_rewired_delayed1 <= s_m_cmd_sequenceNumber_rewired;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: lastSequenceNumber
-- 
------------------------------------------------

ENTITY lastSequenceNumber IS 
    PORT(
        clock : IN STD_LOGIC;
        in_m_cmd_sequenceNumber : IN UNSIGNED(63 downto 0);
        in_connectionActive : IN STD_LOGIC;
        out_aboveWindow : OUT STD_LOGIC;
        out_lastSequenceNumber : OUT UNSIGNED(63 downto 0);
        in_m_cmd_opcode_delayed1 : IN UNSIGNED(2 downto 0);
        in_m_cmd_valid_delayed1 : IN STD_LOGIC;
        in_m_cmd_connectionIndex_delayed1 : IN UNSIGNED(13 downto 0);
        in_unnamed : IN UNSIGNED(13 downto 0);
        in_unnamed_2 : IN UNSIGNED(13 downto 0);
        in_unnamed_3 : IN UNSIGNED(63 downto 0);
        in_unnamed_4 : IN UNSIGNED(13 downto 0);
        out_unnamed : OUT STD_LOGIC;
        out_aboveWindow_delayed3 : OUT STD_LOGIC;
        out_lastSequenceNumber_delayed3 : OUT UNSIGNED(63 downto 0)
    );
END lastSequenceNumber;

ARCHITECTURE impl OF lastSequenceNumber IS 
    SIGNAL s_const_0 : UNSIGNED(63 downto 0); 
    SIGNAL s_lastSequenceNumber : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2 : STD_LOGIC := '0'; 
    SIGNAL s_const_1 : STD_LOGIC; 
    SIGNAL s_const_0_2 : STD_LOGIC; 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_unnamed_3 : STD_LOGIC; 
    SIGNAL s_unnamed_4 : STD_LOGIC; 
    SIGNAL s_unnamed_5 : STD_LOGIC; 
    SIGNAL s_unnamed_6 : STD_LOGIC; 
    SIGNAL s_unnamed_7 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed4 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed4 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed3 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed3 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_2 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_2 : STD_LOGIC; 
    SIGNAL s_lastSequenceNumber_delayed1 : UNSIGNED(63 downto 0); 
    SIGNAL s_aboveWindow_delayed1 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed4 : STD_LOGIC; 
    SIGNAL s_lastSequenceNumber_delayed2 : UNSIGNED(63 downto 0); 
    SIGNAL s_aboveWindow_delayed6 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed5 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed8 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed7 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed9 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed8_2 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2_2 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2_2 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_3 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_3 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_4 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_4 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2_3 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2_3 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed3_2 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed3_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed7_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed6_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed5_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed4_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_delayed3 : STD_LOGIC; 
    SIGNAL s_lastSequenceNumber_delayed3 : UNSIGNED(63 downto 0); 
    SIGNAL s_lastSequenceNumber_delayed2_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_aboveWindow_delayed2_2 : STD_LOGIC; 
    SIGNAL s_lastSequenceNumber_delayed1_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_aboveWindow_delayed1_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow : STD_LOGIC; 
BEGIN
    stage0 : entity work.stage(impl) port map (
        clock => clock,
        in_m_cmd_sequenceNumber => in_m_cmd_sequenceNumber,
        in_const_0 => s_const_0,
        in_const_1 => s_const_1,
        in_const_0_2 => s_const_0_2,
        in_m_cmd_connectionIndex_delayed1 => in_m_cmd_connectionIndex_delayed1,
        in_unnamed => in_unnamed,
        in_unnamed_2 => s_unnamed_2,
        in_unnamed_3 => s_unnamed_4,
        out_unnamed => s_unnamed,
        out_unnamed_2 => s_unnamed_5,
        out_unnamed_3 => s_unnamed_7
    );
    default_comb : PROCESS(all)
        CONSTANT C_OPUPDATE : UNSIGNED(2 downto 0) := "000"; 
        CONSTANT C_OPSETSEQNR : UNSIGNED(2 downto 0) := "010"; 
    BEGIN
        s_const_0 <= "0000000000000000000000000000000000000000000000000000000000000000";
        out_aboveWindow <= s_aboveWindow;
        out_lastSequenceNumber <= s_lastSequenceNumber;
        s_const_1 <= '1';
        s_const_0_2 <= '0';
        s_unnamed_2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2;
        s_unnamed_4 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2;
        s_unnamed_3 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed4;
        s_unnamed_6 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed4;
        out_unnamed <= s_aboveWindow_delayed9;
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr <= (in_m_cmd_valid_delayed1 and bool2stdlogic(in_m_cmd_opcode_delayed1 = C_OPSETSEQNR));
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive <= ((in_m_cmd_valid_delayed1 and bool2stdlogic(in_m_cmd_opcode_delayed1 = C_OPUPDATE)) and in_connectionActive);
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_3;
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_3;
        s_aboveWindow_delayed1 <= s_aboveWindow_delayed1_2;
        s_lastSequenceNumber_delayed1 <= s_lastSequenceNumber_delayed1_2;
        s_aboveWindow_delayed2 <= s_aboveWindow_delayed2_2;
        s_lastSequenceNumber_delayed2 <= s_lastSequenceNumber_delayed2_2;
        out_aboveWindow_delayed3 <= s_aboveWindow_delayed3;
        out_lastSequenceNumber_delayed3 <= s_lastSequenceNumber_delayed3;
        s_aboveWindow_delayed4 <= s_aboveWindow_delayed4_2;
        s_aboveWindow_delayed5 <= s_aboveWindow_delayed5_2;
        s_aboveWindow_delayed6 <= s_aboveWindow_delayed6_2;
        s_aboveWindow_delayed7 <= s_aboveWindow_delayed7_2;
        s_aboveWindow_delayed8 <= s_aboveWindow_delayed8_2;
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_4 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1;
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_4 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1;
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2_3 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2_2;
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2_3 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2_2;
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed3_2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed3;
        s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed3_2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed3;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2_2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_2;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_3 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_3 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2_2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_2;
            s_lastSequenceNumber_delayed2_2 <= s_lastSequenceNumber_delayed1;
            s_aboveWindow_delayed2_2 <= s_aboveWindow_delayed1;
            s_lastSequenceNumber_delayed3 <= s_lastSequenceNumber_delayed2;
            s_aboveWindow_delayed1_2 <= out_aboveWindow;
            s_lastSequenceNumber_delayed1_2 <= out_lastSequenceNumber;
            s_aboveWindow_delayed5_2 <= s_aboveWindow_delayed4;
            s_aboveWindow_delayed9 <= s_aboveWindow_delayed8;
            s_aboveWindow_delayed4_2 <= out_aboveWindow_delayed3;
            s_aboveWindow_delayed8_2 <= s_aboveWindow_delayed7;
            s_aboveWindow_delayed6_2 <= s_aboveWindow_delayed5;
            s_aboveWindow_delayed3 <= s_aboveWindow_delayed2;
            s_aboveWindow_delayed7_2 <= s_aboveWindow_delayed6;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed1_4;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed1_4;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed3 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed2_3;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed3 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed2_3;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed4 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpUpdate_and_connectionActive_delayed3_2;
            s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed4 <= s_m_cmd_valid_delayed1_and_m_cmd_opcode_delayed1_eq_OpSetSeqNr_delayed3_2;
        END IF;
    END PROCESS;

    stage1 : entity work.stage_2(impl) port map (
        clock => clock,
        in_unnamed => in_unnamed_2,
        in_unnamed_2 => s_unnamed,
        in_unnamed_3 => in_unnamed_3,
        in_unnamed_4 => s_unnamed_3,
        in_unnamed_5 => s_unnamed_5,
        in_unnamed_6 => s_unnamed_6,
        in_unnamed_7 => in_unnamed_4,
        in_unnamed_8 => s_unnamed_7,
        out_lastSequenceNumber => s_lastSequenceNumber,
        out_aboveWindow => s_aboveWindow
    );
END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY altera_mf;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: scl_memory
-- 
------------------------------------------------

ENTITY scl_memory IS 
    PORT(
        clock : IN STD_LOGIC;
        in_m_cmd_connectionIndex : IN UNSIGNED(13 downto 0);
        out_unnamed : OUT UNSIGNED(0 downto 0);
        in_m_cmd_valid_and_set_activate : IN STD_LOGIC;
        in_m_cmd_opcode_eq_OpActivate_bit_0 : IN UNSIGNED(0 downto 0)
    );
END scl_memory;

ARCHITECTURE impl OF scl_memory IS 
    SIGNAL s_altsyncram_q_b : UNSIGNED(0 downto 0); 
BEGIN
    altsyncram : altera_mf.altera_mf_components.altsyncram
     generic map (
        address_aclr_a => "NONE",
        address_aclr_b => "NONE",
        address_reg_b => "CLOCK0",
        byteena_reg_b => "CLOCK0",
        indata_reg_b => "CLOCK0",
        intended_device_family => "Arria 10",
        numwords_a => 16384,
        numwords_b => 16384,
        operation_mode => "DUAL_PORT",
        outdata_aclr_a => "NONE",
        outdata_aclr_b => "NONE",
        outdata_reg_a => "UNREGISTERED",
        outdata_reg_b => "UNREGISTERED",
        ram_block_type => "M20K",
        rdcontrol_reg_b => "CLOCK0",
        read_during_write_mode_mixed_ports => "OLD_DATA",
        read_during_write_mode_port_a => "DONT_CARE",
        read_during_write_mode_port_b => "DONT_CARE",
        width_a => 1,
        width_b => 1,
        widthad_a => 14,
        widthad_b => 14,
        wrcontrol_aclr_a => "NONE",
        wrcontrol_aclr_b => "NONE",
        wrcontrol_wraddress_reg_b => "CLOCK0"
    )
     port map (
        clock0 => clock,
        wren_a => in_m_cmd_valid_and_set_activate,
        data_a => STD_LOGIC_VECTOR(in_m_cmd_opcode_eq_OpActivate_bit_0),
        address_a => STD_LOGIC_VECTOR(in_m_cmd_connectionIndex),
        address_b => STD_LOGIC_VECTOR(in_m_cmd_connectionIndex),
        UNSIGNED(q_b) => s_altsyncram_q_b
    );
    default_comb : PROCESS(all)
    BEGIN
        out_unnamed <= s_altsyncram_q_b;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: active
-- 
------------------------------------------------

ENTITY active IS 
    PORT(
        clock : IN STD_LOGIC;
        in_m_cmd_valid : IN STD_LOGIC;
        in_m_cmd_opcode : IN UNSIGNED(2 downto 0);
        in_m_cmd_connectionIndex : IN UNSIGNED(13 downto 0);
        out_connectionActive : OUT STD_LOGIC;
        out_unnamed_2 : OUT STD_LOGIC;
        out_unnamed : OUT STD_LOGIC
    );
END active;

ARCHITECTURE impl OF active IS 
    SIGNAL s_unnamed : UNSIGNED(0 downto 0); 
    SIGNAL s_m_cmd_valid_and_set_activate : STD_LOGIC; 
    SIGNAL s_m_cmd_opcode_eq_OpActivate_bit_0 : UNSIGNED(0 downto 0); 
    SIGNAL s_connectionActive_delayed1 : STD_LOGIC; 
    SIGNAL s_connectionActive_delayed4 : STD_LOGIC; 
    SIGNAL s_connectionActive_delayed2 : STD_LOGIC; 
    SIGNAL s_connectionActive_delayed3 : STD_LOGIC; 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_unnamed_3 : STD_LOGIC; 
    SIGNAL s_unnamed_4 : STD_LOGIC; 
    SIGNAL s_unnamed_5 : STD_LOGIC; 
    SIGNAL s_unnamed_6 : STD_LOGIC; 
    SIGNAL s_unnamed_7 : STD_LOGIC; 
    SIGNAL s_unnamed_8 : STD_LOGIC; 
    SIGNAL s_unnamed_9 : STD_LOGIC; 
    SIGNAL s_unnamed_10 : STD_LOGIC; 
    SIGNAL s_unnamed_11 : STD_LOGIC; 
    SIGNAL s_connectionActive_delayed1_2 : STD_LOGIC; 
    SIGNAL s_connectionActive_delayed2_2 : STD_LOGIC; 
    SIGNAL s_unnamed_12 : STD_LOGIC; 
    SIGNAL s_unnamed_13 : STD_LOGIC; 
    SIGNAL s_unnamed_14 : STD_LOGIC; 
    SIGNAL s_unnamed_15 : STD_LOGIC; 
    SIGNAL s_unnamed_16 : STD_LOGIC; 
    SIGNAL s_unnamed_17 : STD_LOGIC; 
    SIGNAL s_connectionActive_delayed3_2 : STD_LOGIC; 
    SIGNAL s_unnamed_18 : STD_LOGIC; 
    SIGNAL s_connectionActive_delayed5 : STD_LOGIC; 
    SIGNAL s_connectionActive_delayed4_2 : STD_LOGIC; 
BEGIN
    scl_memory0 : entity work.scl_memory(impl) port map (
        clock => clock,
        in_m_cmd_connectionIndex => in_m_cmd_connectionIndex,
        in_m_cmd_valid_and_set_activate => s_m_cmd_valid_and_set_activate,
        in_m_cmd_opcode_eq_OpActivate_bit_0 => s_m_cmd_opcode_eq_OpActivate_bit_0,
        out_unnamed => s_unnamed
    );
    default_comb : PROCESS(all)
        CONSTANT C_OPACTIVATE : UNSIGNED(2 downto 0) := "101"; 
        CONSTANT C_OPDEACTIVATE : UNSIGNED(2 downto 0) := "001"; 
        CONSTANT C_OPACTIVATE_2 : UNSIGNED(2 downto 0) := "101"; 
        VARIABLE v_set_activate : STD_LOGIC; 
    BEGIN
        out_connectionActive <= s_unnamed(0);
        v_set_activate := (bool2stdlogic(in_m_cmd_opcode = C_OPACTIVATE_2) or bool2stdlogic(in_m_cmd_opcode = C_OPDEACTIVATE));
        s_m_cmd_valid_and_set_activate <= (in_m_cmd_valid and v_set_activate);
        s_m_cmd_opcode_eq_OpActivate_bit_0 <= (0 => bool2stdlogic(in_m_cmd_opcode = C_OPACTIVATE));
        out_unnamed_2 <= s_connectionActive_delayed5;
        out_unnamed <= s_unnamed_10;
        s_connectionActive_delayed1 <= s_connectionActive_delayed1_2;
        s_connectionActive_delayed2 <= s_connectionActive_delayed2_2;
        s_connectionActive_delayed3 <= s_connectionActive_delayed3_2;
        s_connectionActive_delayed4 <= s_connectionActive_delayed4_2;
        s_unnamed_2 <= s_unnamed_18;
        s_unnamed_5 <= s_unnamed_17;
        s_unnamed_3 <= s_unnamed_16;
        s_unnamed_4 <= s_unnamed_15;
        s_unnamed_6 <= s_unnamed_14;
        s_unnamed_7 <= s_unnamed_13;
        s_unnamed_9 <= s_unnamed_12;
        s_unnamed_8 <= s_unnamed_11;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_connectionActive_delayed1_2 <= out_connectionActive;
            s_unnamed_18 <= out_unnamed_2;
            s_unnamed_16 <= s_unnamed_5;
            s_connectionActive_delayed5 <= s_connectionActive_delayed4;
            s_unnamed_12 <= s_unnamed_7;
            s_unnamed_15 <= s_unnamed_3;
            s_connectionActive_delayed4_2 <= s_connectionActive_delayed3;
            s_unnamed_13 <= s_unnamed_6;
            s_unnamed_17 <= s_unnamed_2;
            s_unnamed_14 <= s_unnamed_4;
            s_connectionActive_delayed3_2 <= s_connectionActive_delayed2;
            s_unnamed_11 <= s_unnamed_9;
            s_unnamed_10 <= s_unnamed_8;
            s_connectionActive_delayed2_2 <= s_connectionActive_delayed1;
        END IF;
    END PROCESS;

END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: AntiReplay
-- 
------------------------------------------------

ENTITY AntiReplay IS 
    PORT(
        clock : IN STD_LOGIC;
        out_out_valid : OUT STD_LOGIC;
        out_out_code : OUT UNSIGNED(2 downto 0);
        bitmap_rd_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_rd_read : OUT STD_LOGIC;
        bitmap_rd_readdata : IN STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_wr_writedata : OUT STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_write : OUT STD_LOGIC;
        in_in_cmd_valid_and_m_inReady_2 : IN STD_LOGIC;
        in_in_cmd_opcode : IN UNSIGNED(2 downto 0);
        in_in_cmd_connectionIndex : IN UNSIGNED(13 downto 0);
        in_in_cmd_sequenceNumber : IN UNSIGNED(63 downto 0);
        in_in_cmd_windowSize : IN UNSIGNED(23 downto 0)
    );
END AntiReplay;

ARCHITECTURE impl OF AntiReplay IS 
    SIGNAL s_m_cmd_opcode : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_connectionIndex : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_sequenceNumber : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_windowSize : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_valid : STD_LOGIC; 
    SIGNAL s_connectionActive : STD_LOGIC; 
    SIGNAL s_lastSequenceNumber : UNSIGNED(63 downto 0); 
    SIGNAL s_inPhysicalWindow : STD_LOGIC; 
    SIGNAL s_aboveWindow : STD_LOGIC; 
    SIGNAL s_updateState : STD_LOGIC; 
    SIGNAL s_inVirtualWindow : STD_LOGIC; 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_secondStageValid : STD_LOGIC; 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_unnamed_3 : STD_LOGIC; 
    SIGNAL s_m_cmd_connectionIndex_delayed2 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_connectionIndex_delayed3 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_4 : STD_LOGIC; 
    SIGNAL s_unnamed_5 : STD_LOGIC; 
    SIGNAL s_unnamed_6 : STD_LOGIC; 
    SIGNAL s_bitmap : UNSIGNED(255 downto 0); 
    SIGNAL s_m_cmd_valid_delayed10 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed14 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed12 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed13 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_valid_delayed11 : STD_LOGIC := '0'; 
    SIGNAL s_m_cmd_opcode_delayed1 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_connectionIndex_delayed1 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_valid_delayed1 : STD_LOGIC; 
    SIGNAL s_aboveWindow_not : STD_LOGIC; 
    SIGNAL s_unnamed_7 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_8 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_9 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_10 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_11 : STD_LOGIC; 
    SIGNAL s_unnamed_12 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_13 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_14 : STD_LOGIC; 
    SIGNAL s_unnamed_15 : STD_LOGIC; 
    SIGNAL s_unnamed_16 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_17 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_18 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_19 : UNSIGNED(13 downto 0); 
    SIGNAL s_updateState_delayed2 : STD_LOGIC; 
    SIGNAL s_unnamed_20 : STD_LOGIC; 
    SIGNAL s_unnamed_21 : STD_LOGIC; 
    SIGNAL s_unnamed_22 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_valid_delayed7 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_23 : STD_LOGIC; 
    SIGNAL s_updateState_delayed1 : STD_LOGIC; 
    SIGNAL s_unnamed_24 : STD_LOGIC; 
    SIGNAL s_unnamed_25 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_valid_delayed8 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_26 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_27 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_28 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_29 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_30 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_valid_delayed3 : STD_LOGIC; 
    SIGNAL s_unnamed_31 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_valid_delayed2 : STD_LOGIC; 
    SIGNAL s_m_cmd_opcode_delayed2 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_32 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_33 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_opcode_delayed3 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_valid_delayed5 : STD_LOGIC; 
    SIGNAL s_unnamed_34 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_sequenceNumber_delayed1 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_opcode_delayed4 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_35 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_valid_delayed4 : STD_LOGIC; 
    SIGNAL s_m_cmd_connectionIndex_delayed2_2 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_valid_delayed6 : STD_LOGIC; 
    SIGNAL s_aboveWindow_not_delayed1 : STD_LOGIC; 
    SIGNAL s_unnamed_36 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_37 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_opcode_delayed5 : UNSIGNED(2 downto 0); 
    SIGNAL s_updateState_delayed1_2 : STD_LOGIC; 
    SIGNAL s_unnamed_38 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_valid_delayed7_2 : STD_LOGIC; 
    SIGNAL s_unnamed_39 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_40 : UNSIGNED(23 downto 0); 
    SIGNAL s_updateState_delayed2_2 : STD_LOGIC; 
    SIGNAL s_unnamed_41 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_42 : UNSIGNED(13 downto 0); 
    SIGNAL s_aboveWindow_not_delayed3 : STD_LOGIC; 
    SIGNAL s_unnamed_43 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_44 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_valid_delayed8_2 : STD_LOGIC; 
    SIGNAL s_unnamed_45 : UNSIGNED(23 downto 0); 
    SIGNAL s_aboveWindow_delayed3 : STD_LOGIC; 
    SIGNAL s_aboveWindow_not_delayed4 : STD_LOGIC; 
    SIGNAL s_unnamed_46 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_47 : UNSIGNED(63 downto 0); 
    SIGNAL s_lastSequenceNumber_delayed3 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_48 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_49 : UNSIGNED(13 downto 0); 
    SIGNAL s_aboveWindow_not_delayed2 : STD_LOGIC; 
    SIGNAL s_unnamed_50 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed12_2 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed10_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_not_delayed5 : STD_LOGIC; 
    SIGNAL s_unnamed_51 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_52 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_valid_delayed11_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_not_delayed6 : STD_LOGIC; 
    SIGNAL s_unnamed_53 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_54 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_valid_delayed14_2 : STD_LOGIC; 
    SIGNAL s_unnamed_55 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_56 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_57 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_valid_delayed13_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_not_delayed7 : STD_LOGIC; 
    SIGNAL s_unnamed_58 : UNSIGNED(2 downto 0); 
    SIGNAL s_aboveWindow_not_delayed8 : STD_LOGIC; 
    SIGNAL s_unnamed_59 : UNSIGNED(2 downto 0); 
    SIGNAL s_secondStageValid_delayed6 : STD_LOGIC; 
    SIGNAL s_unnamed_60 : UNSIGNED(2 downto 0); 
    SIGNAL s_in_cmd_windowSize_delayed1 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_isReplay : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed15 : STD_LOGIC := '0'; 
    SIGNAL s_isReplay : STD_LOGIC; 
    SIGNAL s_aboveWindow_not_delayed9 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed14_3 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_61 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_62 : UNSIGNED(2 downto 0); 
    SIGNAL s_aboveWindow_not_delayed8_2 : STD_LOGIC; 
    SIGNAL s_bitAddr : UNSIGNED(7 downto 0); 
    SIGNAL s_m_cmd_connectionIndex_delayed3_2 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_63 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_opcode_delayed2_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_valid_delayed3_2 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_64 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_connectionIndex_delayed2_3 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_65 : UNSIGNED(5 downto 0); 
    SIGNAL s_m_cmd_sequenceNumber_delayed1_2 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_66 : STD_LOGIC; 
    SIGNAL s_m_cmd_sequenceNumber_delayed2 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_valid_delayed2_2 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_67 : STD_LOGIC; 
    SIGNAL s_unnamed_68 : STD_LOGIC; 
    SIGNAL s_m_cmd_opcode_delayed3_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_connectionIndex_delayed1_2 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_valid_delayed1_2 : STD_LOGIC := '0'; 
    SIGNAL s_in_cmd_valid_and_m_inReady_2_delayed1 : STD_LOGIC := '0'; 
    SIGNAL s_in_cmd_opcode_delayed1 : UNSIGNED(2 downto 0); 
    SIGNAL s_in_cmd_connectionIndex_delayed1 : UNSIGNED(13 downto 0); 
    SIGNAL s_in_cmd_sequenceNumber_delayed1 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_opcode_delayed1_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_69 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_70 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_71 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_connectionIndex_delayed2_4 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_72 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_73 : STD_LOGIC; 
    SIGNAL s_updateState_delayed1_3 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed7_3 : STD_LOGIC; 
    SIGNAL s_unnamed_74 : STD_LOGIC; 
    SIGNAL s_unnamed_75 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_76 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed11_3 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed10_3 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed12_3 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed13_3 : STD_LOGIC; 
    SIGNAL s_unnamed_77 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_78 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_valid_delayed13_4 : STD_LOGIC := '0'; 
    SIGNAL s_aboveWindow_not_delayed7_2 : STD_LOGIC; 
    SIGNAL s_m_cmd_valid_delayed12_4 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_79 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_80 : UNSIGNED(2 downto 0); 
    SIGNAL s_aboveWindow_not_delayed6_2 : STD_LOGIC; 
    SIGNAL s_unnamed_81 : UNSIGNED(23 downto 0); 
    SIGNAL s_aboveWindow_not_delayed5_2 : STD_LOGIC; 
    SIGNAL s_unnamed_82 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_valid_delayed11_4 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_83 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_cmd_valid_delayed10_4 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_84 : UNSIGNED(23 downto 0); 
    SIGNAL s_aboveWindow_not_delayed4_2 : STD_LOGIC; 
    SIGNAL s_unnamed_85 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_86 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_87 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_88 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_valid_delayed9_2 : STD_LOGIC := '0'; 
    SIGNAL s_aboveWindow_not_delayed3_2 : STD_LOGIC; 
    SIGNAL s_updateState_2 : STD_LOGIC; 
    SIGNAL s_aboveWindow_not_delayed2_2 : STD_LOGIC; 
    SIGNAL s_unnamed_89 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_90 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_91 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_valid_delayed8_3 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_92 : UNSIGNED(13 downto 0); 
    SIGNAL s_updateState_delayed2_3 : STD_LOGIC; 
    SIGNAL s_unnamed_93 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_94 : UNSIGNED(13 downto 0); 
    SIGNAL s_unnamed_95 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_valid_delayed4_2 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_96 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_opcode_delayed4_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_97 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_98 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_99 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_valid_delayed7_4 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_100 : UNSIGNED(63 downto 0); 
    SIGNAL s_aboveWindow_not_delayed1_2 : STD_LOGIC; 
    SIGNAL s_updateState_delayed1_4 : STD_LOGIC; 
    SIGNAL s_unnamed_101 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_valid_delayed6_2 : STD_LOGIC := '0'; 
    SIGNAL s_unnamed_102 : UNSIGNED(63 downto 0); 
    SIGNAL s_m_cmd_opcode_delayed6 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_103 : UNSIGNED(23 downto 0); 
    SIGNAL s_m_cmd_opcode_delayed5_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_unnamed_104 : UNSIGNED(63 downto 0); 
    SIGNAL s_unnamed_105 : UNSIGNED(23 downto 0); 
    SIGNAL s_unnamed_106 : UNSIGNED(13 downto 0); 
    SIGNAL s_m_cmd_valid_delayed5_2 : STD_LOGIC := '0'; 
BEGIN
    active0 : entity work.active(impl) port map (
        clock => clock,
        in_m_cmd_opcode => s_m_cmd_opcode,
        in_m_cmd_connectionIndex => s_m_cmd_connectionIndex,
        in_m_cmd_valid => s_m_cmd_valid,
        out_connectionActive => s_connectionActive,
        out_unnamed => s_unnamed_6,
        out_unnamed_2 => s_unnamed_14
    );
    default_comb : PROCESS(all)
        CONSTANT C_OPSETSEQNR : UNSIGNED(2 downto 0) := "010"; 
        CONSTANT C_OPUPDATE : UNSIGNED(2 downto 0) := "000"; 
        CONSTANT C_OPSETSEQNR_2 : UNSIGNED(2 downto 0) := "010"; 
        CONSTANT C_OPUPDATE_2 : UNSIGNED(2 downto 0) := "000"; 
        VARIABLE v_updateState : STD_LOGIC; 
    BEGIN
        s_m_cmd_valid <= s_in_cmd_valid_and_m_inReady_2_delayed1;
        s_m_cmd_opcode <= s_in_cmd_opcode_delayed1;
        s_m_cmd_connectionIndex <= s_in_cmd_connectionIndex_delayed1;
        s_m_cmd_sequenceNumber <= s_unnamed_70;
        s_m_cmd_windowSize <= s_unnamed_72;
        s_m_isReplay <= s_isReplay;
        s_m_cmd_opcode_delayed1 <= s_m_cmd_opcode_delayed1_2;
        s_m_cmd_valid_delayed1 <= s_m_cmd_valid_delayed1_2;
        s_m_cmd_connectionIndex_delayed1 <= s_m_cmd_connectionIndex_delayed1_2;
        s_unnamed_7 <= s_m_cmd_connectionIndex_delayed3;
        s_unnamed_8 <= s_m_cmd_connectionIndex_delayed3_2;
        s_unnamed_10 <= s_m_cmd_sequenceNumber_delayed2;
        s_unnamed_12 <= s_unnamed_18;
        s_unnamed_13 <= s_unnamed_102;
        s_aboveWindow_not <= ( not s_aboveWindow);
        s_unnamed_9 <= s_unnamed_60;
        s_unnamed_11 <= s_m_cmd_valid_delayed15;
        s_unnamed_15 <= s_aboveWindow_not_delayed9;
        s_unnamed_17 <= s_m_cmd_opcode_delayed6;
        IF (s_unnamed_17 = C_OPSETSEQNR) THEN
            v_updateState := '1';
        ELSE
            v_updateState := ((bool2stdlogic(s_unnamed_17 = C_OPUPDATE_2) and s_unnamed_14) and (s_aboveWindow or s_inPhysicalWindow));
        END IF;
        s_updateState <= v_updateState;
        s_unnamed_16 <= s_unnamed_101;
        s_unnamed_5 <= s_unnamed_24;
        s_unnamed <= ((bool2stdlogic(s_unnamed_17 = C_OPUPDATE) and s_unnamed_14) and s_aboveWindow);
        s_unnamed_29 <= s_unnamed_69;
        s_unnamed_26 <= s_in_cmd_sequenceNumber_delayed1;
        s_unnamed_28 <= s_unnamed_64;
        s_unnamed_27 <= s_unnamed_95;
        s_m_cmd_valid_delayed2 <= s_m_cmd_valid_delayed2_2;
        s_m_cmd_sequenceNumber_delayed1 <= s_m_cmd_sequenceNumber_delayed1_2;
        s_m_cmd_connectionIndex_delayed2_2 <= s_m_cmd_connectionIndex_delayed2_3;
        s_m_cmd_opcode_delayed2 <= s_m_cmd_opcode_delayed2_2;
        s_unnamed_32 <= s_unnamed_93;
        s_m_cmd_valid_delayed3 <= s_m_cmd_valid_delayed3_2;
        s_m_cmd_opcode_delayed3 <= s_m_cmd_opcode_delayed3_2;
        s_unnamed_34 <= s_unnamed_105;
        s_m_cmd_valid_delayed4 <= s_m_cmd_valid_delayed4_2;
        s_unnamed_30 <= s_unnamed_96;
        s_unnamed_33 <= s_unnamed_94;
        s_m_cmd_opcode_delayed4 <= s_m_cmd_opcode_delayed4_2;
        s_unnamed_35 <= s_unnamed_103;
        s_m_cmd_valid_delayed5 <= s_m_cmd_valid_delayed5_2;
        s_unnamed_31 <= s_unnamed_104;
        s_unnamed_42 <= s_unnamed_106;
        s_m_cmd_opcode_delayed5 <= s_m_cmd_opcode_delayed5_2;
        s_unnamed_37 <= s_unnamed_97;
        s_m_cmd_valid_delayed6 <= s_m_cmd_valid_delayed6_2;
        s_unnamed_38 <= s_unnamed_91;
        s_updateState_delayed1_2 <= s_updateState_delayed1_4;
        s_aboveWindow_not_delayed1 <= s_aboveWindow_not_delayed1_2;
        s_m_cmd_valid_delayed7_2 <= s_m_cmd_valid_delayed7_4;
        s_unnamed_39 <= s_unnamed_100;
        s_unnamed_41 <= s_unnamed_99;
        s_unnamed_36 <= s_unnamed_98;
        s_unnamed_40 <= s_unnamed_88;
        s_updateState_delayed2_2 <= s_updateState_delayed2_3;
        s_aboveWindow_not_delayed2 <= s_aboveWindow_not_delayed2_2;
        s_m_cmd_valid_delayed8_2 <= s_m_cmd_valid_delayed8_3;
        s_unnamed_44 <= s_unnamed_90;
        s_unnamed_48 <= s_unnamed_92;
        s_unnamed_46 <= s_unnamed_89;
        s_unnamed_43 <= s_unnamed_84;
        s_aboveWindow_not_delayed3 <= s_aboveWindow_not_delayed3_2;
        s_m_cmd_valid_delayed9 <= s_m_cmd_valid_delayed9_2;
        s_unnamed_47 <= s_unnamed_86;
        s_unnamed_49 <= s_unnamed_87;
        s_unnamed_50 <= s_unnamed_85;
        s_unnamed_45 <= s_unnamed_81;
        s_aboveWindow_not_delayed4 <= s_aboveWindow_not_delayed4_2;
        s_m_cmd_valid_delayed10_2 <= s_m_cmd_valid_delayed10_4;
        s_unnamed_54 <= s_unnamed_83;
        s_unnamed_52 <= s_unnamed_79;
        s_aboveWindow_not_delayed5 <= s_aboveWindow_not_delayed5_2;
        s_m_cmd_valid_delayed11_2 <= s_m_cmd_valid_delayed11_4;
        s_unnamed_51 <= s_unnamed_82;
        s_unnamed_53 <= s_unnamed_77;
        s_aboveWindow_not_delayed6 <= s_aboveWindow_not_delayed6_2;
        s_m_cmd_valid_delayed12_2 <= s_m_cmd_valid_delayed12_4;
        s_unnamed_56 <= s_unnamed_80;
        s_unnamed_57 <= s_unnamed_61;
        s_aboveWindow_not_delayed7 <= s_aboveWindow_not_delayed7_2;
        s_m_cmd_valid_delayed13_2 <= s_m_cmd_valid_delayed13_4;
        s_unnamed_58 <= s_unnamed_78;
        s_unnamed_55 <= s_in_cmd_windowSize_delayed1;
        s_aboveWindow_not_delayed8 <= s_aboveWindow_not_delayed8_2;
        s_m_cmd_valid_delayed14_2 <= s_m_cmd_valid_delayed14_3;
        s_unnamed_59 <= s_unnamed_62;
        s_unnamed_4 <= s_unnamed_23;
        s_unnamed_68 <= s_updateState_delayed2;
        s_unnamed_67 <= s_m_cmd_valid_delayed8;
        s_unnamed_63 <= s_unnamed_25;
        s_unnamed_66 <= s_m_cmd_valid_delayed14;
        s_m_cmd_connectionIndex_delayed2_4 <= s_m_cmd_connectionIndex_delayed2;
        s_unnamed_71 <= s_unnamed_19;
        s_unnamed_73 <= bool2stdlogic(s_unnamed_17 = C_OPSETSEQNR_2);
        s_unnamed_74 <= s_unnamed_20;
        s_unnamed_76 <= s_unnamed_21;
        s_unnamed_75 <= s_unnamed_22;
        s_m_cmd_valid_delayed7_3 <= s_m_cmd_valid_delayed7;
        s_updateState_delayed1_3 <= s_updateState_delayed1;
        s_m_cmd_valid_delayed10_3 <= s_m_cmd_valid_delayed10;
        s_m_cmd_valid_delayed11_3 <= s_m_cmd_valid_delayed11;
        s_m_cmd_valid_delayed12_3 <= s_m_cmd_valid_delayed12;
        s_m_cmd_valid_delayed13_3 <= s_m_cmd_valid_delayed13;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            s_in_cmd_sequenceNumber_delayed1 <= in_in_cmd_sequenceNumber;
            s_unnamed_72 <= s_unnamed_29;
            s_m_cmd_sequenceNumber_delayed1_2 <= s_m_cmd_sequenceNumber;
            s_unnamed_70 <= s_unnamed_26;
            s_m_cmd_connectionIndex_delayed2_3 <= s_m_cmd_connectionIndex_delayed1;
            s_in_cmd_connectionIndex_delayed1 <= in_in_cmd_connectionIndex;
            s_in_cmd_opcode_delayed1 <= in_in_cmd_opcode;
            s_m_cmd_valid_delayed1_2 <= s_m_cmd_valid;
            s_in_cmd_valid_and_m_inReady_2_delayed1 <= in_in_cmd_valid_and_m_inReady_2;
            s_m_cmd_connectionIndex_delayed1_2 <= s_m_cmd_connectionIndex;
            s_m_cmd_opcode_delayed1_2 <= s_m_cmd_opcode;
            s_m_cmd_valid_delayed2_2 <= s_m_cmd_valid_delayed1;
            s_unnamed_69 <= s_unnamed_28;
            s_m_cmd_opcode_delayed2_2 <= s_m_cmd_opcode_delayed1;
            s_unnamed_97 <= s_unnamed_38;
            s_unnamed_104 <= s_unnamed_30;
            s_m_cmd_valid_delayed12_4 <= s_m_cmd_valid_delayed11_2;
            s_m_cmd_valid_delayed4_2 <= s_m_cmd_valid_delayed3;
            s_m_cmd_valid_delayed5_2 <= s_m_cmd_valid_delayed4;
            s_aboveWindow_not_delayed6_2 <= s_aboveWindow_not_delayed5;
            s_unnamed_89 <= s_unnamed_36;
            s_unnamed_92 <= s_unnamed_41;
            s_unnamed_90 <= s_unnamed_39;
            s_unnamed_85 <= s_unnamed_46;
            s_unnamed_84 <= s_unnamed_45;
            s_unnamed_87 <= s_unnamed_48;
            s_unnamed_86 <= s_unnamed_44;
            s_m_cmd_opcode_delayed3_2 <= s_m_cmd_opcode_delayed2;
            s_m_cmd_valid_delayed9_2 <= s_m_cmd_valid_delayed8_2;
            s_unnamed_77 <= s_unnamed_57;
            s_in_cmd_windowSize_delayed1 <= in_in_cmd_windowSize;
            s_m_cmd_valid_delayed8_3 <= s_m_cmd_valid_delayed7_2;
            s_m_cmd_connectionIndex_delayed3_2 <= s_m_cmd_connectionIndex_delayed2_2;
            s_m_cmd_sequenceNumber_delayed2 <= s_m_cmd_sequenceNumber_delayed1;
            s_unnamed_83 <= s_unnamed_50;
            s_aboveWindow_not_delayed3_2 <= s_aboveWindow_not_delayed2;
            s_unnamed_60 <= s_unnamed_59;
            s_unnamed_78 <= s_unnamed_56;
            s_m_cmd_valid_delayed15 <= s_m_cmd_valid_delayed14_2;
            s_m_cmd_valid_delayed10_4 <= s_m_cmd_valid_delayed9;
            s_m_cmd_valid_delayed13_4 <= s_m_cmd_valid_delayed12_2;
            s_aboveWindow_not_delayed9 <= s_aboveWindow_not_delayed8;
            s_updateState_2 <= s_updateState_delayed2_2;
            s_unnamed_64 <= s_unnamed_27;
            s_unnamed_81 <= s_unnamed_52;
            s_aboveWindow_not_delayed7_2 <= s_aboveWindow_not_delayed6;
            s_m_cmd_valid_delayed3_2 <= s_m_cmd_valid_delayed2;
            s_aboveWindow_not_delayed2_2 <= s_aboveWindow_not_delayed1;
            s_aboveWindow_not_delayed4_2 <= s_aboveWindow_not_delayed3;
            s_updateState_delayed2_3 <= s_updateState_delayed1_2;
            s_unnamed_98 <= s_unnamed_17;
            s_unnamed_99 <= s_unnamed_16;
            s_unnamed_100 <= s_unnamed_13;
            s_unnamed_93 <= s_unnamed_34;
            s_m_cmd_valid_delayed7_4 <= s_m_cmd_valid_delayed6;
            s_unnamed_82 <= s_unnamed_54;
            s_m_cmd_valid_delayed11_4 <= s_m_cmd_valid_delayed10_2;
            s_unnamed_95 <= s_unnamed_32;
            s_m_cmd_opcode_delayed4_2 <= s_m_cmd_opcode_delayed3;
            s_unnamed_103 <= s_unnamed_37;
            s_unnamed_61 <= s_unnamed_55;
            s_unnamed_94 <= s_unnamed_8;
            s_aboveWindow_not_delayed5_2 <= s_aboveWindow_not_delayed4;
            s_aboveWindow_not_delayed1_2 <= s_aboveWindow_not;
            s_updateState_delayed1_4 <= s_updateState;
            s_m_cmd_opcode_delayed6 <= s_m_cmd_opcode_delayed5;
            s_unnamed_101 <= s_unnamed_42;
            s_unnamed_79 <= s_unnamed_53;
            s_unnamed_102 <= s_unnamed_31;
            s_unnamed_96 <= s_unnamed_10;
            s_unnamed_62 <= s_unnamed_58;
            s_m_cmd_valid_delayed6_2 <= s_m_cmd_valid_delayed5;
            s_m_cmd_valid_delayed14_3 <= s_m_cmd_valid_delayed13_2;
            s_unnamed_105 <= s_unnamed_35;
            s_unnamed_91 <= s_unnamed_40;
            s_aboveWindow_not_delayed8_2 <= s_aboveWindow_not_delayed7;
            s_m_cmd_opcode_delayed5_2 <= s_m_cmd_opcode_delayed4;
            s_unnamed_106 <= s_unnamed_33;
            s_unnamed_80 <= s_unnamed_51;
            s_unnamed_88 <= s_unnamed_43;
            s_m_cmd_connectionIndex_delayed2 <= s_m_cmd_connectionIndex_delayed1;
            s_m_cmd_connectionIndex_delayed3 <= s_m_cmd_connectionIndex_delayed2_4;
            s_unnamed_19 <= s_unnamed_8;
            s_unnamed_18 <= s_unnamed_71;
            s_updateState_delayed1 <= s_updateState;
            s_unnamed_20 <= s_unnamed;
            s_unnamed_21 <= s_unnamed_73;
            s_unnamed_22 <= s_unnamed_16;
            s_m_cmd_valid_delayed7 <= s_m_cmd_valid_delayed6;
            s_unnamed_23 <= s_unnamed_74;
            s_unnamed_24 <= s_unnamed_76;
            s_unnamed_25 <= s_unnamed_75;
            s_m_cmd_valid_delayed8 <= s_m_cmd_valid_delayed7_3;
            s_updateState_delayed2 <= s_updateState_delayed1_3;
            s_m_cmd_valid_delayed10 <= s_m_cmd_valid_delayed9;
            s_m_cmd_valid_delayed11 <= s_m_cmd_valid_delayed10_3;
            s_m_cmd_valid_delayed12 <= s_m_cmd_valid_delayed11_3;
            s_m_cmd_valid_delayed13 <= s_m_cmd_valid_delayed12_3;
            s_m_cmd_valid_delayed14 <= s_m_cmd_valid_delayed13_3;
        END IF;
    END PROCESS;

    lastSequenceNumber0 : entity work.lastSequenceNumber(impl) port map (
        clock => clock,
        in_m_cmd_sequenceNumber => s_m_cmd_sequenceNumber,
        in_connectionActive => s_connectionActive,
        in_m_cmd_opcode_delayed1 => s_m_cmd_opcode_delayed1,
        in_m_cmd_connectionIndex_delayed1 => s_m_cmd_connectionIndex_delayed1,
        in_m_cmd_valid_delayed1 => s_m_cmd_valid_delayed1,
        in_unnamed => s_unnamed_7,
        in_unnamed_2 => s_unnamed_8,
        in_unnamed_3 => s_unnamed_10,
        in_unnamed_4 => s_unnamed_12,
        out_lastSequenceNumber => s_lastSequenceNumber,
        out_aboveWindow => s_aboveWindow,
        out_unnamed => s_unnamed_3,
        out_aboveWindow_delayed3 => s_aboveWindow_delayed3,
        out_lastSequenceNumber_delayed3 => s_lastSequenceNumber_delayed3
    );
    windowBounds0 : entity work.windowBounds(impl) port map (
        clock => clock,
        in_m_cmd_windowSize => s_m_cmd_windowSize,
        in_lastSequenceNumber => s_lastSequenceNumber,
        in_aboveWindow_not => s_aboveWindow_not,
        in_unnamed => s_unnamed_9,
        in_unnamed_2 => s_unnamed_11,
        in_unnamed_3 => s_unnamed_13,
        in_unnamed_4 => s_unnamed_15,
        out_inPhysicalWindow => s_inPhysicalWindow,
        out_inVirtualWindow => s_inVirtualWindow,
        out_unnamed => s_unnamed_2
    );
    FirstLookupStage0 : entity work.FirstLookupStage(impl) port map (
        clock => clock,
        in_lastSequenceNumber => s_lastSequenceNumber,
        in_unnamed => s_unnamed,
        in_unnamed_2 => s_unnamed_4,
        in_unnamed_3 => s_unnamed_5,
        in_unnamed_4 => s_unnamed_13,
        in_unnamed_5 => s_unnamed_16,
        in_unnamed_6 => s_unnamed_63,
        in_unnamed_7 => s_unnamed_67,
        in_unnamed_8 => s_unnamed_68,
        out_secondStageValid => s_secondStageValid,
        out_secondStageValid_delayed6 => s_secondStageValid_delayed6,
        out_unnamed => s_unnamed_65
    );
    SecondLookupStage0 : entity work.SecondLookupStage(impl) port map (
        clock => clock,
        bitmap_wr_writedata => bitmap_wr_writedata,
        bitmap_rd_readdata => bitmap_rd_readdata,
        bitmap_wr_address => bitmap_wr_address,
        bitmap_rd_address => bitmap_rd_address,
        bitmap_rd_read => bitmap_rd_read,
        bitmap_wr_write => bitmap_wr_write,
        in_secondStageValid => s_secondStageValid,
        in_aboveWindow_delayed3 => s_aboveWindow_delayed3,
        in_unnamed => s_unnamed_47,
        in_lastSequenceNumber_delayed3 => s_lastSequenceNumber_delayed3,
        in_unnamed_2 => s_unnamed_49,
        in_unnamed_3 => s_unnamed_50,
        in_m_cmd_valid_delayed9 => s_m_cmd_valid_delayed9,
        in_unnamed_4 => s_unnamed_65,
        in_unnamed_5 => s_unnamed_66,
        in_updateState => s_updateState_2,
        out_bitmap => s_bitmap,
        out_bitAddr => s_bitAddr
    );
    IsReplay0 : entity work.IsReplay(impl) port map (
        clock => clock,
        in_bitmap => s_bitmap,
        in_secondStageValid_delayed6 => s_secondStageValid_delayed6,
        in_bitAddr => s_bitAddr,
        out_isReplay => s_isReplay
    );
    result0 : entity work.result(impl) port map (
        in_inVirtualWindow => s_inVirtualWindow,
        in_unnamed => s_unnamed_2,
        in_unnamed_2 => s_unnamed_3,
        in_unnamed_3 => s_unnamed_6,
        in_unnamed_4 => s_unnamed_9,
        in_unnamed_5 => s_unnamed_11,
        in_m_isReplay => s_m_isReplay,
        out_out_valid => out_out_valid,
        out_out_code => out_out_code
    );
END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: FrameAdapter
-- 
------------------------------------------------

ENTITY FrameAdapter IS 
    PORT(
        clock : IN STD_LOGIC;
        in_in_cmd_valid : IN STD_LOGIC;
        in_in_frame_valid : IN STD_LOGIC;
        in_in_frame_startofpacket : IN STD_LOGIC;
        in_in_frame_endofpacket : IN STD_LOGIC;
        in_m_inReady_2 : IN STD_LOGIC;
        in_out_ready : IN STD_LOGIC;
        in_in_frame_channel : IN UNSIGNED(0 downto 0);
        in_in_frame_data : IN UNSIGNED(511 downto 0);
        in_in_frame_error : IN UNSIGNED(0 downto 0);
        in_in_frame_empty : IN UNSIGNED(5 downto 0);
        out_m_inReady : OUT STD_LOGIC;
        out_output_reg_data_FrameStream_valid : OUT STD_LOGIC;
        out_output_reg_data_FrameStream_channel : OUT UNSIGNED(0 downto 0);
        out_output_reg_data_FrameStream_data : OUT UNSIGNED(511 downto 0);
        out_output_reg_data_FrameStream_error : OUT UNSIGNED(0 downto 0);
        out_output_reg_data_FrameStream_empty : OUT UNSIGNED(5 downto 0);
        out_output_reg_data_FrameStream_startofpacket : OUT STD_LOGIC;
        out_output_reg_data_FrameStream_endofpacket : OUT STD_LOGIC;
        out_output_reg_data_Valid_valid : OUT STD_LOGIC;
        bitmap_rd_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_rd_read : OUT STD_LOGIC;
        bitmap_rd_readdata : IN STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_wr_writedata : OUT STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_write : OUT STD_LOGIC;
        in_in_cmd_opcode : IN UNSIGNED(2 downto 0);
        in_in_cmd_connectionIndex : IN UNSIGNED(13 downto 0);
        in_in_cmd_sequenceNumber : IN UNSIGNED(63 downto 0);
        in_in_cmd_windowSize : IN UNSIGNED(23 downto 0);
        out_output_reg_data_Valid_data_2 : OUT UNSIGNED(2 downto 0)
    );
END FrameAdapter;

ARCHITECTURE impl OF FrameAdapter IS 
    SIGNAL s_unnamed : STD_LOGIC; 
    SIGNAL s_out_data_FrameStream_valid_and_empty_not_mux1 : STD_LOGIC; 
    SIGNAL s_unnamed_2 : STD_LOGIC; 
    SIGNAL s_empty_not_mux1 : STD_LOGIC; 
    SIGNAL s_full_not : STD_LOGIC; 
    SIGNAL s_empty_not_mux1_2 : STD_LOGIC; 
    SIGNAL s_empty_not : STD_LOGIC; 
    SIGNAL s_empty_not_2 : STD_LOGIC; 
    SIGNAL s_in_cmd_valid_and_m_inReady_2 : STD_LOGIC; 
    SIGNAL s_dbg_dataIn4 : STD_LOGIC; 
    SIGNAL s_out_ready_mux1 : STD_LOGIC; 
    SIGNAL s_empty_not_3 : STD_LOGIC; 
    SIGNAL s_out_data_FrameStream_startofpacket : STD_LOGIC; 
    SIGNAL s_out_data_FrameStream_valid : STD_LOGIC; 
    SIGNAL s_out_data_FrameStream_empty : UNSIGNED(5 downto 0); 
    SIGNAL s_out_data_Bit : STD_LOGIC; 
    SIGNAL s_out_data_FrameStream_endofpacket : STD_LOGIC; 
    SIGNAL s_out_data_FrameStream_channel : UNSIGNED(0 downto 0); 
    SIGNAL s_out_data_FrameStream_data : UNSIGNED(511 downto 0); 
    SIGNAL s_out_data_FrameStream_error : UNSIGNED(0 downto 0); 
    SIGNAL s_full : STD_LOGIC; 
    SIGNAL s_empty : STD_LOGIC; 
    SIGNAL s_almost_full_0 : STD_LOGIC; 
    SIGNAL s_out_valid : STD_LOGIC; 
    SIGNAL s_out_code : UNSIGNED(2 downto 0); 
    SIGNAL s_in_valid : STD_LOGIC; 
    SIGNAL s_out_ready : STD_LOGIC; 
    SIGNAL s_empty_not_4 : STD_LOGIC; 
    SIGNAL s_out_data : UNSIGNED(2 downto 0); 
    SIGNAL s_out_ready_mux1_2 : STD_LOGIC; 
    SIGNAL s_output_reg_data_Valid_valid : STD_LOGIC := '0'; 
    SIGNAL s_out_ready_mux1_3 : STD_LOGIC; 
    SIGNAL s_output_reg_data_Valid_data : UNSIGNED(2 downto 0); 
    SIGNAL s_output_reg_data_FrameStream_valid : STD_LOGIC := '0'; 
    SIGNAL s_output_reg_data_FrameStream_startofpacket : STD_LOGIC; 
    SIGNAL s_output_reg_data_FrameStream_endofpacket : STD_LOGIC; 
    SIGNAL s_output_reg_data_FrameStream_channel : UNSIGNED(0 downto 0); 
    SIGNAL s_output_reg_data_FrameStream_empty : UNSIGNED(5 downto 0); 
    SIGNAL s_output_reg_data_FrameStream_error : UNSIGNED(0 downto 0); 
    SIGNAL s_output_reg_data_FrameStream_data : UNSIGNED(511 downto 0); 
BEGIN
    AntiReplay0 : entity work.AntiReplay(impl) port map (
        clock => clock,
        bitmap_wr_writedata => bitmap_wr_writedata,
        bitmap_rd_readdata => bitmap_rd_readdata,
        bitmap_wr_address => bitmap_wr_address,
        bitmap_rd_address => bitmap_rd_address,
        bitmap_rd_read => bitmap_rd_read,
        bitmap_wr_write => bitmap_wr_write,
        in_in_cmd_sequenceNumber => in_in_cmd_sequenceNumber,
        in_in_cmd_connectionIndex => in_in_cmd_connectionIndex,
        in_in_cmd_opcode => in_in_cmd_opcode,
        in_in_cmd_valid_and_m_inReady_2 => s_in_cmd_valid_and_m_inReady_2,
        in_in_cmd_windowSize => in_in_cmd_windowSize,
        out_out_valid => s_out_valid,
        out_out_code => s_out_code
    );
    default_comb : PROCESS(all)
        CONSTANT C_OPUPDATE : UNSIGNED(2 downto 0) := "000"; 
        VARIABLE v_output_reg_data_Valid_data : UNSIGNED(2 downto 0); 
    BEGIN
        s_out_ready_mux1 <= s_out_ready_mux1_2;
        s_in_valid <= (( not s_full) and s_out_valid);
        s_empty_not_4 <= ( not s_empty);
        out_m_inReady <= (s_full_not and ( not s_almost_full_0));
        out_output_reg_data_FrameStream_valid <= s_output_reg_data_FrameStream_valid;
        out_output_reg_data_FrameStream_channel <= s_output_reg_data_FrameStream_channel;
        out_output_reg_data_FrameStream_data <= s_output_reg_data_FrameStream_data;
        out_output_reg_data_FrameStream_error <= s_output_reg_data_FrameStream_error;
        out_output_reg_data_FrameStream_empty <= s_output_reg_data_FrameStream_empty;
        out_output_reg_data_FrameStream_startofpacket <= s_output_reg_data_FrameStream_startofpacket;
        out_output_reg_data_FrameStream_endofpacket <= s_output_reg_data_FrameStream_endofpacket;
        out_output_reg_data_Valid_valid <= s_output_reg_data_Valid_valid;
        v_output_reg_data_Valid_data := s_output_reg_data_Valid_data;
        s_out_data_FrameStream_valid_and_empty_not_mux1 <= (s_out_data_FrameStream_valid and s_empty_not_mux1);
        s_unnamed_2 <= (s_empty_not_mux1_2 and s_empty_not_mux1);
        s_empty_not_2 <= s_empty_not_4;
        s_out_ready <= (s_out_ready_mux1_3 and s_empty_not_2);
        s_in_cmd_valid_and_m_inReady_2 <= (in_in_cmd_valid and in_m_inReady_2);
        s_dbg_dataIn4 <= (s_in_cmd_valid_and_m_inReady_2 and bool2stdlogic(in_in_cmd_opcode = C_OPUPDATE));
        s_unnamed <= ((in_in_frame_valid or (s_in_cmd_valid_and_m_inReady_2 and bool2stdlogic(in_in_cmd_opcode = C_OPUPDATE))) and in_m_inReady_2);
        out_output_reg_data_Valid_data_2 <= v_output_reg_data_Valid_data;
    END PROCESS;

    default_reg : PROCESS(clock)
    BEGIN
        IF (rising_edge(clock)) THEN
            IF (in_out_ready = '1') THEN
                s_output_reg_data_FrameStream_channel <= s_out_data_FrameStream_channel;
            END IF;
            IF (in_out_ready = '1') THEN
                s_output_reg_data_FrameStream_data <= s_out_data_FrameStream_data;
            END IF;
            IF (in_out_ready = '1') THEN
                s_output_reg_data_FrameStream_error <= s_out_data_FrameStream_error;
            END IF;
            IF (in_out_ready = '1') THEN
                s_output_reg_data_FrameStream_empty <= s_out_data_FrameStream_empty;
            END IF;
            IF (in_out_ready = '1') THEN
                s_output_reg_data_Valid_valid <= s_unnamed_2;
            END IF;
            IF (in_out_ready = '1') THEN
                s_output_reg_data_FrameStream_endofpacket <= s_out_data_FrameStream_endofpacket;
            END IF;
            IF (in_out_ready = '1') THEN
                s_output_reg_data_Valid_data <= s_out_data;
            END IF;
            IF (in_out_ready = '1') THEN
                s_output_reg_data_FrameStream_startofpacket <= s_out_data_FrameStream_startofpacket;
            END IF;
            IF (in_out_ready = '1') THEN
                s_output_reg_data_FrameStream_valid <= s_out_data_FrameStream_valid_and_empty_not_mux1;
            END IF;
        END IF;
    END PROCESS;

    FrameFifo0 : entity work.FrameFifo(impl) port map (
        clock => clock,
        in_in_frame_valid => in_in_frame_valid,
        in_in_frame_startofpacket => in_in_frame_startofpacket,
        in_in_frame_endofpacket => in_in_frame_endofpacket,
        in_unnamed => s_unnamed,
        in_dbg_dataIn4 => s_dbg_dataIn4,
        in_out_ready_mux1 => s_out_ready_mux1,
        in_in_frame_channel => in_in_frame_channel,
        in_in_frame_data => in_in_frame_data,
        in_in_frame_error => in_in_frame_error,
        in_in_frame_empty => in_in_frame_empty,
        out_full_not => s_full_not,
        out_empty_not => s_empty_not,
        out_empty_not_2 => s_empty_not_3,
        out_out_data_FrameStream_startofpacket => s_out_data_FrameStream_startofpacket,
        out_out_data_FrameStream_valid => s_out_data_FrameStream_valid,
        out_out_data_FrameStream_empty => s_out_data_FrameStream_empty,
        out_out_data_Bit => s_out_data_Bit,
        out_out_data_FrameStream_endofpacket => s_out_data_FrameStream_endofpacket,
        out_out_data_FrameStream_channel => s_out_data_FrameStream_channel,
        out_out_data_FrameStream_data => s_out_data_FrameStream_data,
        out_out_data_FrameStream_error => s_out_data_FrameStream_error
    );
    scl_fifo0 : entity work.scl_fifo_2(impl) port map (
        clock => clock,
        in_out_code => s_out_code,
        in_in_valid => s_in_valid,
        in_out_ready => s_out_ready,
        out_full => s_full,
        out_empty => s_empty,
        out_almost_full_0 => s_almost_full_0,
        out_out_data => s_out_data
    );
    merge0 : entity work.merge(impl) port map (
        in_out_ready => in_out_ready,
        in_empty_not => s_empty_not,
        in_empty_not_2 => s_empty_not_2,
        in_empty_not_3 => s_empty_not_3,
        in_out_data_Bit => s_out_data_Bit,
        in_empty_not_4 => s_empty_not_4,
        out_empty_not_mux1 => s_empty_not_mux1,
        out_empty_not_mux1_2 => s_empty_not_mux1_2,
        out_out_ready_mux1 => s_out_ready_mux1_2,
        out_out_ready_mux1_2 => s_out_ready_mux1_3
    );
END impl;

--------------------------------------------------------------------
-- Synogate Anti Replay Core, an IP core implementing an algorithm
-- designed to detect packet duplication based on packet sequence numbers.
-- Copyright (C) 2021 Synogate UG (haftungsbeschraenkt), www.synogate.com 
-- 
-- This program is commercial software: you must not copy, modify or
-- distribute it without explicit written license by Synogate to do so. 
-- 
-- This program is distributed WITHOUT ANY WARRANTY; without even the
-- implied warranty of MERCHANTABILITY.
--
--
-- This file was auto generated from Gatery. Any
-- changes made directly can not be brought back into the source
-- material and will be lost upon regeneration.
--------------------------------------------------------------------
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.all;

LIBRARY work;
USE work.GateryHelperPackage.all;

LIBRARY work;
USE work.interface_package.all;

------------------------------------------------
--  Entity: AntiReplayCore
-- This is the top module of the Synogate Anti-Replay core in the Packet Interface configuration.
------------------------------------------------

ENTITY AntiReplayCore IS 
    PORT(
        clock : IN STD_LOGIC;
        in_cmd_valid : IN STD_LOGIC;
        in_cmd_opcode : IN STD_LOGIC_VECTOR(2 downto 0);
        in_cmd_connectionIndex : IN STD_LOGIC_VECTOR(13 downto 0);
        in_cmd_sequenceNumber : IN STD_LOGIC_VECTOR(63 downto 0);
        in_cmd_windowSize : IN STD_LOGIC_VECTOR(23 downto 0);
        in_ready : OUT STD_LOGIC;
        in_frame_valid : IN STD_LOGIC;
        in_frame_data : IN STD_LOGIC_VECTOR(511 downto 0);
        in_frame_startofpacket : IN STD_LOGIC;
        in_frame_endofpacket : IN STD_LOGIC;
        in_frame_empty : IN STD_LOGIC_VECTOR(5 downto 0);
        in_frame_error : IN STD_LOGIC_VECTOR(0 downto 0);
        in_frame_channel : IN STD_LOGIC_VECTOR(0 downto 0);
        in_frame_userdata : IN STD_LOGIC_VECTOR(-1 downto 0);
        out_ready : IN STD_LOGIC;
        out_frame_valid : OUT STD_LOGIC;
        out_frame_data : OUT STD_LOGIC_VECTOR(511 downto 0);
        out_frame_startofpacket : OUT STD_LOGIC;
        out_frame_endofpacket : OUT STD_LOGIC;
        out_frame_empty : OUT STD_LOGIC_VECTOR(5 downto 0);
        out_frame_error : OUT STD_LOGIC_VECTOR(0 downto 0);
        out_frame_channel : OUT STD_LOGIC_VECTOR(0 downto 0);
        out_frame_userdata : OUT STD_LOGIC_VECTOR(-1 downto 0);
        out_result_valid : OUT STD_LOGIC;
        out_result_data : OUT STD_LOGIC_VECTOR(2 downto 0);
        bitmap_rd_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_rd_read : OUT STD_LOGIC;
        bitmap_rd_readdata : IN STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_address : OUT STD_LOGIC_VECTOR(19 downto 0);
        bitmap_wr_writedata : OUT STD_LOGIC_VECTOR(255 downto 0);
        bitmap_wr_write : OUT STD_LOGIC
    );
END AntiReplayCore;

ARCHITECTURE impl OF AntiReplayCore IS 
    SIGNAL s_in_frame_valid : STD_LOGIC; 
    SIGNAL s_in_frame_startofpacket : STD_LOGIC; 
    SIGNAL s_in_frame_endofpacket : STD_LOGIC; 
    SIGNAL s_m_inReady_2 : STD_LOGIC; 
    SIGNAL s_in_cmd_valid : STD_LOGIC; 
    SIGNAL s_out_ready : STD_LOGIC; 
    SIGNAL s_in_cmd_sequenceNumber : UNSIGNED(63 downto 0); 
    SIGNAL s_in_cmd_connectionIndex : UNSIGNED(13 downto 0); 
    SIGNAL s_in_cmd_opcode : UNSIGNED(2 downto 0); 
    SIGNAL s_in_cmd_windowSize : UNSIGNED(23 downto 0); 
    SIGNAL s_in_frame_channel : UNSIGNED(0 downto 0); 
    SIGNAL s_in_frame_data : UNSIGNED(511 downto 0); 
    SIGNAL s_in_frame_error : UNSIGNED(0 downto 0); 
    SIGNAL s_in_frame_empty : UNSIGNED(5 downto 0); 
    SIGNAL s_output_reg_data_Valid_data_2 : UNSIGNED(2 downto 0); 
    SIGNAL s_m_inReady : STD_LOGIC; 
    SIGNAL s_output_reg_data_FrameStream_valid : STD_LOGIC; 
    SIGNAL s_output_reg_data_FrameStream_startofpacket : STD_LOGIC; 
    SIGNAL s_output_reg_data_FrameStream_data : UNSIGNED(511 downto 0); 
    SIGNAL s_output_reg_data_FrameStream_channel : UNSIGNED(0 downto 0); 
    SIGNAL s_output_reg_data_FrameStream_error : UNSIGNED(0 downto 0); 
    SIGNAL s_output_reg_data_FrameStream_endofpacket : STD_LOGIC; 
    SIGNAL s_output_reg_data_Valid_valid : STD_LOGIC; 
    SIGNAL s_output_reg_data_FrameStream_empty : UNSIGNED(5 downto 0); 
BEGIN
    FrameAdapter0 : entity work.FrameAdapter(impl) port map (
        clock => clock,
        bitmap_wr_writedata => bitmap_wr_writedata,
        bitmap_rd_readdata => bitmap_rd_readdata,
        bitmap_wr_address => bitmap_wr_address,
        bitmap_rd_address => bitmap_rd_address,
        bitmap_rd_read => bitmap_rd_read,
        bitmap_wr_write => bitmap_wr_write,
        in_in_frame_valid => s_in_frame_valid,
        in_in_frame_startofpacket => s_in_frame_startofpacket,
        in_in_frame_endofpacket => s_in_frame_endofpacket,
        in_m_inReady_2 => s_m_inReady_2,
        in_in_cmd_valid => s_in_cmd_valid,
        in_out_ready => s_out_ready,
        in_in_cmd_sequenceNumber => s_in_cmd_sequenceNumber,
        in_in_cmd_connectionIndex => s_in_cmd_connectionIndex,
        in_in_cmd_opcode => s_in_cmd_opcode,
        in_in_cmd_windowSize => s_in_cmd_windowSize,
        in_in_frame_channel => s_in_frame_channel,
        in_in_frame_data => s_in_frame_data,
        in_in_frame_error => s_in_frame_error,
        in_in_frame_empty => s_in_frame_empty,
        out_output_reg_data_Valid_data_2 => s_output_reg_data_Valid_data_2,
        out_m_inReady => s_m_inReady,
        out_output_reg_data_FrameStream_valid => s_output_reg_data_FrameStream_valid,
        out_output_reg_data_FrameStream_startofpacket => s_output_reg_data_FrameStream_startofpacket,
        out_output_reg_data_FrameStream_data => s_output_reg_data_FrameStream_data,
        out_output_reg_data_FrameStream_channel => s_output_reg_data_FrameStream_channel,
        out_output_reg_data_FrameStream_error => s_output_reg_data_FrameStream_error,
        out_output_reg_data_FrameStream_endofpacket => s_output_reg_data_FrameStream_endofpacket,
        out_output_reg_data_Valid_valid => s_output_reg_data_Valid_valid,
        out_output_reg_data_FrameStream_empty => s_output_reg_data_FrameStream_empty
    );
    default_comb : PROCESS(all)
    BEGIN
        s_in_cmd_valid <= in_cmd_valid;
        s_in_frame_valid <= in_frame_valid;
        s_in_frame_startofpacket <= in_frame_startofpacket;
        s_in_frame_endofpacket <= in_frame_endofpacket;
        s_m_inReady_2 <= s_m_inReady;
        s_out_ready <= out_ready;
        s_in_frame_channel <= UNSIGNED(in_frame_channel);
        s_in_frame_data <= UNSIGNED(in_frame_data);
        s_in_frame_error <= UNSIGNED(in_frame_error);
        s_in_frame_empty <= UNSIGNED(in_frame_empty);
        in_ready <= s_m_inReady;
        out_frame_valid <= s_output_reg_data_FrameStream_valid;
        out_frame_data <= STD_LOGIC_VECTOR(s_output_reg_data_FrameStream_data);
        out_frame_startofpacket <= s_output_reg_data_FrameStream_startofpacket;
        out_frame_endofpacket <= s_output_reg_data_FrameStream_endofpacket;
        out_frame_empty <= STD_LOGIC_VECTOR(s_output_reg_data_FrameStream_empty);
        out_frame_error <= STD_LOGIC_VECTOR(s_output_reg_data_FrameStream_error);
        out_frame_channel <= STD_LOGIC_VECTOR(s_output_reg_data_FrameStream_channel);
        out_result_valid <= s_output_reg_data_Valid_valid;
        out_result_data <= STD_LOGIC_VECTOR(s_output_reg_data_Valid_data_2);
        s_in_cmd_opcode <= UNSIGNED(in_cmd_opcode);
        s_in_cmd_connectionIndex <= UNSIGNED(in_cmd_connectionIndex);
        s_in_cmd_sequenceNumber <= UNSIGNED(in_cmd_sequenceNumber);
        s_in_cmd_windowSize <= UNSIGNED(in_cmd_windowSize);
    END PROCESS;

END impl;
